@@ -1,53 +1,28 @@
-//  $Id: file_.cpp,v 1.15 2005/01/20 17:27:06 keb Exp $
+//  $Id: file_.cpp,v 1.19 2008/07/08 15:28:14 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    20.01.05   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -56,7 +31,7 @@
 //  **** Classes :  CFile  - file I/O Support.
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2005
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -113,7 +88,7 @@ pstr P;
   return FilePath;
 }
 
-pstr GetFName ( const pstr FilePath, int syskey )  {
+cpstr GetFName ( cpstr FilePath, int syskey )  {
 pstr P;
   if (syskey==syskey_unix)     
     P = strrchr(FilePath,'/');
@@ -127,14 +102,14 @@ pstr P;
      else  return P + 1;
 }
 
-pstr GetFExt ( const pstr FilePath )  {
+cpstr GetFExt ( cpstr FilePath )  {
 pstr P;
   P = strchr ( GetFName(FilePath),'.');
   if (!P) return &(FilePath[strlen(FilePath)]);
      else return P;
 }
 
-pstr ChangeExt ( pstr FilePath, const pstr newExt, int syskey )  {
+pstr ChangeExt ( pstr FilePath, cpstr newExt, int syskey )  {
 int i;
   i = strlen(FilePath)-1;
   if (syskey==syskey_unix)
@@ -154,7 +129,7 @@ int i;
   return FilePath;
 }
 
-Boolean FileExists ( pstr FileName, PCFile f )  {
+Boolean FileExists ( cpstr FileName, PCFile f )  {
 PCFile  g;
 Boolean B;
   if (FileName)  {
@@ -209,7 +184,7 @@ void  CFile::FreeBuffer ()  {
   }
 }
 
-void  CFile::assign ( const pstr FileName, Boolean Text, Boolean UniB,
+void  CFile::assign ( cpstr FileName, Boolean Text, Boolean UniB,
                       byte gzMode )  {
 #ifndef _MVS
 pstr p;
@@ -571,11 +546,11 @@ Boolean  CFile::exists()  {
 
 }
 
-Boolean  CFile::parse ( const pstr FileName )  {
+Boolean  CFile::parse ( cpstr FileName )  {
   return True;
 }
 
-Boolean  CFile::rename ( const pstr NewFileName )  {
+Boolean  CFile::rename ( cpstr NewFileName )  {
   if (!FName)  return False;
   shut();
   if (!StdIO)
@@ -762,7 +737,7 @@ word  i,j;
 }
 
 
-Boolean  CFile::WriteLine ( const pstr Line )  {
+Boolean  CFile::WriteLine ( cpstr Line )  {
   if ((!memIO) && TextMode)  {
     if (hFile==NULL)  return False;
     fputs ( Line,hFile );
@@ -774,7 +749,7 @@ Boolean  CFile::WriteLine ( const pstr Line )  {
   }
 }
 
-Boolean  CFile::Write ( pstr const Line )  {
+Boolean  CFile::Write ( cpstr Line )  {
   if ((!memIO) && TextMode)  {
     if (hFile==NULL)  return False;
     return (fputs(Line,hFile)>=0);
@@ -818,7 +793,7 @@ Boolean  CFile::WriteDataLine ( realtype X, realtype Y, int length )  {
   return LF();
 }
 
-Boolean  CFile::WriteParameter ( const pstr S, realtype X,
+Boolean  CFile::WriteParameter ( cpstr S, realtype X,
                                  int ParColumn, int length )  {
 int  l=strlen(S);
   if ((!memIO) && TextMode)  {
@@ -838,7 +813,7 @@ int  l=strlen(S);
   return LF();
 }
 
-Boolean  CFile::WriteParameters ( const pstr S, int n_X, rvector X,
+Boolean  CFile::WriteParameters ( cpstr S, int n_X, rvector X,
                                   int ParColumn, int length )  {
 int  i;
 int  l=strlen(S);
@@ -913,7 +888,7 @@ realtype  V;
   }
 }
 
-Boolean  CFile::CreateWrite ( const pstr Line )  {
+Boolean  CFile::CreateWrite ( cpstr Line )  {
 wordUniBin wUB;
 word       i;
   if (UniBin)  {
@@ -941,6 +916,8 @@ word       i;
   }
 }
 
+#define _max_dyn_string_len  1073741824
+
 word  CFile::CreateRead ( pstr & Line )  {
 wordUniBin wUB;
 word       i;
@@ -954,14 +931,14 @@ word       i;
     UniBin2word ( wUB,i );
   } else
     ReadFile ( &i,sizeof(i) );
-  if (i)  {
+  if ((i>0) && (i<_max_dyn_string_len))  {
     Line = new char[i];
     ReadFile ( Line,i );
   }
   return i;
 }
 
-Boolean  CFile::WriteTerLine ( const pstr Line, Boolean longLine )  {
+Boolean  CFile::WriteTerLine ( cpstr Line, Boolean longLine )  {
 wordUniBin wUB;
 word       ll;
 byte       sl;
@@ -1019,7 +996,7 @@ word  Cnt;
     return  0;
 }
 
-Boolean  CFile::WriteFile ( void * Buffer, word Count )  {
+Boolean  CFile::WriteFile ( const void * Buffer, word Count )  {
 pstr IOB;
 word Cnt;
 long Pos;
@@ -1460,7 +1437,8 @@ byte       t;
   return IOSuccess;
 }
 
-Boolean  CFile::CreateReadVector ( rvector & V, int & len, int Shift )  {
+Boolean  CFile::CreateReadVector ( rvector & V, int & len,
+                                   int Shift )  {
 intUniBin  iUB;
 realUniBin rUB;
 int        i;
@@ -1496,7 +1474,8 @@ int len;
   return CreateReadVector ( V,len,Shift );
 }
 
-Boolean  CFile::CreateReadVector ( ivector & iV, int & len, int Shift )  {
+Boolean  CFile::CreateReadVector ( ivector & iV, int & len,
+                                   int Shift )  {
 intUniBin  iUB;
 int        i,iB;
   FreeVectorMemory ( iV,Shift );
@@ -1530,7 +1509,8 @@ int len;
   return CreateReadVector ( iV,len,Shift );
 }
 
-Boolean  CFile::CreateReadVector ( lvector & lV, int & len, int Shift )  {
+Boolean  CFile::CreateReadVector ( lvector & lV, int & len,
+                                   int Shift )  {
 intUniBin  iUB;
 longUniBin lUB;
 int        i;
@@ -1567,7 +1547,8 @@ int len;
 }
 
 
-Boolean  CFile::CreateReadVector ( bvector & B, int & len, int Shift )  {
+Boolean  CFile::CreateReadVector ( bvector & B, int & len,
+                                   int Shift )  {
 intUniBin  iUB;
 int        i;
 byte       t;
@@ -1625,7 +1606,8 @@ int        i,j;
   return IOSuccess;
 }
 
-Boolean  CFile::CreateReadMatrix ( rmatrix & A, int ShiftN, int ShiftM )  {
+Boolean  CFile::CreateReadMatrix ( rmatrix & A, int ShiftN,
+                                   int ShiftM )  {
 int N,M;
   return CreateReadMatrix ( A,N,M,ShiftN,ShiftM );
 }
@@ -1720,7 +1702,8 @@ int  CFile::ReadColumns ( int maxlen, rvector X, rvector Y,
 
 
 int  CFile::CreateReadColumns ( rvector & X, rvector & Y, rvector & Z,
-                                int xCol, int yCol, int zCol, int Shift )  {
+                                int xCol, int yCol, int zCol,
+                                int Shift )  {
 //   ReadColumns reads data stored by WriteColumns. X, Y, and Z
 // must be set to NULL prior to call. They will be allocated
 // within the procedure.
@@ -1755,7 +1738,8 @@ Boolean  Ok;
         Ok = Ok && GetVectorMemory(Z,DataLen,Shift);
       if (Ok)  {
         reset();
-        _ReadColumns ( DataLen,S,sizeof(S),X,Y,Z,xCol,yCol,zCol,Shift );
+        _ReadColumns ( DataLen,S,sizeof(S),X,Y,Z,xCol,yCol,
+                       zCol,Shift );
       } else  ErrCode = FileError_NoMemory;
     } else  ErrCode = FileError_NoDataFound;
   } else  ErrCode = FileError_NoDataFound;
@@ -1769,7 +1753,8 @@ int  CFile::CreateReadColumns ( rvector & X, rvector & Y,
 
 void  CFile::_ReadColumns ( int & DLen, pstr S, int SLen,
                             rvector X, rvector Y, rvector Z,
-                            int xCol, int yCol, int zCol, int Shift )  {
+                            int xCol, int yCol, int zCol,
+                            int Shift )  {
 int      i,is,j,k,m,n,cmax;
 char     SV[256];
 realtype Res;
@@ -1823,7 +1808,7 @@ int j;
       S[j] = ' ';
 }
 
-void  PickOutNumber ( const pstr S, pstr SV, int SLen, int & j )  {
+void  PickOutNumber ( cpstr S, pstr SV, int SLen, int & j )  {
 int l;
   l = 0;
   while ((j<SLen) && (S[j]==' '))  j++;
@@ -1833,7 +1818,7 @@ int l;
   SV[l] = char(0);
 }
 
-realtype  CFile::GetNumber ( const pstr S )  {
+realtype  CFile::GetNumber ( cpstr S )  {
 char *   endptr;
 realtype V;
   V = strtod ( S,&endptr );
@@ -1842,16 +1827,16 @@ realtype V;
   return V;
 }
 
-pstr FileError ( int ErrCode )  {
+cpstr FileError ( int ErrCode )  {
   switch (ErrCode)  {
-    case 0                          : return pstr("Ok");
-    case FileError_NoMemory         : return pstr("Insufficient memory");
-    case FileError_NoDataFound      : return pstr("No data found");
-    case FileError_NoColumn         : return pstr("No column structure");
-    case FileError_BadData          : return pstr("Incorrect data format");
+    case 0                      : return "Ok";
+    case FileError_NoMemory     : return "Insufficient memory";
+    case FileError_NoDataFound  : return "No data found";
+    case FileError_NoColumn     : return "No column structure";
+    case FileError_BadData      : return "Incorrect data format";
     case FileError_WrongMemoryAllocation
-                                    : return pstr("Wrong Memory Allocation");
-    default : return pstr("Unknown I/O error");
+                                : return "Wrong Memory Allocation";
+    default : return "Unknown I/O error";
   }
 }
 
