@@ -1,53 +1,28 @@
-//  $Id: mattype_.h,v 1.8 2005/05/09 10:07:56 keb Exp $
+//  $Id: mattype_.h,v 1.11 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    30.05.03   <--  Date of Last Modification.
+//    28.07.06   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -62,7 +37,7 @@
 //               strcpy_n0  ( like strcpy_n and adds terminating 0  )
 //               PadSpaces  ( pads a string with spaces             )
 //
-//  E. Krissinel 2000-2003
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -127,14 +102,15 @@
 #define   strlen      (int)strlen
 #endif
 
-typedef   char     *        pstr;
-typedef   unsigned int      word;
-typedef   unsigned char     byte;
-typedef   signed   char     short_int;
-typedef   byte              Boolean;
-typedef   unsigned int      word2;
-typedef   byte *            byteptr;
-typedef   unsigned long     lword;
+typedef   char     *         pstr;
+typedef   const char *       cpstr;
+typedef   unsigned int       word;
+typedef   unsigned char      byte;
+typedef   signed   char      short_int;
+typedef   byte               Boolean;
+typedef   unsigned int       word2;
+typedef   byte *             byteptr;
+typedef   unsigned long      lword;
 
 typedef   byte intUniBin  [4];
 typedef   byte shortUniBin[2];
@@ -326,20 +302,27 @@ extern Boolean GetMatrix3Memory
 //         the same as those at allocation !
 //        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-extern  void FreeMatrix3Memory ( rmatrix3  & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( imatrix3  & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( wmatrix3  & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( bmatrix3  & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( lmatrix3  & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( lwmatrix3 & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
-extern  void FreeMatrix3Memory ( psmatrix3 & A,  word N, word M,
-                             word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( rmatrix3  & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( imatrix3  & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( wmatrix3  & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( bmatrix3  & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( lmatrix3  & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( lwmatrix3 & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
+extern void FreeMatrix3Memory
+               ( psmatrix3 & A, word N, word M,
+                 word ShiftN=1, word ShiftM=1, word ShiftK=1 );
 
 // -------------------------------------------------------------
 
@@ -356,16 +339,16 @@ extern  realtype  Pow   ( realtype X, int      y );
 extern  realtype  Pow1  ( realtype X, realtype Y );
 extern  realtype  Exp   ( realtype X );   // use to avoid catastrophies
 extern  Boolean   Odd   ( int      i );
-extern  long    HexValL ( const pstr S );
-extern  long    OctValL ( const pstr S );
-extern  long    BinValL ( const pstr S );
+extern  long    HexValL ( cpstr S );
+extern  long    OctValL ( cpstr S );
+extern  long    BinValL ( cpstr S );
 extern  pstr    BinValS ( long L, pstr S  );  // S[sizeof(long)+1] at least
 
-extern  pstr ParamStr ( pstr D, const pstr S, realtype V, int M=5,
-                        const pstr S1=(pstr)"" );
-extern  pstr ParamStr ( pstr D, const pstr S, realtype V, int M,
-                        const pstr S1, realtype V2, int M2=5,
-                        const pstr S2=(pstr)"" );
+extern  pstr ParamStr ( pstr D, cpstr S, realtype V, int M=5,
+                        cpstr S1=(pstr)"" );
+extern  pstr ParamStr ( pstr D, cpstr S, realtype V, int M,
+                        cpstr S1, realtype V2, int M2=5,
+                        cpstr S2=(pstr)"" );
 
 
 
@@ -375,47 +358,47 @@ extern  pstr ParamStr ( pstr D, const pstr S, realtype V, int M,
 // Source into it. If Dest is not NULL prior calling the function,
 // it is attempted to deallocate first.
 
-extern pstr CreateCopy     ( pstr & Dest, const pstr Source );
-extern pstr CreateCopy_n   ( pstr & Dest, const pstr Source, int n );
+extern pstr CreateCopy     ( pstr & Dest, cpstr Source );
+extern pstr CreateCopy_n   ( pstr & Dest, cpstr Source, int n );
 
-extern pstr CreateConcat   ( pstr & Dest, const pstr Source );
-extern pstr CreateConcat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2 );
-extern pstr CreateConcat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3 );
-extern pstr CreateConcat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3,
-                                          const pstr Source4 );
-extern pstr CreateConcat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3,
-                                          const pstr Source4,
-                                          const pstr Source5 );
+extern pstr CreateConcat   ( pstr & Dest, cpstr Source );
+extern pstr CreateConcat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2 );
+extern pstr CreateConcat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3 );
+extern pstr CreateConcat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3,
+                                          cpstr Source4 );
+extern pstr CreateConcat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3,
+                                          cpstr Source4,
+                                          cpstr Source5 );
 
-extern pstr CreateCopCat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3,
-                                          const pstr Source4,
-                                          const pstr Source5 );
-extern pstr CreateCopCat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3,
-                                          const pstr Source4 );
-extern pstr CreateCopCat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2,
-                                          const pstr Source3 );
-extern pstr CreateCopCat   ( pstr & Dest, const pstr Source1,
-                                          const pstr Source2 );
+extern pstr CreateCopCat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3,
+                                          cpstr Source4,
+                                          cpstr Source5 );
+extern pstr CreateCopCat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3,
+                                          cpstr Source4 );
+extern pstr CreateCopCat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2,
+                                          cpstr Source3 );
+extern pstr CreateCopCat   ( pstr & Dest, cpstr Source1,
+                                          cpstr Source2 );
 
-extern pstr LastOccurence  ( const pstr S     , char c      );
-extern pstr FirstOccurence ( const pstr S     , char c      );
-extern int  indexOf        ( const pstr S     , char c      );
-extern pstr FirstOccurence ( const pstr S, int Slen,
-                             const pstr Q, int Qlen );
-extern int  indexOf        ( const pstr S, int Slen,
-                             const pstr Q, int Qlen );
+extern pstr LastOccurence  ( cpstr S     , char c      );
+extern pstr FirstOccurence ( cpstr S     , char c      );
+extern int  indexOf        ( cpstr S     , char c      );
+extern pstr FirstOccurence ( cpstr S, int Slen,
+                             cpstr Q, int Qlen );
+extern int  indexOf        ( cpstr S, int Slen,
+                             cpstr Q, int Qlen );
 
 extern pstr LowerCase ( pstr s );
 extern pstr UpperCase ( pstr s );
@@ -423,7 +406,7 @@ extern pstr UpperCase ( pstr s );
 //   GetString(..) copies first M characters of string S into string
 // L, appending the terminating null. If S contains less then M
 // characters, L will be padded with spaces.
-extern void GetString ( pstr L, const pstr S, int M );
+extern void GetString ( pstr L, cpstr S, int M );
 
 //   GetStrTer(..) copies at least n (or LMax if LMax<n) first symbols
 // of string S into string L, then continues copying until first space
@@ -434,82 +417,98 @@ extern void GetString ( pstr L, const pstr S, int M );
 //   LMax and SMax are the buffer lengths of L and S,
 // respectively. Even if no space is found, the last character
 // in L will be the terminating null.
-extern void GetStrTer ( pstr L, const pstr S, int n, int LMax, int SMax );
+extern void GetStrTer ( pstr L, cpstr S, int n, int LMax,
+                        int SMax );
+
+
+// Version of GetStrTer(..) allowing for spaces in the string.
+//
+//   Copies at least n (or LMax if LMax<n) first symbols of
+// string S into string L, then continues copying until first
+// terminating null is found. If the terminating null
+// is met among the first n characters or if SMax<n, the string
+// L will be padded with spaces till the length of minimum of
+// n and LMax and then terminated with the null. 
+//   SMax are buffer lengths of L and S, respectively. The last
+// character in L will be the terminating null.
+extern void GetStrTerWin32File ( pstr L, cpstr S, int n,
+                                 int LMax, int SMax );
+
 
 //   strcpy_n(..) copies at most n symbols from string s to d,
 // but no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS NEITHER appended OR copied
 // to d.
-extern void strcpy_n  ( pstr d, const pstr s, int n );
+extern void strcpy_n  ( pstr d, cpstr s, int n );
 
 //   strcpy_n1(..) copies at most n last symbols from string s
 // to d, but no more than strlen(s) (s must contain a terminating
 // null). The string in d is aligned to the right and added with
 // spaces at the left, if necessary. The terminating null
 // IS NEITHER appended OR copied to d.
-extern void strcpy_n1 ( pstr d, const pstr s, int n );
+extern void strcpy_n1 ( pstr d, cpstr s, int n );
 
 //   Copies at most n symbols from string s to d, but no
 // more than strlen(s) (s must contain a terminating null).
 // The string in d is aligned to the right and added with
 // spaces at the left, if necessary. The terminating null
 // IS NEITHER appended NOR copied to d.
-extern void strcpy_nr ( pstr d, const pstr s, int n );
+extern void strcpy_nr ( pstr d, cpstr s, int n );
 
 //   strcpy_ns(..) copies at most n symbols from string s to d,
 // but no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS NEITHER appended NOR copied
 // to d; rather, d is padded with spaces up to the length of n
 // if strlen(s)<n.
-extern void strcpy_ns ( pstr d, const pstr s, int n );
+extern void strcpy_ns ( pstr d, cpstr s, int n );
 
 //   strcpy_cs(..) copies string s to string d cutting all
 // spaces at at the end. Thus, " abcde   " will be copied
 // like " abcde" (terminating null appended).
 //   The function returns d.
-extern pstr strcpy_cs ( pstr d, const pstr s );
+extern pstr strcpy_cs ( pstr d, cpstr s );
 
 //   strcpy_ncs(..) copies at most n characters from string s
 // to string d cutting all spaces at at the end. Thus, " abcde   "
 // will be copied like " abc" at n=4 and like " abcde" at n>5
 // (terminating null appended).
 //   The function returns d.
-extern pstr strcpy_ncs ( pstr d, const pstr s, int n );
+extern pstr strcpy_ncs ( pstr d, cpstr s, int n );
 
 //   strcpy_css(..) copies string s to string d cutting all
 // spaces at the begining and at the end. Thus, " ab c de  "
 // will be copied like "ab c de" (terminating null appended).
 //   The function returns d.
-extern pstr strcpy_css ( pstr d, const pstr s );
+extern pstr strcpy_css ( pstr d, cpstr s );
 
 //   strcpy_ncss(..) copies at most n characters from string s
 // to string d cutting all spaces at the begining and at the end.
 // Thus, " ab c de  " will be copied like "ab" at n=3 (terminating
 // null appended).
 //   The function returns d.
-extern pstr strcpy_ncss ( pstr d, const pstr s, int n );
+extern pstr strcpy_ncss ( pstr d, cpstr s, int n );
 
 //   strcpy_n0(..) copies at most n symbols from string s to d,
 // but no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS appended to d.
 //   The function returns d.
-extern pstr strcpy_n0 ( pstr d, const pstr s, int n );
+extern pstr strcpy_n0 ( pstr d, cpstr s, int n );
 
 //   strlen_des returns the length of a string as if all extra
 // spaces from the latter have been deleted. Extra spaces
 // include all leading and tracing spaces and any sequential
 // spaces when more than one. The string does not change.
-extern int strlen_des ( const pstr s );
+extern int strlen_des ( cpstr s );
 
 //   strcpy_des copies string s into string d removing all extra
 // spaces from the latter. Extra spaces include all leading and
 // tracing spaces and any sequential spaces when more than one.
-extern pstr strcpy_des ( pstr d, const pstr s );
+extern pstr strcpy_des ( pstr d, cpstr s );
 
 //   strcpy_des appends string s to string d removing all extra
 // spaces from the latter. Extra spaces include all leading and
 // tracing spaces and any sequential spaces when more than one.
-extern pstr strcat_des ( pstr d, const pstr s );
+extern pstr strcat_des ( pstr d, cpstr s );
 
 //  PadSpaces(..) pads string S with spaces making its length
 // equal to len. The terminating zero is added, so that S should
@@ -560,14 +559,14 @@ extern void mem_write ( pstr     L, int len, pstr S, int & l );
 extern void mem_write ( pstr     L, pstr S, int & l );
 extern void mem_write ( Boolean  B, pstr S, int & l );
 
-extern void mem_read  ( int      & I, const pstr S, int & l );
-extern void mem_read  ( short    & I, const pstr S, int & l );
-extern void mem_read  ( long     & I, const pstr S, int & l );
-extern void mem_read  ( word     & W, const pstr S, int & l );
-extern void mem_read  ( realtype & R, const pstr S, int & l );
-extern void mem_read  ( pstr     L, int len, const pstr S, int & l );
-extern void mem_read  ( pstr     & L, const pstr S, int & l );
-extern void mem_read  ( Boolean  & B, const pstr S, int & l );
+extern void mem_read  ( int      & I, cpstr S, int & l );
+extern void mem_read  ( short    & I, cpstr S, int & l );
+extern void mem_read  ( long     & I, cpstr S, int & l );
+extern void mem_read  ( word     & W, cpstr S, int & l );
+extern void mem_read  ( realtype & R, cpstr S, int & l );
+extern void mem_read  ( pstr     L, int len, cpstr S, int & l );
+extern void mem_read  ( pstr     & L, cpstr S, int & l );
+extern void mem_read  ( Boolean  & B, cpstr S, int & l );
 
 #endif
 
