@@ -1,49 +1,24 @@
-//  $Id: mmdb_atom.cpp,v 1.18 2005/11/17 18:07:17 keb Exp $
+//  $Id: mmdb_atom.cpp,v 1.25 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CAtom    ( atom class    )
 //       ~~~~~~~~~  CResidue ( residue class )
 //
-//  Copyright (C) E. Krissinel 2000-2004
+//  Copyright (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -95,6 +70,10 @@
 #include "mmdb_cifdefs.h"
 #endif
 
+#ifndef IOTBX_PDB_HYBRID_36_C_H
+#include "hybrid_36.h"
+#endif
+
 
 //  ================================================================
 
@@ -283,13 +262,26 @@ void  CAtom::SetResidue ( PCResidue res )  {
   residue = res;
 }
 
-void  CAtom::StandardPDBOut ( const pstr Record, pstr S )  {
+void  CAtom::StandardPDBOut ( cpstr Record, pstr S )  {
 char N[10];
-  strcpy     ( S,Record );
-  PadSpaces  ( S,80     );
+  strcpy    ( S,Record );
+  PadSpaces ( S,80     );
+  if (serNum>99999)  {
+    hy36encode ( 5,serNum,N );
+    strcpy_n   ( &(S[6]),N,5 );
+  } else if (serNum>0)
+    PutInteger ( &(S[6]),serNum,5 );
+  else if (index<=99999)
+    PutInteger ( &(S[6]),index,5 );
+  else {
+    hy36encode ( 5,index,N );
+    strcpy_n   ( &(S[6]),N,5 );
+  }
+  /*
   if (serNum>0)
         PutInteger ( &(S[6]),serNum,5 );
   else  PutInteger ( &(S[6]),index ,5 );
+  */
   if (!Ter)  {
     if (altLoc[0])  S[16] = altLoc[0];
     strcpy_n  ( &(S[12]),name   ,4 );
@@ -301,11 +293,18 @@ char N[10];
     } else
       strcpy_n ( &(S[78]),"  ",2 );
   }
-  strcpy_nr  ( &(S[17]),residue->name,3 );
-  S[21] = residue->chain->chainID[0];
-  if (!S[21])  S[21] = ' ';
-  if (residue->seqNum>MinInt4)
-    PutIntIns  ( &(S[22]),residue->seqNum,4,residue->insCode );
+  strcpy_nr ( &(S[17]),residue->name,3 );
+  strcpy_nr ( &(S[20]),residue->chain->chainID,2 );
+//  S[21] = residue->chain->chainID[0];
+//  if (!S[21])  S[21] = ' ';
+  if (residue->seqNum>MinInt4)  {
+    if (-999<=residue->seqNum<=9999)
+      PutIntIns  ( &(S[22]),residue->seqNum,4,residue->insCode );
+    else  {
+      hy36encode ( 4,residue->seqNum,N );
+      strcpy_n   ( &(S[22]),N,4 );
+    }
+  }
 }
 
 void  CAtom::PDBASCIIDump ( RCFile f )  {
@@ -548,7 +547,7 @@ Boolean      singleModel = True;
 
 }
 
-int CAtom::ConvertPDBATOM ( int ix, const pstr S )  {
+int CAtom::ConvertPDBATOM ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII ATOM record.
 //   This function DOES NOT check the "ATOM" keyword and
 // does not decode the chain and residue parameters! These
@@ -604,7 +603,7 @@ void  CAtom::SetAtomName ( int            ix,
   WhatIsSet = 0;
 }
 
-int CAtom::ConvertPDBSIGATM ( int ix, const pstr S )  {
+int CAtom::ConvertPDBSIGATM ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII SIGATM record.
 //   This function DOES NOT check the "SIGATM" keyword and
 // does not decode the chain and residue parameters! These
@@ -638,7 +637,7 @@ int CAtom::ConvertPDBSIGATM ( int ix, const pstr S )  {
 
 }
 
-int CAtom::ConvertPDBANISOU ( int ix, const pstr S )  {
+int CAtom::ConvertPDBANISOU ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII ANISOU record.
 //   This function DOES NOT check the "ANISOU" keyword and
 // does not decode chain and residue parameters! These must
@@ -667,7 +666,8 @@ int CAtom::ConvertPDBANISOU ( int ix, const pstr S )  {
 
   WhatIsSet |= ASET_Anis_tFac;
 
-  if (WhatIsSet & (ASET_Coordinates | ASET_CoordSigma | ASET_Anis_tFSigma))  
+  if (WhatIsSet & (ASET_Coordinates | ASET_CoordSigma |
+                   ASET_Anis_tFSigma))  
     // something was already submitted. check complience
     return CheckData ( S );
   else  
@@ -678,7 +678,7 @@ int CAtom::ConvertPDBANISOU ( int ix, const pstr S )  {
 
 }
 
-int CAtom::ConvertPDBSIGUIJ ( int ix, const pstr S )  {
+int CAtom::ConvertPDBSIGUIJ ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII SIGUIJ record.
 //   This function DOES NOT check the "SIGUIJ" keyword and
 // does not decode the chain and residue parameters! These
@@ -719,7 +719,7 @@ int CAtom::ConvertPDBSIGUIJ ( int ix, const pstr S )  {
 
 }
 
-int CAtom::ConvertPDBTER ( int ix, const pstr S )  {
+int CAtom::ConvertPDBTER ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII TER record.
 //   This function DOES NOT check the "TER" keyword and
 // does not decode the chain and residue parameters! These
@@ -728,11 +728,16 @@ int CAtom::ConvertPDBTER ( int ix, const pstr S )  {
 
   index = ix;
 
-  if (!(GetInteger(serNum,&(S[6]),5)))  serNum = -1;
-  //   Although against strict PDB format, 'TER' is
-  // actually allowed not to have a serial number.
-  // This negative value implies that the number is
-  // not set.
+  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  { 
+    //   Although against strict PDB format, 'TER' is
+    // actually allowed not to have a serial number.
+    // This negative value implies that the number is
+    // not set.
+    if (!(GetInteger(serNum,&(S[6]),5)))  serNum = -1;
+  } else
+    hy36decode ( 5,&(S[6]),5,&serNum );
+
+//  if (!(GetInteger(serNum,&(S[6]),5)))  serNum = -1;
 
   if (WhatIsSet & ASET_Coordinates)
     return Error_ATOM_AlreadySet;
@@ -898,7 +903,7 @@ void  CAtom::CalcAtomStatistics ( RSAtomStat AS )  {
       AS.WhatIsSet &= ~ASET_Coordinates;
 
     if (AS.WhatIsSet & WhatIsSet & ASET_Occupancy)
-          GetStat ( occupancy,AS.occ_min,AS.occ_max,AS.occ_m,AS.occ_m2 );
+          GetStat(occupancy,AS.occ_min,AS.occ_max,AS.occ_m,AS.occ_m2);
     else  AS.WhatIsSet &= ~ASET_Occupancy;
 
     if (AS.WhatIsSet & WhatIsSet & ASET_tempFactor)
@@ -928,6 +933,14 @@ realtype dx,dy,dz;
   return  dx*dx + dy*dy + dz*dz;
 }
 
+realtype CAtom::GetDist2 ( PCAtom a, mat44 & tm )  {
+realtype dx,dy,dz;
+  dx = tm[0][0]*a->x + tm[0][1]*a->y + tm[0][2]*a->z + tm[0][3] - x;
+  dy = tm[1][0]*a->x + tm[1][1]*a->y + tm[1][2]*a->z + tm[1][3] - y;
+  dz = tm[2][0]*a->x + tm[2][1]*a->y + tm[2][2]*a->z + tm[2][3] - z;
+  return  dx*dx + dy*dy + dz*dz;
+}
+
 realtype CAtom::GetDist2 ( realtype ax, realtype ay, realtype az )  {
 realtype dx,dy,dz;
   dx = ax - x;
@@ -979,7 +992,7 @@ void  CAtom::SetElementName ( const Element elName )  {
   }
 }
 
-void  CAtom::SetCharge ( const pstr chrg )  {
+void  CAtom::SetCharge ( cpstr chrg )  {
 pstr p;
   charge = strtod ( chrg,&p );
   if (p!=chrg)  WhatIsSet |= ASET_Charge;
@@ -1070,7 +1083,7 @@ char S[50];
   
 
 
-int CAtom::ConvertPDBHETATM ( int ix, const pstr S )  {
+int CAtom::ConvertPDBHETATM ( int ix, cpstr S )  {
 //   Gets data from the PDB ASCII HETATM record.
 //   This function DOES NOT check the "HETATM" keyword and
 // does not decode the chain and residue parameters! These
@@ -1082,13 +1095,20 @@ int RC;
   return RC;
 }
 
-void CAtom::GetData ( const pstr S )  {
+void CAtom::GetData ( cpstr S )  {
 pstr p;
-  if (!(GetInteger(serNum,&(S[6]),5)))  serNum = index;
-  //   Here we forgive cards with unreadable serial numbers
-  // as we always have index (ix) for the card. For the sake
-  // of strict PDB syntax we would have to return
-  // Error_UnrecognizedInteger .
+
+  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  { 
+    //   Here we forgive cards with unreadable serial numbers
+    // as we always have index (ix) for the card. For the sake
+    // of strict PDB syntax we would have to return
+    // Error_UnrecognizedInteger .
+    if (!(GetInteger(serNum,&(S[6]),5)))  serNum = -1;
+  } else
+    hy36decode ( 5,&(S[6]),5,&serNum );
+
+//  if (!(GetInteger(serNum,&(S[6]),5)))  serNum = index;
+
   altLoc[0] = S[16];
   if (altLoc[0]==' ')  altLoc[0] = char(0);
                  else  altLoc[1] = char(0);
@@ -1098,9 +1118,10 @@ pstr p;
   charge = strtod ( &(S[78]),&p );
   if ((charge!=0.0) && (p!=&(S[78])))
     WhatIsSet |= ASET_Charge;
+
 }
 
-int  CAtom::CheckData ( const pstr S )  {
+int  CAtom::CheckData ( cpstr S )  {
 int      sN;
 AltLoc   aloc;
 SegID    sID;
@@ -1221,7 +1242,7 @@ int  RC;
   if (RC)
     CIFGetString ( element,Loop,CIFTAG_ATOM_TYPE_SYMBOL,k,
                         sizeof(element),pstr("  ") );
-  MakePDBAtomName();
+  //  MakePDBAtomName();
 
   RC = CIFGetReal1 ( sigX,Loop,CIFTAG_CARTN_X_ESD,k );
   if (!RC) RC = CIFGetReal1 ( sigY,Loop,CIFTAG_CARTN_Y_ESD,k );
@@ -1396,7 +1417,7 @@ int  CAtom::PutUDData ( int UDDhandle, realtype rudd )  {
   else  return  UDDATA_WrongUDRType;
 }
 
-int  CAtom::PutUDData ( int UDDhandle, const pstr sudd )  {
+int  CAtom::PutUDData ( int UDDhandle, cpstr sudd )  {
   if (UDDhandle & UDRF_ATOM)
         return  CUDData::putUDData ( UDDhandle,sudd );
   else  return  UDDATA_WrongUDRType;
@@ -1507,14 +1528,14 @@ pstr p1,p2;
   return 1;
 }
 
-int CAtom::CheckIDS ( const pstr ID )  {
+int CAtom::CheckIDS ( cpstr ID )  {
 AtomName aname;
 Element  elname;
 AltLoc   aloc;
 pstr     p;
   p = strrchr ( ID,'/' );
   if (p)  p++;
-    else  p = ID;
+    else  p = pstr(ID);
   ParseAtomID ( p,aname,elname,aloc );
   return  CheckID ( aname,elname,aloc );
 }
@@ -1761,7 +1782,7 @@ MakeFactoryFunctions(CAtom)
 
 
 
-//  ============================  CResidue  ============================
+//  ===========================  CResidue  ===========================
 
 
 void  SAtomStat::Init()  {
@@ -2313,8 +2334,10 @@ PPCAtom A;
 
 }
 
-void  CResidue::_copy ( PCResidue res, PPCAtom atm, int & atom_index )  {
-//  modify both CResidue::_copy and CResidue::Copy methods simultaneously!
+void  CResidue::_copy ( PCResidue res, PPCAtom atm,
+                        int & atom_index )  {
+//  modify both CResidue::_copy and CResidue::Copy methods
+// simultaneously!
 //
 //  This function physically copies the atoms, creating new atom
 // instances and putting them into array 'atm' sequentially from
@@ -2534,7 +2557,7 @@ int CResidue::CheckID ( int * snum,
   return 1;
 }
 
-int CResidue::CheckIDS ( const pstr CID )  {
+int CResidue::CheckIDS ( cpstr CID )  {
 ChainID  chn;
 InsCode  inscode;
 ResName  resname;
@@ -2858,7 +2881,7 @@ int  CResidue::PutUDData ( int UDDhandle, realtype rudd )  {
   else  return  UDDATA_WrongUDRType;
 }
 
-int  CResidue::PutUDData ( int UDDhandle, const pstr sudd )  {
+int  CResidue::PutUDData ( int UDDhandle, cpstr sudd )  {
   if (UDDhandle & UDRF_RESIDUE)
         return  CUDData::putUDData ( UDDhandle,sudd );
   else  return  UDDATA_WrongUDRType;
