@@ -1,53 +1,28 @@
-//  $Id: mmdb_atom.h,v 1.13 2005/09/22 16:32:38 keb Exp $
+//  $Id: mmdb_atom.h,v 1.17 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    07.06.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -60,7 +35,7 @@
 //  **** Functions :  BondAngle
 //       ~~~~~~~~~~~
 //
-//  Copyright (C) E. Krissinel 2001-2004
+//  Copyright (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -261,12 +236,12 @@ class CAtom : public CUDData  {
     // CMMDBFile::ReadPDBAtom().
     //   The atom reference is updated in the corresponding
     // residue.
-    int ConvertPDBATOM   ( int ix, const pstr S );
-    int ConvertPDBSIGATM ( int ix, const pstr S );
-    int ConvertPDBANISOU ( int ix, const pstr S );
-    int ConvertPDBSIGUIJ ( int ix, const pstr S );
-    int ConvertPDBTER    ( int ix, const pstr S );
-    int ConvertPDBHETATM ( int ix, const pstr S );
+    int ConvertPDBATOM   ( int ix, cpstr S );
+    int ConvertPDBSIGATM ( int ix, cpstr S );
+    int ConvertPDBANISOU ( int ix, cpstr S );
+    int ConvertPDBSIGUIJ ( int ix, cpstr S );
+    int ConvertPDBTER    ( int ix, cpstr S );
+    int ConvertPDBHETATM ( int ix, cpstr S );
 
     int GetCIF           ( int ix, PCMMCIFLoop Loop,
                            PCMMCIFLoop LoopAnis );
@@ -282,7 +257,7 @@ class CAtom : public CUDData  {
     //  This only renames the atom
     void  SetAtomName    ( const AtomName atomName );
     void  SetElementName ( const Element  elName   );
-    void  SetCharge      ( const pstr     chrg     );
+    void  SetCharge      ( cpstr          chrg     );
     void  SetCharge      ( realtype       chrg     );
 
     void  SetAtomIndex   ( int ix ); // don't use in your applications!
@@ -314,6 +289,7 @@ class CAtom : public CUDData  {
 
     void  CalcAtomStatistics ( RSAtomStat AS );
     realtype GetDist2 ( PCAtom a );
+    realtype GetDist2 ( PCAtom a, mat44 & tm );  // tm applies to A
     realtype GetDist2 ( realtype ax, realtype ay, realtype az );
 
     // GetCosine(a1,a2) calculates cosine of angle a1-this-a2,
@@ -395,7 +371,7 @@ class CAtom : public CUDData  {
     //   *[*]:A      (same as above)
     //   *[*]:*      (any atom)
     //   *           (any atom with no alternate location indicator)
-    int   CheckIDS ( const pstr ID );
+    int   CheckIDS ( cpstr ID );
 
 
     // -------  transform coordinates: x := m*x + v
@@ -408,9 +384,9 @@ class CAtom : public CUDData  {
 
 
     // -------  user-defined data handlers
-    int   PutUDData ( int UDDhandle, int        iudd );
-    int   PutUDData ( int UDDhandle, realtype   rudd );
-    int   PutUDData ( int UDDhandle, const pstr sudd );
+    int   PutUDData ( int UDDhandle, int      iudd );
+    int   PutUDData ( int UDDhandle, realtype rudd );
+    int   PutUDData ( int UDDhandle, cpstr    sudd );
 
     int   GetUDData ( int UDDhandle, int      & iudd );
     int   GetUDData ( int UDDhandle, realtype & rudd );
@@ -434,9 +410,9 @@ class CAtom : public CUDData  {
 
     void  InitAtom       ();
     void  FreeMemory     ();
-    void  StandardPDBOut ( const pstr Record, pstr S );
-    void  GetData        ( const pstr S );
-    int   CheckData      ( const pstr S );
+    void  StandardPDBOut ( cpstr Record, pstr S );
+    void  GetData        ( cpstr S );
+    int   CheckData      ( cpstr S );
     void  GetStat        ( realtype   v,
                            realtype & v_min, realtype & v_max,
                            realtype & v_m,   realtype & v_m2 );
@@ -599,7 +575,7 @@ class CResidue : public CUDData  {
     //        *(*).A      (same as above)
     //        *(*).*      (any residue)
     //        *           (any residue with no insertion code)
-    int  CheckIDS ( const pstr ID );
+    int  CheckIDS ( cpstr ID );
 
 
     //  --------------------  Extracting atoms  ----------------------
@@ -608,7 +584,7 @@ class CResidue : public CUDData  {
     int  GetNumberOfAtoms ( Boolean countTers );
 
     PCAtom GetAtom ( const AtomName aname, const Element elname=NULL,
-                     const AltLoc aloc=pstr("") );
+                     const AltLoc aloc=cpstr("") );
     PCAtom GetAtom ( int atomNo );
 
     void GetAtomTable  ( PPCAtom & atomTable, int & NumberOfAtoms );
@@ -629,7 +605,7 @@ class CResidue : public CUDData  {
     //  ---------------------  Deleting atoms  -----------------------
 
     int  DeleteAtom ( const AtomName aname, const Element elname=NULL,
-                      const AltLoc aloc=pstr("") );
+                      const AltLoc aloc=cpstr("") );
     int  DeleteAtom ( int atomNo );
     int  DeleteAllAtoms();
 
@@ -683,9 +659,9 @@ class CResidue : public CUDData  {
 
 
     // -------  user-defined data handlers
-    int   PutUDData ( int UDDhandle, int        iudd );
-    int   PutUDData ( int UDDhandle, realtype   rudd );
-    int   PutUDData ( int UDDhandle, const pstr sudd );
+    int   PutUDData ( int UDDhandle, int      iudd );
+    int   PutUDData ( int UDDhandle, realtype rudd );
+    int   PutUDData ( int UDDhandle, cpstr    sudd );
 
     int   GetUDData ( int UDDhandle, int      & iudd );
     int   GetUDData ( int UDDhandle, realtype & rudd );
