@@ -1,53 +1,28 @@
-//  $Id: mmdb_coormngr.h,v 1.13 2005/12/19 16:51:28 keb Exp $
+//  $Id: mmdb_coormngr.h,v 1.18 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    26.11.03   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CBrick           ( space brick                  )
 //       ~~~~~~~~~  CMMDBCoorManager ( MMDB atom coordinate manager )
 //
-//  E. Krissinel 2001-2003
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -151,7 +126,7 @@ class CGenSym : public CSymOps  {
 
     void FreeMemory();
 
-    int  AddSymOp    ( const pstr XYZOperation );
+    int  AddSymOp    ( cpstr XYZOperation );
     //  the number of just added operation may be obtained as
     //  Nop = CGenSym::GetNofSymOps()-1 .
 
@@ -246,7 +221,7 @@ class CMMDBCoorManager : public CMMDBFile  {
 
     //  ----------------------------------------------------------
 
-    int  SetDefaultCoorID ( const pstr CID );
+    int  SetDefaultCoorID ( cpstr CID );
 
 
     //  ----------------  Bricking  ------------------------------
@@ -277,14 +252,16 @@ class CMMDBCoorManager : public CMMDBFile  {
     //  ----------------  Extracting models  ---------------------
 
     int GetNumberOfModels ()  { return nModels; }
+    int  GetFirstModelNum ();
+    PCModel GetFirstDefinedModel();
     PCModel      GetModel ( int   modelNo  );  // 1<=modelNo<=nModels
-    PCModel      GetModel ( const pstr CID );
+    PCModel      GetModel ( cpstr CID );
     void    GetModelTable ( PPCModel & modTable,
                             int & NumberOfModels );
 
     //  ----------------  Deleting models  -----------------------
 
-    int  DeleteModel ( const pstr CID );
+    int  DeleteModel ( cpstr CID );
     int  DeleteModel ( int modelNo );  // 1<=modelNo<=nOfModels
 
     //  ----------------  Adding/Inserting models  ---------------
@@ -297,13 +274,13 @@ class CMMDBCoorManager : public CMMDBFile  {
     //  ----------------  Extracting chains  ---------------------
 
     int GetNumberOfChains ( int   modelNo  );
-    int GetNumberOfChains ( const pstr CID );
+    int GetNumberOfChains ( cpstr CID );
     PCChain      GetChain ( int modelNo, const ChainID chainID );
     PCChain      GetChain ( int modelNo, int   chainNo );
-    PCChain      GetChain ( const pstr CID );
+    PCChain      GetChain ( cpstr CID );
     void    GetChainTable ( int modelNo, PPCChain & chainTable,
                             int & NumberOfChains );
-    void    GetChainTable ( const pstr CID, PPCChain & chainTable,
+    void    GetChainTable ( cpstr CID, PPCChain & chainTable,
                             int & NumberOfChains );
 
     //  -----------------  Deleting chains  ----------------------
@@ -321,7 +298,7 @@ class CMMDBCoorManager : public CMMDBFile  {
 
     int GetNumberOfResidues ( int modelNo, const ChainID chainID );
     int GetNumberOfResidues ( int modelNo, int   chainNo );
-    int GetNumberOfResidues ( const pstr CID );
+    int GetNumberOfResidues ( cpstr CID );
     PCResidue    GetResidue ( int modelNo, const ChainID chainID,
                               int seqNo,   const InsCode insCode );
     PCResidue    GetResidue ( int modelNo, int   chainNo,
@@ -329,7 +306,7 @@ class CMMDBCoorManager : public CMMDBFile  {
     PCResidue    GetResidue ( int modelNo, const ChainID chainID,
                                                      int resNo );
     PCResidue    GetResidue ( int modelNo, int   chainNo, int resNo );
-    PCResidue    GetResidue ( const pstr CID );
+    PCResidue    GetResidue ( cpstr CID );
     int        GetResidueNo ( int modelNo, const ChainID chainID,
                               int seqNo,   const InsCode insCode );
     int        GetResidueNo ( int modelNo, int   chainNo,
@@ -342,7 +319,7 @@ class CMMDBCoorManager : public CMMDBFile  {
     void    GetResidueTable ( int modelNo, int chainNo,
                               PPCResidue & resTable,
                               int & NumberOfResidues );
-    void    GetResidueTable ( const pstr CID, PPCResidue & resTable,
+    void    GetResidueTable ( cpstr CID, PPCResidue & resTable,
                               int & NumberOfResidues );
 
 
@@ -377,7 +354,7 @@ class CMMDBCoorManager : public CMMDBFile  {
     int   GetNumberOfAtoms ( int modelNo, const ChainID chainID,
                                                     int resNo );
     int   GetNumberOfAtoms ( int modelNo, int   chainNo, int resNo );
-    int   GetNumberOfAtoms ( const pstr CID );
+    int   GetNumberOfAtoms ( cpstr CID );
 
     PCAtom GetAtom (
               int            modelNo, // model serial number 1...
@@ -469,7 +446,7 @@ class CMMDBCoorManager : public CMMDBFile  {
     //           alternate location indicator is looked for.
     // All spaces are ignored, all identifiers should be in capital
     // letters (comparisons are case-sensitive).
-    PCAtom GetAtom ( const pstr CID );
+    PCAtom GetAtom ( cpstr CID );
 
 
     void GetAtomTable ( PPCAtom & atomTable, int & NumberOfAtoms );
@@ -483,7 +460,7 @@ class CMMDBCoorManager : public CMMDBFile  {
                         PPCAtom & atomTable, int & NumberOfAtoms );
     void GetAtomTable ( int modelNo, int     chainNo, int resNo,
                         PPCAtom & atomTable, int & NumberOfAtoms );
-    void GetAtomTable ( const pstr CID, PPCAtom & atomTable,
+    void GetAtomTable ( cpstr CID, PPCAtom & atomTable,
                         int & NumberOfAtoms );
 
 
@@ -508,7 +485,7 @@ class CMMDBCoorManager : public CMMDBFile  {
                          PPCAtom & atomTable, int & NumberOfAtoms );
     void GetAtomTable1 ( int modelNo, int     chainNo, int resNo,
                          PPCAtom & atomTable, int & NumberOfAtoms );
-    void GetAtomTable1 ( const pstr CID, PPCAtom & atomTable,
+    void GetAtomTable1 ( cpstr CID, PPCAtom & atomTable,
                          int & NumberOfAtoms );
 
 
@@ -891,8 +868,8 @@ extern void  GetRotParameters ( mat33 & vrm, realtype & alpha,
 // the center of rotation -- actually a point belonging to the
 // rotation axis.
 extern void  GetVecTMatrix  ( mat44 & vrm, realtype alpha, 
-                              realtype vx, realtype vy,  realtype vz,
-                              realtype x0, realtype y0,  realtype z0 );
+                              realtype vx, realtype vy, realtype vz,
+                              realtype x0, realtype y0, realtype z0 );
 
 //   Vector rotation is rotation by angle alpha about arbitrary
 // vector directed as (vx,vy,vz) = (vx2-vx1,vy2-vy1,vz2-vz1).
