@@ -1,53 +1,28 @@
-//  $Id: mmdb_cryst.cpp,v 1.17 2005/09/22 16:32:38 keb Exp $
+//  $Id: mmdb_cryst.cpp,v 1.24 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    06.12.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -60,7 +35,7 @@
 //                  CTVect          ( translational vector class    )
 //                  CMMDBCryst      ( MMDB cryst. section class     )
 //
-//  E. Krissinel 2000-2004
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -102,7 +77,7 @@ PCContainerClass CCrystContainer::MakeContainerClass ( int ClassID )  {
   }
 }
 
-int CCrystContainer::AddMTRIXLine ( const pstr S )  {
+int CCrystContainer::AddMTRIXLine ( cpstr S )  {
 int i,RC;
   RC = Error_NCSM_WrongSerial;
   for (i=0;i<length;i++)  {
@@ -122,7 +97,7 @@ CNCSMatrix::CNCSMatrix() : CContainerClass()  {
   Init();
 }
 
-CNCSMatrix::CNCSMatrix ( const pstr S ) : CContainerClass()  {
+CNCSMatrix::CNCSMatrix ( cpstr S ) : CContainerClass()  {
   Init();
   ConvertPDBASCII ( S );
 }
@@ -168,7 +143,7 @@ int  i,j;
 
 }
 
-int CNCSMatrix::ConvertPDBASCII ( const pstr S )  {
+int CNCSMatrix::ConvertPDBASCII ( cpstr S )  {
 int      sN,iG;
 realtype m0,m1,m2,v0;
 
@@ -402,7 +377,7 @@ CTVect::CTVect() : CContainerClass()  {
   Init();
 }
 
-CTVect::CTVect ( const pstr S ) : CContainerClass()  {
+CTVect::CTVect ( cpstr S ) : CContainerClass()  {
   Init();
   ConvertPDBASCII ( S );
 }
@@ -434,7 +409,7 @@ void CTVect::PDBASCIIDump ( pstr S, int N )  {
     strncpy ( &(S[40]),comment,IMin(30,strlen(comment)) );
 }
 
-int CTVect::ConvertPDBASCII ( const pstr S )  {
+int CTVect::ConvertPDBASCII ( cpstr S )  {
   GetInteger ( serNum ,&(S[7]) ,3  );
   GetReal    ( t[0]   ,&(S[10]),10 );
   GetReal    ( t[1]   ,&(S[20]),10 );
@@ -608,32 +583,39 @@ void  CMMDBCryst::Reset()  {
   Init ( False );
 }
 
-const pstr rhombohedral[] = {
-  pstr("R 3"  ),
-  pstr("R 3"  ),
-  pstr("R 3 2"),
-  pstr("R 3 2")
+cpstr rhombohedral[] = {
+  cpstr("R 3"  ),
+  cpstr("R 3"  ),
+  cpstr("R 3 2"),
+  cpstr("R 3 2")
 };
 
-const pstr short_mono[] = {
-  pstr("P 2" ),
-  pstr("P 21"),
-  pstr("C 2" ),
-  pstr("A 2" ),
-  pstr("B 2" ),
-  pstr("I 2" )
+cpstr short_mono[] = {
+  cpstr("P 2" ),
+  cpstr("P 21"),
+  cpstr("C 2" ),
+  cpstr("A 2" ),
+  cpstr("B 2" ),
+  cpstr("I 2" )
 };
 
-const pstr special[] = {
-  pstr("A1"     ), pstr("Hall:  P 1 (-x,-1/2*y+1/2*z,1/2*y+1/2*z)"  ),
-  pstr("C1211"  ), pstr("Hall:  C 2y (x+1/4,y+1/4,z)"               ),
-  pstr("C21"    ), pstr("Hall:  C 2y (x+1/4,y+1/4,z)"               ),
-  pstr("I1211"  ), pstr("Hall:  C 2y (x+1/4,y+1/4,-x+z-1/4)"        ),
-  pstr("I21"    ), pstr("Hall:  C 2y (x+1/4,y+1/4,-x+z-1/4)"        ),
-  pstr("P21212A"), pstr("Hall:  P 2 2ab (x+1/4,y+1/4,z)"            ),
-  pstr("F422"   ), pstr("Hall:  I 4 2 (1/2*x+1/2*y,-1/2*x+1/2*y,z)" ),
-  pstr("C4212"  ),
-            pstr("Hall:  P 4 2 (1/2*x+1/2*y-1/4,-1/2*x+1/2*y-1/4,z)")
+cpstr special[] = {
+  cpstr("A1"     ),
+            cpstr("Hall:  P 1 (-x,-1/2*y+1/2*z,1/2*y+1/2*z)"  ),
+  cpstr("C1211"  ),
+            cpstr("Hall:  C 2y (x+1/4,y+1/4,z)"               ),
+  cpstr("C21"    ),
+            cpstr("Hall:  C 2y (x+1/4,y+1/4,z)"               ),
+  cpstr("I1211"  ),
+            cpstr("Hall:  C 2y (x+1/4,y+1/4,-x+z-1/4)"        ),
+  cpstr("I21"    ),
+            cpstr("Hall:  C 2y (x+1/4,y+1/4,-x+z-1/4)"        ),
+  cpstr("P21212A"),
+            cpstr("Hall:  P 2 2ab (x+1/4,y+1/4,z)"            ),
+  cpstr("F422"   ),
+            cpstr("Hall:  I 4 2 (1/2*x+1/2*y,-1/2*x+1/2*y,z)" ),
+  cpstr("C4212"  ),
+            cpstr("Hall:  P 4 2 (1/2*x+1/2*y-1/4,-1/2*x+1/2*y-1/4,z)")
 };
 
 
@@ -1090,18 +1072,18 @@ char          S[200];
 
 
 
-pstr OrthCode[6] = {
-  pstr("A/X0, C*/Z0"), // (standard brookhaven)
-  pstr("B/X0, A*/Z0"),
-  pstr("C/X0, B*/Z0"),
-  pstr("HEX A+B/X0, C*/Z0"),
-  pstr("A*/X0, C/Z0 (rollett)"),
-  pstr("A/X0, B*/Y0")
+cpstr OrthCode[6] = {
+  cpstr("A/X0, C*/Z0"), // (standard brookhaven)
+  cpstr("B/X0, A*/Z0"),
+  cpstr("C/X0, B*/Z0"),
+  cpstr("HEX A+B/X0, C*/Z0"),
+  cpstr("A*/X0, C/Z0 (rollett)"),
+  cpstr("A/X0, B*/Y0")
 };
 
-pstr getOrthCodeName ( int NCode )  {
+cpstr getOrthCodeName ( int NCode )  {
   if ((NCode>0) && (NCode<=6))  return OrthCode[NCode-1];
-  return "CUSTOM";
+  return cpstr("CUSTOM");
 }
 
 void  CMMDBCryst::CalcCoordTransforms()  {
@@ -1637,6 +1619,20 @@ int   i,j,k;
 
 }
 
+int  CMMDBCryst::GetSymOpMatrix ( mat44 & TMatrix, int Nop )  {
+//
+//  GetSymOpMatrix(..) returns the transformation matrix for
+//  Nop-th symmetry operator in the space group
+//
+//  Return 0 means everything's fine,
+//         1 there's no symmetry operation Nop defined
+//         2 fractionalizing/orthogonalizing matrices were not
+//           calculated
+//         3 cell parameters were not set up.
+//
+  return SymOps.GetTMatrix ( TMatrix,Nop );
+}
+
 
 Boolean CMMDBCryst::Cryst2Orth ( rvector U )  {
 mat33    A,AT,Tmp,TmpMat;
@@ -1792,17 +1788,18 @@ int  i,j;
       RFU[i][i] = 1.0;
     }
 
-    CalcCoordTransforms ();
+    CalcCoordTransforms();
 
-    for (i=0;i<3;i++)  {
-      for (j=0;j<3;j++)
-        RO[i][j] = RR[NCode][i][j];
-      RO[i][3] = 0.0;
-      RO[3][i] = 0.0;
+    if (!(CellCheck & CCHK_NoOrthCode))  {
+      for (i=0;i<3;i++)  {
+        for (j=0;j<3;j++)
+          RO[i][j] = RR[NCode][i][j];
+        RO[i][3] = 0.0;
+        RO[3][i] = 0.0;
+      }
+      RO[3][3] = 1.0;
+      Mat4Inverse ( RO,RF );
     }
-    RO[3][3] = 1.0;
-
-    Mat4Inverse ( RO,RF );
 
     WhatIsSet |= CSET_Transforms;
 
@@ -1812,7 +1809,7 @@ int  i,j;
 
 }
 
-void CMMDBCryst::SetSyminfoLib ( const pstr syminfoLib )  {
+void CMMDBCryst::SetSyminfoLib ( cpstr syminfoLib )  {
   CreateCopy ( syminfo_lib,syminfoLib );
 }
 
@@ -1820,7 +1817,7 @@ pstr CMMDBCryst::GetSyminfoLib()  {
   return syminfo_lib;
 }
 
-int CMMDBCryst::SetSpaceGroup ( const pstr spGroup )  {
+int CMMDBCryst::SetSpaceGroup ( cpstr spGroup )  {
 //  This function does not attempt to fix the space group
 int RC,l;
   RC = SYMOP_UnknownSpaceGroup;
@@ -1912,6 +1909,21 @@ int  CMMDBCryst::GetNumberOfNCSMatrices()  {
   return  NCSMatrix.Length();
 }
 
+int  CMMDBCryst::GetNumberOfNCSMates()  {
+// Returns the number of NCS mates not given in the file (iGiven==0)
+int         i,l,iG;
+PCNCSMatrix NCSM;
+  iG = 0;
+  l  = NCSMatrix.Length();
+  for (i=0;i<l;i++)  {
+    NCSM = PCNCSMatrix(NCSMatrix.GetContainerClass(i));
+    if (NCSM)  {
+      if (!NCSM->iGiven)  iG++;
+    }
+  }
+  return iG;
+}
+
 Boolean CMMDBCryst::GetNCSMatrix ( int NCSMatrixNo,
                                    mat33 & ncs_m, vect3 & ncs_v )  {
 int         i,j;
