@@ -1,53 +1,28 @@
-//  $Id: mmdb_file.cpp,v 1.21 2005/09/22 16:32:38 keb Exp $
+//  $Id: mmdb_file.cpp,v 1.31 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    07.06.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CMMDBFile  ( macromolecular data file class )
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2000-2004
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -212,7 +187,7 @@ void CMMDBFile::RemoveFlag ( word Flag )  {
 }
 
 
-int  CMMDBFile::ReadPDBASCII1 ( const pstr PDBLFName,
+int  CMMDBFile::ReadPDBASCII1 ( cpstr PDBLFName,
                                 byte gzipMode )  {
 pstr FName;
   FName = getenv ( PDBLFName );
@@ -240,7 +215,7 @@ Boolean Done;
   PadSpaces  ( L,80 );
 }
 
-int  CMMDBFile::ReadPDBASCII ( const pstr PDBFileName,
+int  CMMDBFile::ReadPDBASCII ( cpstr PDBFileName,
                                byte gzipMode )  {
 CFile f;
 int   RC;
@@ -272,7 +247,7 @@ int   RC;
 int  CMMDBFile::ReadPDBASCII ( RCFile f )  {
 PCContString ContString;
 word         cleanKey;
-int          RC;
+int          RC,modNum;
 Boolean      fixSpaceGroup,fend;
 
   //  remove previous data
@@ -357,8 +332,18 @@ Boolean      fixSpaceGroup,fend;
          strncmp(S,"TER   ",6) &&
          strncmp(S,"HETATM",6) &&
          strncmp(S,"ENDMDL",6))  {
-    ContString = new CContString(S);
-    SA.AddData ( ContString );
+    if (!strncmp(S,"LINK  ",6))
+      crModel->ConvertPDBString(S);
+    else if (!strncmp(S,"CISPEP",6)) {
+      GetInteger ( modNum,&(S[43]),3 );
+      if (modNum<=0)  modNum = 1;
+      if (modNum!=1)  SwitchModel ( modNum );
+      crModel->ConvertPDBString(S);
+      if (modNum!=1)  SwitchModel ( 1 );
+    } else  {
+      ContString = new CContString(S);
+      SA.AddData ( ContString );
+    }
     ReadPDBLine ( f,S,sizeof(S) );
     Title.TrimInput ( S );
     lcount++;
@@ -427,10 +412,14 @@ Boolean      fixSpaceGroup,fend;
 //  commented on 28.05.2004, it appears that "CHAIN_ORDER" should not
 //  be enforced here
 //  cleanKey = PDBCLEAN_ATNAME | PDBCLEAN_CHAIN_ORDER;
-  cleanKey = PDBCLEAN_ATNAME;
+  cleanKey = 0x00000000;
+  if (Flags & MMDBF_EnforceAtomNames)
+    cleanKey = PDBCLEAN_ATNAME;
   if (Flags & MMDBF_AutoSerials)
     cleanKey |= PDBCLEAN_SERIAL;
-  PDBCleanup ( cleanKey );
+
+  if (cleanKey)
+    PDBCleanup ( cleanKey );
 
   if ((!f.FileEnd()) && (RC!=Error_WrongSection))  return RC;
 
@@ -452,16 +441,14 @@ Boolean      fixSpaceGroup,fend;
 }
 
 
-int  CMMDBFile::ReadCIFASCII1 ( const pstr CIFLFName,
-                                byte gzipMode )  {
+int  CMMDBFile::ReadCIFASCII1 ( cpstr CIFLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( CIFLFName );
   if (FName)  return ReadCIFASCII ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::ReadCIFASCII ( const pstr CIFFileName,
-                               byte gzipMode )  {
+int  CMMDBFile::ReadCIFASCII ( cpstr CIFFileName, byte gzipMode )  {
 CFile f;
 int   W;
 Boolean fixSpaceGroup;
@@ -520,10 +507,12 @@ Boolean fixSpaceGroup;
 }
 
 
-int CMMDBFile::ReadFromCIF ( PCMMCIFData CIFD, Boolean fixSpaceGroup )  {
-int          RC,i,l,j,n,retc;
-pstr         F,FC;
+int CMMDBFile::ReadFromCIF ( PCMMCIFData CIFD,
+                             Boolean fixSpaceGroup )  {
 PCMMCIFLoop  Loop1,Loop2;
+pstr         F,FC;
+word         cleanKey;
+int          RC,i,l,j,n,retc;
 
   RC = Title.GetCIF ( CIFD );
 
@@ -586,18 +575,26 @@ PCMMCIFLoop  Loop1,Loop2;
     CIFD->Optimize   ();
   }
 
+  cleanKey = 0x00000000;
+  if (Flags & MMDBF_EnforceAtomNames)
+    cleanKey = PDBCLEAN_ATNAME;
+  if (Flags & MMDBF_AutoSerials)
+    cleanKey |= PDBCLEAN_SERIAL;
+  if (cleanKey)
+    PDBCleanup ( cleanKey );
+
   return RC;
 
 }
 
-int CMMDBFile::ReadCoorFile1 ( const pstr LFName, byte gzipMode )  {
+int CMMDBFile::ReadCoorFile1 ( cpstr LFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( LFName );
   if (FName)  return ReadCoorFile ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int CMMDBFile::ReadCoorFile ( const pstr CFName, byte gzipMode )  {
+int CMMDBFile::ReadCoorFile ( cpstr CFName, byte gzipMode )  {
 // auto format recognition
 int     kin;
 Boolean IBL;
@@ -629,11 +626,12 @@ word  CMMDBFile::PDBCleanup ( word CleanKey )  {
 //  PDBCLEAN_CHAIN   generates 1-character chain ids instead of
 //                   those many-character
 //  PDBCLEAN_CHAIN_STRONG generates 1-character chain ids starting
-//                   from 'A' on for all ids, including single-character
+//                   from 'A' on for all ids, including single-char
 //  PDBCLEAN_ALTCODE generates 1-character alternative codes instead
 //                   of those many-character
 //  PDBCLEAN_ALTCODE_STRONG generates 1-character alternative codes
-//                   from 'A' on for all codes, including single-character
+//                   from 'A' on for all codes, including
+//                   single-character ones
 //  PDBCLEAN_SERIAL  puts serial numbers in due order
 //  PDBCLEAN_INDEX   reorders the internal index of atoms such that
 //                   it follows the actual order of atoms in
@@ -945,7 +943,7 @@ Boolean   NewChain,Done,Solvent;
     nAtoms = k;
   }
 
-  if (CleanKey & PDBCLEAN_CHAIN_ORDER)  {
+  if (CleanKey & (PDBCLEAN_CHAIN_ORDER | PDBCLEAN_CHAIN_ORDER_IX))  {
     for (i=0;i<nModels;i++)  {
       crModel0 = Model[i];
       if (crModel0)  {
@@ -965,12 +963,20 @@ Boolean   NewChain,Done,Solvent;
         crModel0->nChains = k;
       }
     }
-    for (i=0;i<nAtoms;i++)
-      if (Atom[i])  {
-        crChain0 = Atom[i]->GetChain();
-        crChain0->nWeights++;
-        crChain0->Weight += Atom[i]->serNum;
-      }
+    if (CleanKey & PDBCLEAN_CHAIN_ORDER)
+      for (i=0;i<nAtoms;i++)
+        if (Atom[i])  {
+          crChain0 = Atom[i]->GetChain();
+          crChain0->nWeights++;
+          crChain0->Weight += Atom[i]->serNum;
+        }
+    else
+      for (i=0;i<nAtoms;i++)
+        if (Atom[i])  {
+          crChain0 = Atom[i]->GetChain();
+          crChain0->nWeights++;
+          crChain0->Weight += Atom[i]->GetIndex();
+        }
     for (i=0;i<nModels;i++)  {
       crModel0 = Model[i];
       if (crModel0)  {
@@ -983,7 +989,8 @@ Boolean   NewChain,Done,Solvent;
         do {
           Done = True;
           for (j=1;j<crModel0->nChains;j++)
-            if (crModel0->Chain[j-1]->Weight>crModel0->Chain[j]->Weight)  {
+            if (crModel0->Chain[j-1]->Weight >
+                crModel0->Chain[j]->Weight)  {
               crChain0             = crModel0->Chain[j-1];
               crModel0->Chain[j-1] = crModel0->Chain[j];
               crModel0->Chain[j]   = crChain0;
@@ -1062,7 +1069,7 @@ Boolean   NewChain,Done,Solvent;
 
 }
 
-void  CMMDBFile::MakeHetAtoms ( const pstr chainID, Boolean Make )  {
+void  CMMDBFile::MakeHetAtoms ( cpstr chainID, Boolean Make )  {
 //  Makes all atoms in chain 'chainID', in all models, as 'Het' atoms
 //  if Make is set True, and makes them 'ordinary' atoms otherwise.
 //  'Ter' is automatically removed when converting to 'Het' atoms,
@@ -1095,14 +1102,14 @@ PCResidue crRes0;
                 SwitchModel ( Model[i]->GetSerNum() );
                 if (n<nAtoms-1)
                   PutAtom ( -(n+2),Atom[n]->serNum+1,pstr("TER"),
-                            Atom[n]->GetResName(),Atom[n]->GetChainID(),
-                            Atom[n]->GetSeqNum (),Atom[n]->GetInsCode(),
-                            pstr(" "),pstr(" "),pstr(" ") );
+                          Atom[n]->GetResName(),Atom[n]->GetChainID(),
+                          Atom[n]->GetSeqNum (),Atom[n]->GetInsCode(),
+                          pstr(" "),pstr(" "),pstr(" ") );
                 else
                   PutAtom ( 0,nAtoms+1,pstr("TER"),
-                            Atom[n]->GetResName(),Atom[n]->GetChainID(),
-                            Atom[n]->GetSeqNum (),Atom[n]->GetInsCode(),
-                            pstr(" "),pstr(" "),pstr(" ") );
+                          Atom[n]->GetResName(),Atom[n]->GetChainID(),
+                          Atom[n]->GetSeqNum (),Atom[n]->GetInsCode(),
+                          pstr(" "),pstr(" "),pstr(" ") );
                 Atom[n+1]->MakeTer();
               }
             }
@@ -1301,88 +1308,6 @@ int i,j;
   nModels = j;
 }
 
-/*
-int  CMMDBFile::GenerateNCSMates()  {
-//
-//   Generates NCS mates according to NCS matrices given
-// in Cryst. This will result in generating many-character
-// chain names, composed as 'x_n' where 'x' is the original
-// name and 'n' is a unique number, which will coincide with
-// the symmetry operation (order) number. Another side
-// effect will be a disorder in atoms' serial numbers.
-//   The hierarchy should therefore be cleaned after
-// generating the NCS mates. An appropriate way to do that
-// is to issue the following call:
-//
-//   PDBCleanup ( PDBCLEAN_TER | PDBCLEAN_ALTCODE_STRONG |
-//                PDBCLEAN_CHAIN_STRONG | PDBCLEAN_SERIAL );
-//
-PPCMMDBFile Mate;
-int         i,j,k,n,nMates,nMates1,nAtoms1;
-PPCAtom     Atom1;
-PPCModel    Model1;
-
-  nMates = Cryst.GetNumberOfNCSMatrices();
-  if (nMates<=0)  return 1;
-
-  nMates1 = nMates-1;
-  if (nMates1>0)  {
-
-    //  Generate symmetry mates in parallel hierarchies
-    Mate = new PCMMDBFile[nMates1];
-    for (i=0;i<nMates1;i++)  {
-      Mate[i] = new CMMDBFile();
-      Mate[i]->Copy ( this );
-      Mate[i]->ApplyNCSTransform ( i+1 );
-    }
-
-    //  Transform the original hierarchy
-    ApplyNCSTransform ( 0 );
-
-    //  Gather all symmetry mates in 'this' hierarchy
-    nAtoms1 = nMates*nAtoms;
-    Atom1   = new PCAtom[nAtoms1];
-
-    if (nModels>0)  Model1 = new PCModel[nModels];
-              else  Model1 = NULL;
-
-    k = 0;  // index of collected atoms
-    for (i=0;i<nModels;i++)
-      if (Model[i])  {
-        Model1[i] = newCModel();
-        Model1[i]->SetMMDBManager ( PCMMDBManager(this),i+1 );
-        for (j=0;j<Model[i]->nChains;j++)  {
-          Model1[i]->MoveChain ( Model[i]->Chain[j],Atom,Atom1,k,0 );
-          for (n=0;n<nMates1;n++)
-            Model1[i]->MoveChain ( Mate[n]->Model[i]->Chain[j],
-                                   Mate[n]->Atom,Atom1,k,n+1 );
-        }
-      } else
-        Model1[i] = NULL;
-
-    if (Model) delete[] Model;
-    Model = Model1;
-
-    for (i=0;i<nAtoms;i++)
-      if (Atom[i])  delete Atom[i];  // should never happen
-    if (Atom)  delete[] Atom;
-    Atom   = Atom1;
-    AtmLen = nAtoms1;   // length of Atom array
-    nAtoms = k;
-
-    //  Dispose parallel hierarchies
-    for (i=0;i<nMates1;i++)
-      delete Mate[i];
-    delete[] Mate;
-
-  } else
-    // transform just the original hierarchy
-    ApplyNCSTransform ( 0 );
-
-  return 0;
-
-}
-*/
 
 int  CMMDBFile::GenerateNCSMates()  {
 //
@@ -1449,7 +1374,7 @@ int   i;
 }
 
 
-int  CMMDBFile::PutPDBString ( const pstr PDBString )  {
+int  CMMDBFile::PutPDBString ( cpstr PDBString )  {
 int          RC;
 PCContString ContString;
 
@@ -1490,14 +1415,14 @@ PCContString ContString;
 }
 
 
-int  CMMDBFile::AddPDBASCII1 ( const pstr PDBLFName, byte gzipMode )  {
+int  CMMDBFile::AddPDBASCII1 ( cpstr PDBLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( PDBLFName );
   if (FName)  return AddPDBASCII ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::AddPDBASCII ( const pstr PDBFileName, byte gzipMode )  {
+int  CMMDBFile::AddPDBASCII ( cpstr PDBFileName, byte gzipMode ) {
 int   RC;
 CFile f;
   //  open the file as ASCII for reading
@@ -1610,12 +1535,17 @@ int  CMMDBFile::GetNumberOfNCSMatrices()  {
   return Cryst.GetNumberOfNCSMatrices();
 }
 
+int  CMMDBFile::GetNumberOfNCSMates()  {
+// Returns the number of NCS mates not given in the file (iGiven==0)
+  return Cryst.GetNumberOfNCSMates();
+}
+
 Boolean  CMMDBFile::GetNCSMatrix ( int NCSMatrixNo, // 0..N-1
                                    mat44 & ncs_m, int & iGiven )  {
   return Cryst.GetNCSMatrix ( NCSMatrixNo,ncs_m,iGiven );
 }
 
-int  CMMDBFile::ReadPDBAtom ( const pstr L )  {
+int  CMMDBFile::ReadPDBAtom ( cpstr L )  {
 //   If string L belongs to the coordinate section
 // (records ATOM, SIGATM, ANISOU, SIGUIJ, TER, HETATM),
 // the correspondent information is retrieved and
@@ -1798,11 +1728,13 @@ int i,kndex,RC;
 
 
 int CMMDBFile::PutAtom ( int    index,  // same meaning as above
-                         PCAtom A,      // pointer to completed atom class
-                         int    serNum  // 0 means that the serial number
-                                        // will be set equal to index.
-                                        // Otherwise the serial number
-                                        // is set to the specified value
+                         PCAtom A,      // pointer to completed atom
+                                        // class
+                         int    serNum  // 0 means that the serial
+                                        // number will be set equal
+                                        // to "index". Otherwise,
+                                        // the serial number is set
+                                        // to the specified value
                        )  {
 int i,kndex,RC,sn;
 
@@ -1857,7 +1789,8 @@ int i,kndex,RC,sn;
 }
 
 int CMMDBFile::CheckInAtom ( int index, // same meaning as above
-                             PCAtom  A  // pointer to completed atom class
+                             PCAtom  A  // pointer to completed
+                                        // atom class
                            )  {
 int i,kndex;
 
@@ -1977,7 +1910,7 @@ int     i,j,k,k1,kndex;
 }
 
 
-int CMMDBFile::SwitchModel ( const pstr L )  {
+int CMMDBFile::SwitchModel ( cpstr L )  {
 int nM;
 
   if (!GetInteger(nM,&(L[10]),4))
@@ -1988,13 +1921,16 @@ int nM;
 }
 
 int CMMDBFile::SwitchModel ( int nM )  {
-int      i;
 PPCModel Mdl;
+int      i;
+Boolean  Transfer;
 
   if (nM<=0)
     return Error_WrongModelNo;
 
   if (nM>nModels)  {
+    if ((nModels==1) && Model[0])  Transfer = (nAtoms<=0);
+                             else  Transfer = False;
     Mdl = new PCModel[nM];
     for (i=0;i<nModels;i++)
       Mdl[i] = Model[i];
@@ -2003,12 +1939,15 @@ PPCModel Mdl;
     if (Model) delete[] Model;
     Model   = Mdl;
     nModels = nM;
+    if (Transfer)  {
+      Model[nM-1] = Model[0];
+      Model[0]    = NULL;
+    }
   }
 
-  if (!Model[nM-1])  {
+  if (!Model[nM-1])
     Model[nM-1] = newCModel();
-    Model[nM-1]->SetMMDBManager ( PCMMDBManager(this),nM );
-  }
+  Model[nM-1]->SetMMDBManager ( PCMMDBManager(this),nM );
 
   crModel = Model[nM-1];
   crChain = NULL;  // new model - new chain
@@ -2018,7 +1957,7 @@ PPCModel Mdl;
 
 }
 
-int CMMDBFile::CheckAtomPlace ( int index, const pstr L )  {
+int CMMDBFile::CheckAtomPlace ( int index, cpstr L )  {
 //   This function gets the residue/chain information stored
 // in PDB string L (the records should start with the
 // keywords ATOM, SIGATM, ANISOU, SIGUIJ, TER, HETATM) and
@@ -2051,7 +1990,11 @@ InsCode  insCode;
   }
 
   // get chain ID
-  if (L[21]!=' ')  {
+  if (L[20]!=' ')  {
+    chainID[0] = L[20];
+    chainID[1] = L[21];
+    chainID[2] = char(0);
+  } else if (L[21]!=' ')  {
     chainID[0] = L[21];
     chainID[1] = char(0);
   } else
@@ -2170,7 +2113,7 @@ int  CMMDBFile::AllocateAtom ( int           index,
   if (!crRes)  {
     // either the chain or residue was changed -- get a new residue
     crRes = crChain->GetResidueCreate ( resName,seqNum,insCode,
-                                        Flags & MMDBF_IgnoreDuplSeqNum );
+                                      Flags & MMDBF_IgnoreDuplSeqNum );
     if (!crRes)  return  Error_DuplicateSeqNum;
   }
 
@@ -2230,14 +2173,14 @@ int     i;
 }
 
 
-int  CMMDBFile::WritePDBASCII1 ( const pstr PDBLFName, byte gzipMode )  {
+int  CMMDBFile::WritePDBASCII1 ( cpstr PDBLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( PDBLFName );
   if (FName)  return WritePDBASCII ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::WritePDBASCII ( const pstr PDBFileName, byte gzipMode )  {
+int  CMMDBFile::WritePDBASCII ( cpstr PDBFileName, byte gzipMode )  {
 CFile f;
 
   //  opening it in pseudo-text mode ensures that the line terminators
@@ -2263,10 +2206,17 @@ int  i;
 
   Title.PDBASCIIDump ( f );
 
-  if (nModels>0)  {
-    if (Model[0])
-      Model[0]->PDBASCIIDumpPS ( f );
-  }
+  i = 0;
+  while (i<nModels)
+    if (Model[i])  break;
+             else  i++;
+  if (i<nModels)
+    Model[i]->PDBASCIIDumpPS ( f );
+
+  // output cispep records
+  for (i=0;i<nModels;i++)
+    if (Model[i])
+      Model[i]->PDBASCIIDumpCP ( f );
 
   SA      .PDBASCIIDump ( f );
   Footnote.PDBASCIIDump ( f );
@@ -2277,21 +2227,21 @@ int  i;
     if (Model[i])
       Model[i]->PDBASCIIDump ( f );
 
-  SC   .PDBASCIIDump ( f );
+  SC.PDBASCIIDump ( f );
 
   f.WriteLine ( pstr("END") );
 
 }
 
 
-int  CMMDBFile::WriteCIFASCII1 ( const pstr CIFLFName, byte gzipMode )  {
+int  CMMDBFile::WriteCIFASCII1 ( cpstr CIFLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( CIFLFName );
   if (FName)  return WriteCIFASCII ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::WriteCIFASCII ( const pstr CIFFileName, byte gzipMode )  {
+int  CMMDBFile::WriteCIFASCII ( cpstr CIFFileName, byte gzipMode )  {
 int  i;
 
   if (!CIF)  CIF = new CMMCIFData();
@@ -2333,14 +2283,14 @@ PCAtom  CMMDBFile::GetAtomI ( int index )  {
 #define MMDBFLabel  "**** This is MMDB binary file ****"
 #define Edition     1
 
-int  CMMDBFile::ReadMMDBF1 ( const pstr MMDBLFName, byte gzipMode )  {
+int  CMMDBFile::ReadMMDBF1 ( cpstr MMDBLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( MMDBLFName );
   if (FName)  return ReadCoorFile ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::ReadMMDBF ( const pstr MMDBFileName, byte gzipMode )  {
+int  CMMDBFile::ReadMMDBF ( cpstr MMDBFileName, byte gzipMode )  {
 char  Label[100];
 byte  Version;
 CFile f;
@@ -2368,14 +2318,14 @@ CFile f;
 }
 
 
-int  CMMDBFile::WriteMMDBF1 ( const pstr MMDBLFName, byte gzipMode )  {
+int  CMMDBFile::WriteMMDBF1 ( cpstr MMDBLFName, byte gzipMode )  {
 pstr FName;
   FName = getenv ( MMDBLFName );
   if (FName)  return WriteMMDBF ( FName,gzipMode );
         else  return Error_NoLogicalName;
 }
 
-int  CMMDBFile::WriteMMDBF ( const pstr MMDBFileName, byte gzipMode )  {
+int  CMMDBFile::WriteMMDBF ( cpstr MMDBFileName, byte gzipMode )  {
 char  Label[100];
 byte  Version=Edition;
 CFile f;
@@ -2404,7 +2354,7 @@ void  CMMDBFile::SetEntryID ( const IDCode idCode )  {
   strcpy ( Title.idCode,idCode );
 }
 
-void CMMDBFile::SetSyminfoLib ( const pstr syminfo_lib )  {
+void CMMDBFile::SetSyminfoLib ( cpstr syminfo_lib )  {
   Cryst.SetSyminfoLib ( syminfo_lib );
 }
 
@@ -2412,7 +2362,7 @@ pstr CMMDBFile::GetSyminfoLib()  {
   return Cryst.GetSyminfoLib();
 }
 
-int CMMDBFile::SetSpaceGroup ( const pstr spGroup )  {
+int CMMDBFile::SetSpaceGroup ( cpstr spGroup )  {
   return Cryst.SetSpaceGroup ( spGroup );
 }
 
@@ -2574,25 +2524,35 @@ int CMMDBFile::GetFractMatrix ( mat44 & TMatrix, int Nop,
                                 cellshift_c,NULL );
 }
 
+int  CMMDBFile::GetSymOpMatrix ( mat44 & TMatrix, int Nop )  {
+//
+//  GetSymOpMatrix(..) returns the transformation matrix for
+//  Nop-th symmetry operator in the space group
+//
+//  Return 0 means everything's fine,
+//         1 there's no symmetry operation Nop defined
+//         2 fractionalizing/orthogonalizing matrices were not
+//           calculated
+//         3 cell parameters were not set up.
+//
+  return Cryst.GetSymOpMatrix ( TMatrix,Nop );
+}
 
 //  -------------  User-Defined Data  ------------------------
 
-int  CMMDBFile::RegisterUDInteger ( int udr_type,
-                                    const pstr UDDataID )  {
+int  CMMDBFile::RegisterUDInteger ( int udr_type, cpstr UDDataID )  {
   return UDRegister.RegisterUDInteger ( udr_type,UDDataID );
 }
 
-int  CMMDBFile::RegisterUDReal ( int udr_type,
-                                 const pstr UDDataID )  {
+int  CMMDBFile::RegisterUDReal ( int udr_type, cpstr UDDataID )  {
   return UDRegister.RegisterUDReal ( udr_type,UDDataID );
 }
 
-int  CMMDBFile::RegisterUDString ( int udr_type,
-                                   const pstr UDDataID )  {
+int  CMMDBFile::RegisterUDString ( int udr_type, cpstr UDDataID )  {
   return UDRegister.RegisterUDString ( udr_type,UDDataID );
 }
 
-int  CMMDBFile::GetUDDHandle ( int udr_type, const pstr UDDataID )  {
+int  CMMDBFile::GetUDDHandle ( int udr_type, cpstr UDDataID )  {
   return UDRegister.GetUDDHandle ( udr_type,UDDataID );
 }
 
@@ -2713,7 +2673,7 @@ int  CMMDBFile::PutUDData ( int UDDhandle, realtype rudd )  {
   else  return  UDDATA_WrongUDRType;
 }
 
-int  CMMDBFile::PutUDData ( int UDDhandle, const pstr sudd )  {
+int  CMMDBFile::PutUDData ( int UDDhandle, cpstr sudd )  {
   if (UDDhandle & UDRF_HIERARCHY)
         return  CUDData::putUDData ( UDDhandle,sudd );
   else  return  UDDATA_WrongUDRType;
@@ -2859,7 +2819,7 @@ MakeStreamFunctions(CMMDBFile)
 
 
 
-int isMMDBBIN ( const pstr FName, byte gzipMode )  {
+int isMMDBBIN ( cpstr FName, byte gzipMode )  {
 char  Label[100];
 byte  Version;
 CFile f;
@@ -2885,8 +2845,7 @@ CFile f;
 }
 
 
-int isPDB ( const pstr FName, byte gzipMode,
-            Boolean IgnoreBlankLines )  {
+int isPDB ( cpstr FName, byte gzipMode, Boolean IgnoreBlankLines )  {
 CFile   f;
 char    S[256];
 int     i;
@@ -2964,6 +2923,7 @@ Boolean Done;
     if (!strncasecmp(S,"CONECT",6))  return 0;
     if (!strncasecmp(S,"MASTER",6))  return 0;
     if (!strncasecmp(S,"END   ",6))  return 0;
+    if (!strncasecmp(S,"USER  ",6))  return 0;
     return  1;
   } else
     return -1;
