@@ -1,53 +1,28 @@
-//  $Id: mmdb_file.h,v 1.18 2005/09/22 16:32:38 keb Exp $
+//  $Id: mmdb_file.h,v 1.24 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    07.06.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CMMDBFile
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2000-2004
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -117,6 +92,7 @@
 #define MMDBF_IgnoreRemarks           0x00002000
 #define MMDBF_AllowDuplChainID        0x00004000
 #define MMDBF_FixSpaceGroup           0x00008000
+#define MMDBF_EnforceAtomNames        0x00010000
 
 // file types:
 #define MMDB_FILE_Undefined     -1
@@ -134,10 +110,11 @@
 #define PDBCLEAN_SERIAL         0x00000040
 #define PDBCLEAN_SEQNUM         0x00000080
 #define PDBCLEAN_CHAIN_ORDER    0x00000100
-#define PDBCLEAN_SOLVENT        0x00000200
-#define PDBCLEAN_INDEX          0x00000400
-#define PDBCLEAN_ELEMENT        0x00000800
-#define PDBCLEAN_ELEMENT_STRONG 0x00001000
+#define PDBCLEAN_CHAIN_ORDER_IX 0x00000200
+#define PDBCLEAN_SOLVENT        0x00000400
+#define PDBCLEAN_INDEX          0x00000800
+#define PDBCLEAN_ELEMENT        0x00001000
+#define PDBCLEAN_ELEMENT_STRONG 0x00002000
 
 
 // crystallographic info inquery
@@ -178,48 +155,48 @@ class CMMDBFile : public CUDData  {
     void  SetFlag        ( word Flag );
     void  RemoveFlag     ( word Flag );
 
-    int   ReadPDBASCII   ( const pstr PDBFileName,
+    int   ReadPDBASCII   ( cpstr PDBFileName,
                            byte gzipMode=GZM_CHECK );
-    int   ReadPDBASCII1  ( const pstr PDBLFName,
+    int   ReadPDBASCII1  ( cpstr PDBLFName,
                            byte gzipMode=GZM_CHECK );
     int   ReadPDBASCII   ( RCFile f );
 
-    int   ReadCIFASCII   ( const pstr CIFFileName,
+    int   ReadCIFASCII   ( cpstr CIFFileName,
                            byte gzipMode=GZM_CHECK );
-    int   ReadCIFASCII1  ( const pstr CIFLFName,
+    int   ReadCIFASCII1  ( cpstr CIFLFName,
                            byte gzipMode=GZM_CHECK );
     int   ReadFromCIF    ( PCMMCIFData CIFD, Boolean fixSpaceGroup );
 
     // adds info from PDB file
-    int   AddPDBASCII1   ( const pstr PDBLFName,
+    int   AddPDBASCII1   ( cpstr PDBLFName,
                            byte gzipMode=GZM_CHECK );
-    int   AddPDBASCII    ( const pstr PDBFileName,
+    int   AddPDBASCII    ( cpstr PDBFileName,
                            byte gzipMode=GZM_CHECK );
 
     // auto format recognition
-    int   ReadCoorFile   ( const pstr LFName,
+    int   ReadCoorFile   ( cpstr LFName,
                            byte gzipMode=GZM_CHECK );
-    int   ReadCoorFile1  ( const pstr CFName,
+    int   ReadCoorFile1  ( cpstr CFName,
                            byte gzipMode=GZM_CHECK );
 
-    int   WritePDBASCII  ( const pstr PDBFileName,
+    int   WritePDBASCII  ( cpstr PDBFileName,
                            byte gzipMode=GZM_CHECK );
-    int   WritePDBASCII1 ( const pstr PDBLFName,
+    int   WritePDBASCII1 ( cpstr PDBLFName,
                            byte gzipMode=GZM_CHECK );
     void  WritePDBASCII  ( RCFile f );
 
-    int   WriteCIFASCII  ( const pstr CIFFileName,
+    int   WriteCIFASCII  ( cpstr CIFFileName,
                            byte gzipMode=GZM_CHECK );
-    int   WriteCIFASCII1 ( const pstr CIFLFName,
+    int   WriteCIFASCII1 ( cpstr CIFLFName,
                            byte gzipMode=GZM_CHECK );
     
-    int   ReadMMDBF      ( const pstr MMDBFileName,
+    int   ReadMMDBF      ( cpstr MMDBFileName,
                            byte gzipMode=GZM_CHECK );
-    int   ReadMMDBF1     ( const pstr MMDBLFName,
+    int   ReadMMDBF1     ( cpstr MMDBLFName,
                            byte gzipMode=GZM_CHECK );
-    int   WriteMMDBF     ( const pstr MMDBFileName,
+    int   WriteMMDBF     ( cpstr MMDBFileName,
                            byte gzipMode=GZM_CHECK );
-    int   WriteMMDBF1    ( const pstr MMDBLFName,
+    int   WriteMMDBF1    ( cpstr MMDBLFName,
                            byte gzipMode=GZM_CHECK );
 
     void  GetInputBuffer ( pstr Line, int & count );
@@ -229,12 +206,12 @@ class CMMDBFile : public CUDData  {
     // is namely added meaning that it will be the
     // last REMARK, last JRNL, last ATOM etc. string
     // -- always the last one in its group.
-    int   PutPDBString   ( const pstr PDBString );
+    int   PutPDBString   ( cpstr PDBString );
 
 
 
     //  PDBCleanup(..) cleans coordinate part to comply with PDB
-    // standards:
+    // standards and MMDB "expectations":
     //
     //  PDBCLEAN_ATNAME  pads atom names with spaces to form
     //                   4-symbol names
@@ -254,6 +231,8 @@ class CMMDBFile : public CUDData  {
     //                   incrementally-by-one without insertion codes
     //  PDBCLEAN_CHAIN_ORDER puts chains in order of atom's serial
     //                   numbers
+    //  PDBCLEAN_CHAIN_ORDER_IX puts chains in order of atom's
+    //                   indices internal to MMDB
     //  PDBCLEAN_SOLVENT moves solvent chains at the end of each model
     //
     //  Return codes (as bits):
@@ -272,7 +251,7 @@ class CMMDBFile : public CUDData  {
     //  when converting to 'ordinary' atoms. This may cause
     //  disorder in serial numbers -- just call
     //  PDBClean(PDBCLEAN_SERIAL) when necessary to fix this.
-    void  MakeHetAtoms ( const pstr chainID, Boolean Make );
+    void  MakeHetAtoms ( cpstr chainID, Boolean Make );
 
     //  ---------------  Working with atoms by serial numbers  ---
 
@@ -375,6 +354,9 @@ class CMMDBFile : public CUDData  {
     Boolean isScaleMatrix ();  // SCALEx PDB records
     Boolean isNCSMatrix   ();  // MTRIXx PDB records
     int     GetNumberOfNCSMatrices();
+    int     GetNumberOfNCSMates   ();  // Returns the number of
+                                       // NCS mates not given in
+                                       // the file (iGiven==0)
     Boolean GetNCSMatrix  ( int NCSMatrixNo, // 0..N-1
                             mat44 & ncs_m, int & iGiven );
 
@@ -384,16 +366,16 @@ class CMMDBFile : public CUDData  {
 
     //  -------------  User-Defined Data  ------------------------
 
-    int RegisterUDInteger ( int udr_type, const pstr UDDataID );
-    int RegisterUDReal    ( int udr_type, const pstr UDDataID );
-    int RegisterUDString  ( int udr_type, const pstr UDDataID );
-    int GetUDDHandle      ( int udr_type, const pstr UDDataID );
+    int RegisterUDInteger ( int udr_type, cpstr UDDataID );
+    int RegisterUDReal    ( int udr_type, cpstr UDDataID );
+    int RegisterUDString  ( int udr_type, cpstr UDDataID );
+    int GetUDDHandle      ( int udr_type, cpstr UDDataID );
 
     //  ----------------------------------------------------------
 
-    void  SetSyminfoLib ( const pstr syminfo_lib );
+    void  SetSyminfoLib ( cpstr syminfo_lib );
     pstr  GetSyminfoLib ();
-    int   SetSpaceGroup ( const pstr spGroup );
+    int   SetSpaceGroup ( cpstr spGroup );
     pstr  GetSpaceGroup ();
     pstr  GetSpaceGroupFix();
 
@@ -488,6 +470,18 @@ class CMMDBFile : public CUDData  {
                          int cellshift_c );
 
 
+    //  GetSymOpMatrix(..) returns the transformation matrix for
+    //  Nop-th symmetry operator in the space group
+    //
+    //  Return 0 means everything's fine,
+    //         1 there's no symmetry operation Nop defined
+    //         2 fractionalizing/orthogonalizing matrices were not
+    //           calculated
+    //         3 cell parameters were not set up.
+    //
+    int GetSymOpMatrix ( mat44 & TMatrix, int Nop );
+
+
     int   AddNCSMatrix    ( mat33 & ncs_m, vect3 & ncs_v, int iGiven );
     int   GenerateNCSMates(); // 1: no NCS matrices, 0: Ok
 
@@ -504,9 +498,9 @@ class CMMDBFile : public CUDData  {
     void  SetShortBinary();  // leaves only coordinates in binary files
 
     // -------  user-defined data handlers
-    int   PutUDData ( int UDDhandle, int        iudd );
-    int   PutUDData ( int UDDhandle, realtype   rudd );
-    int   PutUDData ( int UDDhandle, const pstr sudd );
+    int   PutUDData ( int UDDhandle, int      iudd );
+    int   PutUDData ( int UDDhandle, realtype rudd );
+    int   PutUDData ( int UDDhandle, cpstr    sudd );
 
     int   GetUDData ( int UDDhandle, int      & iudd );
     int   GetUDData ( int UDDhandle, realtype & rudd );
@@ -569,11 +563,11 @@ class CMMDBFile : public CUDData  {
     void  InitMMDBFile    ();
     void  FreeCoordMemory ();
     void  ReadPDBLine     ( RCFile f, pstr L, int maxlen );
-    int   ReadPDBAtom     ( const pstr L );
+    int   ReadPDBAtom     ( cpstr L );
     int   ReadCIFAtom     ( PCMMCIFData CIFD   );
-    int   CheckAtomPlace  ( int  index, const pstr L );
+    int   CheckAtomPlace  ( int  index, cpstr L );
     int   CheckAtomPlace  ( int  index, PCMMCIFLoop Loop );
-    int   SwitchModel     ( const pstr L );
+    int   SwitchModel     ( cpstr L );
     int   SwitchModel     ( int nM );
     int   AllocateAtom    ( int           index,
                             const ChainID chainID,
@@ -611,13 +605,13 @@ class CMMDBFile : public CUDData  {
 //     2   if file FName is likely a MMDB BIN (binary) file,
 //         but of a wrong edition (i.e. produced by a lower
 //         version of MMDB).
-extern int isMMDBBIN ( const pstr FName, byte gzipMode=GZM_CHECK );
+extern int isMMDBBIN ( cpstr FName, byte gzipMode=GZM_CHECK );
 
 //  isPDB will return
 //    -1   if file FName does not exist
 //     0   if file FName is likely a PDB file
 //     1   if file FName is not a PDB file
-extern int isPDB ( const pstr FName, byte gzipMode=GZM_CHECK,
+extern int isPDB ( cpstr FName, byte gzipMode=GZM_CHECK,
                    Boolean IgnoreBlankLines=False );
 
 #endif
