@@ -1,53 +1,28 @@
-//  $Id: mmdb_graph.cpp,v 1.12 2005/12/19 16:51:28 keb Exp $
+//  $Id: mmdb_graph.cpp,v 1.18 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    03.02.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +34,7 @@
 //                  CMatch      ( match of structural graphs         )
 //                  CGraphMatch ( CSIA algorithms for graph matching )
 //
-//    E. Krissinel 2001-2004
+//   (C) E. Krissinel 2000-2008
 //
 //  When used, please cite:
 //
@@ -97,7 +72,7 @@ CVertex::CVertex ( RPCStream Object ) : CStream(Object)  {
   InitVertex();
 }
 
-CVertex::CVertex ( const pstr chem_elem )
+CVertex::CVertex ( cpstr chem_elem )
        : CStream()  {
   InitVertex();
   SetVertex ( chem_elem );
@@ -109,13 +84,13 @@ CVertex::CVertex ( int vtype )
   SetVertex ( vtype );
 }
 
-CVertex::CVertex ( int vtype, const pstr vname )
+CVertex::CVertex ( int vtype, cpstr vname )
        : CStream()  {
   InitVertex();
   SetVertex ( vtype,vname );
 }
 
-CVertex::CVertex ( const pstr chem_elem, const pstr vname )
+CVertex::CVertex ( cpstr chem_elem, cpstr vname )
        : CStream()  {
   InitVertex ();
   SetVertex  ( chem_elem  );
@@ -134,7 +109,25 @@ void  CVertex::InitVertex()  {
   userid   = 0;
 }
 
-void  CVertex::SetVertex ( const pstr chem_elem )  {
+
+void  CVertex::SetVertex ( cpstr chem_elem )  {
+//   This function generates vertex type according to a chemical
+// element name passed in chem_elem.
+//
+//   The element type has the following meaning:
+//
+//     0xCHSSTTTT
+//
+// where
+//        T - resreved for elemenmt type
+//        S - resreved for symmetry relief
+//        H - reserved for hydrogen bonds
+//        C - resreved for chirality flags
+//
+// Note that when more than 2 symbols are used for chemical element
+// name (custom use), fields S may be reallocated for element type
+// and then symmetry relief becomes impossible.
+//
   CreateCopy ( name,chem_elem );
   type = getElementNo ( chem_elem );
   if (type==ELEMENT_UNKNOWN)  {
@@ -151,6 +144,8 @@ void  CVertex::SetVertex ( const pstr chem_elem )  {
 }
 
 void  CVertex::SetVertex ( int vtype )  {
+//  This function sets vertex type. See comments above for
+// the general rule for vertex types implied by this code.
 char N[50];
 int  type0;
   type  = vtype;
@@ -167,12 +162,12 @@ void  CVertex::SetType ( int vtype )  {
   type = vtype;
 }
 
-void  CVertex::SetVertex ( int vtype, const pstr vname )  {
+void  CVertex::SetVertex ( int vtype, cpstr vname )  {
   type = vtype;
   CreateCopy ( name,vname );
 }
 
-void  CVertex::SetName ( const pstr vname )  {
+void  CVertex::SetName ( cpstr vname )  {
   CreateCopy ( name,vname );
 }
 
@@ -266,7 +261,7 @@ void  CVertex::mem_write ( pstr S, int & l )  {
   ::mem_write ( userid  ,S,l );
 }
 
-void  CVertex::mem_read ( const pstr S, int & l )  {
+void  CVertex::mem_read ( cpstr S, int & l )  {
   ::mem_read ( name    ,S,l );
   ::mem_read ( type    ,S,l );
   ::mem_read ( property,S,l );
@@ -311,7 +306,7 @@ static pstr BondType[NofBondTypes+1] = {
 };
 
 
-void  CEdge::SetEdge ( int vx1, int vx2, const pstr btype )  {
+void  CEdge::SetEdge ( int vx1, int vx2, cpstr btype )  {
   v1   = vx1;
   v2   = vx2;
   type = 0;
@@ -388,7 +383,7 @@ void  CEdge::mem_write ( pstr S, int & l )  {
   ::mem_write ( property,S,l );
 }
 
-void  CEdge::mem_read ( const pstr S, int & l )  {
+void  CEdge::mem_read ( cpstr S, int & l )  {
   ::mem_read ( v1      ,S,l );
   ::mem_read ( v2      ,S,l );
   ::mem_read ( type    ,S,l );
@@ -406,7 +401,7 @@ CGraph::CGraph() : CStream()  {
   InitGraph();
 }
 
-CGraph::CGraph ( PCResidue R, const pstr altLoc ) : CStream()  {
+CGraph::CGraph ( PCResidue R, cpstr altLoc ) : CStream()  {
   InitGraph();
   MakeGraph ( R,altLoc );
 }
@@ -472,7 +467,7 @@ void  CGraph::Reset()  {
   CreateCopy ( name,pstr("UNNAMED") );  
 }
 
-void  CGraph::SetName ( const pstr gname )  {
+void  CGraph::SetName ( cpstr gname )  {
   CreateCopy ( name,gname );
 }
 
@@ -607,8 +602,7 @@ void  CGraph::GetEdges ( PPCEdge & E, int & nE )  {
 }
 
 
-
-int  CGraph::MakeGraph ( PCResidue R, const pstr altLoc )  {
+int  CGraph::MakeGraph ( PCResidue R, cpstr altLoc )  {
 int      i,j, a1,a2,e1,e2, nAltLocs,alflag, rc;
 Boolean  B;
 rvector  occupancy;
@@ -715,13 +709,14 @@ PCEdge   G;
     e1--;
     for (j=i+1;j<nVertices;j++)  {
       a2 = Vertex[j]->id;
-      e2 = Vertex[i]->type;
+      e2 = Vertex[j]->type;
       if (e2>nElementNames)  e2 = 6;
       e2--;
       dx = R->atom[a2]->x - R->atom[a1]->x;
       dy = R->atom[a2]->y - R->atom[a1]->y;
       dz = R->atom[a2]->z - R->atom[a1]->z;
-      sr = CovalentRadius[e1] + CovalentRadius[e2] + 0.15;
+//      sr = CovalentRadius[e1] + CovalentRadius[e2] + 0.15;
+      sr = CovalentRadius[e1] + CovalentRadius[e2] + 0.25;
       if (dx*dx+dy*dy+dz*dz<sr*sr)  {  // it's a bond
         G = new CEdge(i+1,j+1,1);
         AddEdge ( G );
@@ -880,6 +875,81 @@ int i;
     if (Vertex[i])  Vertex[i]->LeaveChirality ( eltype );
 }
 
+void CGraph::MakeSymmetryRelief ( Boolean noCO2 )  {
+//  This function looks for groups of equivalent vertices
+// attached to a single vertice (e.g. chemical SO3 or
+// PO3 groups), and re-lables them by adding a unique
+// symmetry-relief number. This eliminates equivalent
+// matches (3! for each SO3/PO3 group), and increases
+// vertex diversity, which considerably speeds up matching.
+// The function is cheap and harmless even if such groups
+// of vertices are not found.
+//  If noCO2 is True then CO2 symmetry is not releaved.
+ivector v,vc;
+int     i,j,k,n,m,almask,vjtype, ctype,otype;
+Boolean noOxygens;
+
+  GetVectorMemory ( v ,nVertices,0 );
+  GetVectorMemory ( vc,nVertices,1 );
+
+  for (i=1;i<=nVertices;i++)
+    vc[i] = 0;
+
+  for (j=0;j<nEdges;j++)  {
+    if ((Edge[j]->v1>0) && (Edge[j]->v1<=nVertices))
+      vc[Edge[j]->v1]++;
+    if ((Edge[j]->v2>0) && (Edge[j]->v2<=nVertices))
+      vc[Edge[j]->v2]++;
+  }
+
+  almask = ~ATOM_LEAVING;
+
+  if (noCO2)  {
+    ctype = getElementNo ( "C" );
+    otype = getElementNo ( "O" );
+  }
+
+  noOxygens = False;
+
+  for (i=1;i<=nVertices;i++)
+    if (vc[i]>1)  {  // vertex at more than 1 edge
+      // v[] will list connected vertices, k will be their number
+      k = 0;
+      for (j=0;j<nEdges;j++)  {
+        if ((Edge[j]->v1==i) && (vc[Edge[j]->v2]==1) && (k<nVertices))
+          v[k++] = Edge[j]->v2-1;
+        if ((Edge[j]->v2==i) && (vc[Edge[j]->v1]==1) && (k<nVertices))
+          v[k++] = Edge[j]->v1-1;
+      }
+      if (k>1)  {
+        if (noCO2) noOxygens = ((Vertex[i-1]->type & almask)==ctype);
+        // A group of vertices with single connection is
+        // identified. Assign symmetry relief modifiers
+        // to *equivalent* vertices in the group
+        for (j=0;j<k;j++)
+          if ((v[j]>=0) && (v[j]<nVertices))  {
+            vjtype = Vertex[v[j]]->type & almask;
+            if ((!noOxygens) || (vjtype!=otype))  {
+              n = 1; // symmetry relief modifier
+              for (m=j+1;m<k;m++)
+                if ((v[m]>=0) && (v[m]<nVertices))  {
+                  if (Vertex[v[j]]->type==
+                      (Vertex[v[m]]->type & almask))  {
+                    Vertex[v[m]]->type |= (n << 16);
+                    n++;
+                    v[m] = -1;
+                  }
+                }
+            }
+          }
+      }
+    }
+
+  FreeVectorMemory ( v ,0 );
+  FreeVectorMemory ( vc,1 );
+
+}
+
 int  CGraph::Build ( Boolean bondOrder )  {
 int i,j, rc;
 
@@ -1082,7 +1152,7 @@ Boolean bondOrder=False;
     }
 }
 
-void  CGraph::mem_read ( const pstr S, int & l )  {
+void  CGraph::mem_read ( cpstr S, int & l )  {
 int     i,k;
 Boolean bondOrder;
 
@@ -1285,7 +1355,7 @@ int i;
   }
 }
 
-void CMatch::mem_read ( const pstr S, int & l )  {
+void CMatch::mem_read ( cpstr S, int & l )  {
 int i;
   FreeVectorMemory ( F1,1 );
   FreeVectorMemory ( F2,1 );
@@ -1505,7 +1575,6 @@ int  n1;
 
   }
 
-
 }
 
 
@@ -1520,6 +1589,37 @@ int     i,j,v1type,almask,iW,pl;
     jF1[i] = i;
 
   almask = ~ATOM_LEAVING;
+
+/*  -- experiment for symmetry reliefs
+int v2type,v1type_sr,srmask;
+  srmask = ~SYMREL_MASK;
+
+  for (i=1;i<=n;i++)  {
+    if (vertexType) {
+      ix[i]  = 0;
+      v1type = V1[i-1]->type & almask;
+      v1type_sr = v1type & srmask;
+      pl     = 0;
+      for (j=1;j<=m;j++)  {
+        v2type = V2[j-1]->type & almask;
+        if ((v1type==v2type) ||
+            (v1type_sr==v2type) ||
+            (v1type==(v2type & srmask)))
+          P[1][i][++pl] = j;
+      }
+      P[1][i][0] = pl;
+      if (pl)  ix[i] = i;
+    } else {
+      ix[i] = i;
+      for (j=1;j<=m;j++)
+        P[1][i][j] = j;
+      P[1][i][0] = m;
+    }
+    F1[i] = 0;
+    F2[i] = 0;
+  }
+ */
+
   for (i=1;i<=n;i++)  {
     if (vertexType) {
       ix[i]  = 0;
@@ -1963,7 +2063,7 @@ int i;
     Match[i]->mem_write ( S,l );
 }
 
-void  CGraphMatch::mem_read ( const pstr S, int & l )  {
+void  CGraphMatch::mem_read ( cpstr S, int & l )  {
 int i;
   FreeMemory ();
   ::mem_read ( nMatches,S,l );
