@@ -1,53 +1,28 @@
-//  $Id: mmdb_mmcif.cpp,v 1.8 2004/03/26 15:00:58 keb Exp $
+//  $Id: mmdb_mmcif.cpp,v 1.11 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    19.03.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -61,7 +36,7 @@
 //                  CMMCIFData     ( mmCIF data block  )
 //                  CMMCIFFile     ( mmCIF file        )
 //
-//  E. Krissinel 2001-2004
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -263,7 +238,7 @@ CMMCIFCategory::CMMCIFCategory() : CStream()  {
   InitMMCIFCategory();
 }
 
-CMMCIFCategory::CMMCIFCategory ( const pstr N ) : CStream()  {
+CMMCIFCategory::CMMCIFCategory ( cpstr N ) : CStream()  {
   InitMMCIFCategory();
   if (N[0])  CreateCopy ( name,N );
   else  {
@@ -272,7 +247,8 @@ CMMCIFCategory::CMMCIFCategory ( const pstr N ) : CStream()  {
   }
 }
 
-CMMCIFCategory::CMMCIFCategory ( RPCStream Object ) : CStream(Object)  {
+CMMCIFCategory::CMMCIFCategory ( RPCStream Object )
+              : CStream(Object)  {
   InitMMCIFCategory();
 }
 
@@ -371,7 +347,7 @@ psvector tag1;
   }
 }
 
-int  CMMCIFCategory::GetTagNo ( const pstr ttag )  {
+int  CMMCIFCategory::GetTagNo ( cpstr ttag )  {
 //   Binary search for index of tag ttag in tag[].
 // Return:
 //    >=0 : position of the tag found
@@ -405,7 +381,7 @@ int l1,l2,l,k;
   return -2-l1;                  // would be at l1+1=(-RC-1)th position
 }
 
-int  CMMCIFCategory::AddTag ( const pstr ttag )  {
+int  CMMCIFCategory::AddTag ( cpstr ttag )  {
 //  return -1: the tag has been added on the top of array;
 //             index is added and sorted automatically
 //        >=0: the tag is already in the array -- its position
@@ -456,7 +432,7 @@ int i;
   return True;
 }
 
-void  CMMCIFCategory::PutCategoryName ( const pstr newName )  {
+void  CMMCIFCategory::PutCategoryName ( cpstr newName )  {
   CreateCopy ( name,newName );
 }
 
@@ -517,7 +493,7 @@ CMMCIFStruct::CMMCIFStruct() : CMMCIFCategory() {
   InitMMCIFStruct();
 }
 
-CMMCIFStruct::CMMCIFStruct ( const pstr N ) : CMMCIFCategory(N) {
+CMMCIFStruct::CMMCIFStruct ( cpstr N ) : CMMCIFCategory(N) {
   InitMMCIFStruct();
 }
 
@@ -567,7 +543,7 @@ psvector f1;
   }
 }
 
-void CMMCIFStruct::AddField ( const pstr F, const pstr T,
+void CMMCIFStruct::AddField ( cpstr F, cpstr T,
                               Boolean Concatenate )  {
 psvector field1;
 int      i,nAT;
@@ -615,7 +591,7 @@ pstr CMMCIFStruct::GetField ( int tagNo )  {
   return NULL;
 }
 
-int  CMMCIFStruct::GetString ( pstr & S, const pstr TName,
+int  CMMCIFStruct::GetString ( pstr & S, cpstr TName,
                                Boolean Remove )  {
 int k = GetTagNo ( TName );
   if (S)  delete[] S;
@@ -636,7 +612,7 @@ int k = GetTagNo ( TName );
   return 0;
 }
 
-pstr CMMCIFStruct::GetString ( const pstr TName, int & RC )  {
+pstr CMMCIFStruct::GetString ( cpstr TName, int & RC )  {
 int k = GetTagNo ( TName );
   if (k<0)  {
     RC = CIFRC_NoTag;
@@ -655,7 +631,7 @@ int k = GetTagNo ( TName );
   return field[k];
 }
 
-int  CMMCIFStruct::DeleteField ( const pstr TName )  {
+int  CMMCIFStruct::DeleteField ( cpstr TName )  {
 int k = GetTagNo ( TName );
   if ((k>=0) && (field)) {
     if (field[k])  delete[] field[k];
@@ -664,7 +640,7 @@ int k = GetTagNo ( TName );
   return k;
 }
 
-int  CMMCIFStruct::GetReal ( realtype & R, const pstr TName,
+int  CMMCIFStruct::GetReal ( realtype & R, cpstr TName,
                              Boolean Remove )  {
 pstr endptr;
 int  RC;
@@ -686,7 +662,8 @@ int  k = GetTagNo ( TName );
   return RC;
 }
 
-int  CMMCIFStruct::GetInteger ( int & I, const pstr TName, Boolean Remove )  {
+int  CMMCIFStruct::GetInteger ( int & I, cpstr TName,
+                                Boolean Remove )  {
 pstr endptr;
 int  RC;
 int  k = GetTagNo ( TName );
@@ -711,12 +688,12 @@ int  k = GetTagNo ( TName );
 }
 
 
-void CMMCIFStruct::PutString ( const pstr S, const pstr T,
+void CMMCIFStruct::PutString ( cpstr S, cpstr T,
                                Boolean NonBlankOnly )  {
 pstr p;
   if (!S)  PutNoData ( CIF_NODATA_QUESTION,T );
   else  {
-    p = S;
+    p = pstr(S);
     if (NonBlankOnly) 
       while (*p==' ')  p++;
     if (!(*p))  PutNoData ( CIF_NODATA_DOT,T );
@@ -724,7 +701,7 @@ pstr p;
   }
 }
 
-void CMMCIFStruct::PutDate ( const pstr T )  {
+void CMMCIFStruct::PutDate ( cpstr T )  {
 time_t t;
 tm *   tstruct;
  char   S[100];
@@ -732,13 +709,13 @@ tm *   tstruct;
   tstruct = localtime(&t);
   if (tstruct)
         sprintf ( S,"%4i-%02i-%02i",
-             tstruct->tm_year+1900,tstruct->tm_mon+1,tstruct->tm_mday );
+            tstruct->tm_year+1900,tstruct->tm_mon+1,tstruct->tm_mday );
   else  strcpy  ( S,"YYYY-MM-DD" );
   AddField ( S,T,False );
 }
 
 
-void CMMCIFStruct::PutNoData ( int NoDataType, const pstr T )  {
+void CMMCIFStruct::PutNoData ( int NoDataType, cpstr T )  {
 char S[10];
   S[0] = char(2);
   if (NoDataType==CIF_NODATA_DOT)  S[1] = '.';
@@ -748,13 +725,13 @@ char S[10];
 }
 
 
-void CMMCIFStruct::PutReal ( realtype R, const pstr T, int prec )  {
+void CMMCIFStruct::PutReal ( realtype R, cpstr T, int prec )  {
 char rS[100];
   sprintf  ( rS,"%.*g",prec,R );
   AddField ( rS,T,False );
 }
 
-void CMMCIFStruct::PutInteger ( int I, const pstr T )  {
+void CMMCIFStruct::PutInteger ( int I, cpstr T )  {
 char iS[100];
   if (I>MinInt4)  {
     sprintf  ( iS,"%i",I );
@@ -768,7 +745,7 @@ char iS[100];
 #define  NODATA_Q  pstr("?")
 #define  NODATA_P  pstr(".")
 
-Boolean CMMCIFStruct::WriteMMCIFStruct ( const pstr FName,
+Boolean CMMCIFStruct::WriteMMCIFStruct ( cpstr FName,
                                          byte gzipMode )  {
 CFile f;
   f.assign ( FName,True,False,gzipMode );
@@ -881,7 +858,7 @@ CMMCIFLoop::CMMCIFLoop() : CMMCIFCategory()  {
   InitMMCIFLoop();
 }
 
-CMMCIFLoop::CMMCIFLoop ( const pstr N ) : CMMCIFCategory(N)  {
+CMMCIFLoop::CMMCIFLoop ( cpstr N ) : CMMCIFCategory(N)  {
   InitMMCIFLoop();
 }
 
@@ -952,7 +929,8 @@ psmatrix f1;
       } else
         nR++;             // count non-empty row
     }
-  if ((nT<=0) || (nR<=0))  FreeMemory();  // the loop is completely empty
+  if ((nT<=0) || (nR<=0))
+    FreeMemory();  // the loop is completely empty
   else if ((nT!=nTags) || (nR!=nAllocRows))  {
     f1 = new psvector[nR];
     m  = 0;
@@ -991,7 +969,7 @@ int i,j;
   }
 }
 
-void CMMCIFLoop::AddLoopTag ( const pstr T, Boolean Remove )  {
+void CMMCIFLoop::AddLoopTag ( cpstr T, Boolean Remove )  {
 psmatrix  f1;
 int       i,j,nT1;
   if (Remove)  {
@@ -1039,12 +1017,12 @@ psmatrix field1;
   }
 }
 
-void CMMCIFLoop::AddString ( const pstr S, Boolean NonBlankOnly )  {
+void CMMCIFLoop::AddString ( cpstr S, Boolean NonBlankOnly )  {
 int  i;
 pstr p;
   if (!S)  AddNoData ( CIF_NODATA_QUESTION );
   else  {
-    p = S;
+    p = pstr(S);
     if (NonBlankOnly)
       while (*p==' ')  p++;
     if (!(*p))  AddNoData ( CIF_NODATA_DOT );
@@ -1096,7 +1074,7 @@ pstr CMMCIFLoop::GetField ( int rowNo, int tagNo )  {
   return NULL;
 }
 
-int  CMMCIFLoop::GetString ( pstr & S, const pstr TName, int nrow,
+int  CMMCIFLoop::GetString ( pstr & S, cpstr TName, int nrow,
                              Boolean Remove)  {
 int k = GetTagNo ( TName );
   if (S)  delete[] S;
@@ -1118,7 +1096,7 @@ int k = GetTagNo ( TName );
   return 0;
 }
 
-pstr CMMCIFLoop::GetString ( const pstr TName, int nrow, int & RC )  {
+pstr CMMCIFLoop::GetString ( cpstr TName, int nrow, int & RC )  {
 int k = GetTagNo ( TName );
   if (k<0)  {
     RC = CIFRC_NoTag;
@@ -1142,7 +1120,7 @@ int k = GetTagNo ( TName );
   return field[nrow][k];
 }
 
-int CMMCIFLoop::DeleteField ( const pstr TName, int nrow )  {
+int CMMCIFLoop::DeleteField ( cpstr TName, int nrow )  {
 int k = GetTagNo ( TName );
   if (k<0)  return CIFRC_NoTag;
   if ((nrow<0) || (nrow>=nRows))
@@ -1170,7 +1148,7 @@ int i;
   return 0;
 }
 
-int  CMMCIFLoop::GetReal ( realtype & R, const pstr TName, int nrow,
+int  CMMCIFLoop::GetReal ( realtype & R, cpstr TName, int nrow,
                            Boolean Remove )  {
 pstr endptr;
 int  k = GetTagNo ( TName );
@@ -1190,7 +1168,7 @@ int  k = GetTagNo ( TName );
   return 0;
 }
 
-int  CMMCIFLoop::GetInteger ( int & I, const pstr TName, int nrow,
+int  CMMCIFLoop::GetInteger ( int & I, cpstr TName, int nrow,
                               Boolean Remove )  {
 pstr endptr;
 int  k = GetTagNo ( TName );
@@ -1214,7 +1192,7 @@ int  k = GetTagNo ( TName );
 }
 
 
-int  CMMCIFLoop::GetSVector ( psvector & S, const pstr TName,
+int  CMMCIFLoop::GetSVector ( psvector & S, cpstr TName,
                               int i1, int i2, Boolean Remove )  {
 int j,k,r1,r2;
   r1 = IMin(i1,i2);
@@ -1251,7 +1229,7 @@ int j,k,r1,r2;
   return 0;
 }
 
-int  CMMCIFLoop::GetRVector ( rvector & R, const pstr TName,
+int  CMMCIFLoop::GetRVector ( rvector & R, cpstr TName,
                               int i1, int i2, Boolean Remove )  {
 int  j,k,r1,r2,RC;
 pstr endptr;
@@ -1279,7 +1257,7 @@ pstr endptr;
   return RC;
 }
 
-int  CMMCIFLoop::GetIVector ( ivector & I, const pstr TName,
+int  CMMCIFLoop::GetIVector ( ivector & I, cpstr TName,
                               int i1, int i2, Boolean Remove )  {
 int  j,k,r1,r2,RC;
 pstr endptr;
@@ -1308,7 +1286,7 @@ pstr endptr;
 }
 
 
-void CMMCIFLoop::PutString ( const pstr S, const pstr T, int nrow )  {
+void CMMCIFLoop::PutString ( cpstr S, cpstr T, int nrow )  {
 psmatrix field1;
 int      nT,nR,iT,i,j;
   nT = nTags;
@@ -1350,7 +1328,7 @@ int      nT,nR,iT,i,j;
 }
 
 
-void CMMCIFLoop::PutNoData ( int NoDataType, const pstr T, int nrow )  {
+void CMMCIFLoop::PutNoData ( int NoDataType, cpstr T, int nrow )  {
 char S[10];
   S[0] = char(2);
   if (NoDataType==CIF_NODATA_DOT)  S[1] = '.';
@@ -1360,14 +1338,14 @@ char S[10];
 }
 
 
-void CMMCIFLoop::PutReal ( realtype R, const pstr T, int nrow,
+void CMMCIFLoop::PutReal ( realtype R, cpstr T, int nrow,
                            int prec )  {
 char rS[100];
   sprintf ( rS,"%.*g",prec,R );
   PutString ( rS,T,nrow );
 }
 
-void CMMCIFLoop::PutInteger ( int I, const pstr T, int nrow )  {
+void CMMCIFLoop::PutInteger ( int I, cpstr T, int nrow )  {
 char iS[100];
   if (I>MinInt4)  {
     sprintf ( iS,"%i",I );
@@ -1376,8 +1354,7 @@ char iS[100];
     PutNoData ( CIF_NODATA_DOT,T,nrow );
 }
 
-void CMMCIFLoop::PutSVector ( psvector S, const pstr T,
-                              int i1, int i2 )  {
+void CMMCIFLoop::PutSVector ( psvector S, cpstr T, int i1, int i2 )  {
 int i,j,k;
   PutString ( S[i2],T,i2 );
   if (iColumn==0)  k = nTags-1;
@@ -1392,7 +1369,7 @@ int i,j,k;
   }
 }
 
-void CMMCIFLoop::PutRVector ( rvector R, const pstr T,
+void CMMCIFLoop::PutRVector ( rvector R, cpstr T,
                               int i1, int i2, int prec )  {
 int  i,j,k;
 char rS[100];
@@ -1410,7 +1387,7 @@ char rS[100];
   }
 }
 
-void CMMCIFLoop::PutIVector ( ivector I, const pstr T,
+void CMMCIFLoop::PutIVector ( ivector I, cpstr T,
                               int i1, int i2 )  {
 int  l,j,k;
 char iS[100];
@@ -1428,7 +1405,7 @@ char iS[100];
   }
 }
 
-Boolean CMMCIFLoop::WriteMMCIFLoop ( const pstr FName,
+Boolean CMMCIFLoop::WriteMMCIFLoop ( cpstr FName,
                                      byte gzipMode )  {
 CFile f;
   f.assign ( FName,True,False,gzipMode );
@@ -1626,7 +1603,7 @@ CMMCIFData::CMMCIFData() : CStream()  {
   InitMMCIFData();
 }
 
-CMMCIFData::CMMCIFData ( const pstr N ) : CStream()  {
+CMMCIFData::CMMCIFData ( cpstr N ) : CStream()  {
   InitMMCIFData();
   CreateCopy ( name,N );
 }
@@ -1732,7 +1709,7 @@ int i,lc,lt;
   }
 }
 
-Boolean CMMCIFData::CheckWrongField ( const pstr C, const pstr T )  {
+Boolean CMMCIFData::CheckWrongField ( cpstr C, cpstr T )  {
 int i;
   for (i=0;i<nWrongFields;i++)
     if ((!strcasecmp(C,WrongCat[i])) &&
@@ -1741,7 +1718,7 @@ int i;
 }
 
 
-int  CMMCIFData::ReadMMCIFData ( const pstr FName, byte gzipMode )  {
+int  CMMCIFData::ReadMMCIFData ( cpstr FName, byte gzipMode )  {
 CFile f;
 char  S[500];
 int   lcount;
@@ -1920,7 +1897,8 @@ int           RC,i;
     Warning |= CIFW_MissingField;
     if (flags & CIFFL_PrintWarnings)
       printf ( "\n **** mmCIF READ WARNING "
-               "<<line %i: expected data field missing>>\n%s\n",lcount,S );
+               "<<line %i: expected data field missing>>\n%s\n",
+               lcount,S );
   }
 
   while ((*p==' ') || (*p==char(9)))  p++;
@@ -1980,7 +1958,8 @@ int         RC,i,nC;
 
       //  look for category
       i = AddCategory ( T );
-      if ((i!=nC) && (nC>=0))  {  // empty loop; most probably corrupted file
+      if ((i!=nC) && (nC>=0))  {  // empty loop; most probably
+                                  // a corrupted file
         p = p1;   // play back to last category
         Warning |= CIFW_EmptyLoop;
         if (flags & CIFFL_PrintWarnings)
@@ -2053,7 +2032,8 @@ int         RC,i,nC;
         Warning |= CIFW_UnexpectedEOF;
         if (flags & CIFFL_PrintWarnings)
           printf ( "\n **** mmCIF READ WARNING "
-                   "<<line %i: unexpected end of file>>\n%s\n",lcount,S );
+                   "<<line %i: unexpected end of file>>\n%s\n",
+                   lcount,S );
       }
     } else
       Repeat = False;
@@ -2064,7 +2044,7 @@ int         RC,i,nC;
     do  {
       while ((*p==' ') || (*p==char(9)))  p++;
       if (!(*p) || (*p=='#'))  {
-        Repeat = !f.FileEnd();;
+        Repeat = !f.FileEnd();
         if (Repeat)  {
           f.ReadLine ( S );
           lcount++;
@@ -2248,7 +2228,7 @@ psvector cnames;
 
 }
 
-int  CMMCIFData::GetCategoryNo ( const pstr cname )  {
+int  CMMCIFData::GetCategoryNo ( cpstr cname )  {
 //   Binary search for index of category cname in Category[].
 // Return:
 //    >=0 : position of the category found
@@ -2279,9 +2259,9 @@ int l1,l2,l,k;
     if (l2!=l1)  {
       k = strcasecmp(cname,Category[index[l2]]->name);
       if (k==0)  return index[l2]; // is at RCth position
-      if (k>0)   return -2-l2;     // would be at l2+1=(-RC-1)th position
+      if (k>0)   return -2-l2;   // would be at l2+1=(-RC-1)th position
     }
-    return -2-l1;                  // would be at l1+1=(-RC-1)th position
+    return -2-l1;                // would be at l1+1=(-RC-1)th position
   } else
     // 'root' category should be always on top
     if (Category[index[0]]->name[0]==char(1))  return index[0];
@@ -2290,7 +2270,7 @@ int l1,l2,l,k;
 
 }
 
-int  CMMCIFData::AddCategory ( const pstr cname )  {
+int  CMMCIFData::AddCategory ( cpstr cname )  {
 //  return -1: a place for category has been added on the top of array;
 //             index is added and sorted automatically
 //        >=0: the category is already in the array -- its position
@@ -2328,10 +2308,11 @@ ivector          index1;
   Category = Category1;
   index    = index1;
   nCategories++;
-  return -nCategories; // the category has been added on the top of array
+  return -nCategories; // the category has been added on
+                       // the top of array
 }
 
-Boolean CMMCIFData::WriteMMCIFData ( const pstr FName, byte gzipMode )  {
+Boolean CMMCIFData::WriteMMCIFData ( cpstr FName, byte gzipMode )  {
 CFile f;
   f.assign ( FName,True,False,gzipMode );
   if (f.rewrite())  {
@@ -2360,7 +2341,7 @@ int  i;
 
 // ---------------  Retrieving data
 
-int  CMMCIFData::DeleteCategory ( const pstr CName )  {
+int  CMMCIFData::DeleteCategory ( cpstr CName )  {
 int k;
   k = GetCategoryNo ( CName );
   if (k<0)  return CIFRC_NoCategory;
@@ -2389,7 +2370,7 @@ int i;
   return 0;
 }
 
-int  CMMCIFData::DeleteStructure ( const pstr CName )  {
+int  CMMCIFData::DeleteStructure ( cpstr CName )  {
 int k;
   k = GetCategoryNo ( CName );
   if (k<0)  return CIFRC_NoCategory;
@@ -2398,7 +2379,7 @@ int k;
   else  return CIFRC_NotAStructure;
 }
 
-int  CMMCIFData::DeleteLoop ( const pstr CName )  {
+int  CMMCIFData::DeleteLoop ( cpstr CName )  {
 int k;
   k = GetCategoryNo ( CName );
   if (k<0)  return CIFRC_NoCategory;
@@ -2413,7 +2394,7 @@ PCMMCIFCategory CMMCIFData::GetCategory ( int categoryNo )  {
   return NULL;
 }
 
-PCMMCIFStruct CMMCIFData::GetStructure ( const pstr CName )  {
+PCMMCIFStruct CMMCIFData::GetStructure ( cpstr CName )  {
 int i;
   i = GetCategoryNo ( CName );
   if (i<0)  return NULL;
@@ -2422,7 +2403,7 @@ int i;
   else  return NULL;
 }
 
-PCMMCIFLoop CMMCIFData::GetLoop ( const pstr CName )  {
+PCMMCIFLoop CMMCIFData::GetLoop ( cpstr CName )  {
 int i;
   i = GetCategoryNo ( CName );
   if (i<0)  return NULL;
@@ -2452,7 +2433,7 @@ void CMMCIFData::GetDataName ( pstr & dname, Boolean Remove )  {
     CreateCopy ( dname,name );
 }
 
-int  CMMCIFData::CheckData ( const pstr CName, const pstr TName )  {
+int  CMMCIFData::CheckData ( cpstr CName, cpstr TName )  {
 //   CheckData(..) returns positive value if the field is in the
 // file:
 //   CIFRC_Structure  category CName is a structure
@@ -2517,8 +2498,8 @@ PPCMMCIFCategory C1;
   }
 }
 
-int  CMMCIFData::GetString  ( pstr & Dest, const pstr CName,
-                              const pstr TName, Boolean Remove )  {
+int  CMMCIFData::GetString  ( pstr & Dest, cpstr CName,
+                              cpstr TName, Boolean Remove )  {
 //   GetString(..), GetReal(..) and GetInteger(..) return 0 if the
 // requested field was found and successfully converted. Negative
 // returns mean:
@@ -2539,8 +2520,7 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFStruct(Category[i])->GetString ( Dest,TName,Remove );
 }
 
-pstr CMMCIFData::GetString ( const pstr CName, const pstr TName,
-                             int & RC )  {
+pstr CMMCIFData::GetString ( cpstr CName, cpstr TName, int & RC )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  {
     RC = CIFRC_NoCategory;
@@ -2553,7 +2533,7 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFStruct(Category[i])->GetString ( TName,RC );
 }
 
-int  CMMCIFData::DeleteField ( const pstr CName, const pstr TName )  {
+int  CMMCIFData::DeleteField ( cpstr CName, cpstr TName )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  return CIFRC_NoCategory;
   if (Category[i]->GetCategoryID()!=MMCIF_Struct)
@@ -2561,8 +2541,8 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFStruct(Category[i])->DeleteField ( TName );
 }
 
-int  CMMCIFData::GetReal ( realtype & R, const pstr CName,
-                           const pstr TName, Boolean Remove )  {
+int  CMMCIFData::GetReal ( realtype & R, cpstr CName,
+                           cpstr TName, Boolean Remove )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  return CIFRC_NoCategory;
   if (Category[i]->GetCategoryID()!=MMCIF_Struct)
@@ -2570,8 +2550,8 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFStruct(Category[i])->GetReal ( R,TName,Remove );
 }
 
-int  CMMCIFData::GetInteger ( int & I, const pstr CName,
-                              const pstr TName, Boolean Remove )  {
+int  CMMCIFData::GetInteger ( int & I, cpstr CName,
+                              cpstr TName, Boolean Remove )  {
 int j = GetCategoryNo ( CName );
   if (j<0)  return CIFRC_NoCategory;
   if (Category[j]->GetCategoryID()!=MMCIF_Struct)
@@ -2579,7 +2559,7 @@ int j = GetCategoryNo ( CName );
   return PCMMCIFStruct(Category[j])->GetInteger ( I,TName,Remove );
 }
 
-int  CMMCIFData::GetLoopLength ( const pstr CName )  {
+int  CMMCIFData::GetLoopLength ( cpstr CName )  {
 //   GetLoopLength(..) returns CIFRC_NotALoop if the category CName
 // is not a loop, CIFRC_NoCategory if the category CName is not
 // found. Non-negative returns give the length of the loop (may be
@@ -2592,8 +2572,8 @@ int i;
   return PCMMCIFLoop(Category[i])->nRows;
 }
 
-int  CMMCIFData::GetLoopString  ( pstr & Dest, const pstr CName,
-                                  const pstr TName, int nrow,
+int  CMMCIFData::GetLoopString  ( pstr & Dest, cpstr CName,
+                                  cpstr TName, int nrow,
                                   Boolean Remove )  {
 //   GetLoopString(..), GetLoopReal(..) and GetLoopInteger(..) act
 // like GetString(..), GetReal(..) and GetInteger(..) above for
@@ -2607,7 +2587,7 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFLoop(Category[i])->GetString ( Dest,TName,nrow,Remove );
 }
 
-pstr CMMCIFData::GetLoopString  ( const pstr CName, const pstr TName,
+pstr CMMCIFData::GetLoopString  ( cpstr CName, cpstr TName,
                                   int nrow, int & RC )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  {
@@ -2621,7 +2601,7 @@ int i = GetCategoryNo ( CName );
   return  PCMMCIFLoop(Category[i])->GetString ( TName,nrow,RC );
 }
 
-int CMMCIFData::DeleteLoopField ( const pstr CName, const pstr TName,
+int CMMCIFData::DeleteLoopField ( cpstr CName, cpstr TName,
                                   int nrow )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  return CIFRC_NoCategory;
@@ -2630,8 +2610,8 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFLoop(Category[i])->DeleteField ( TName,nrow );
 }
 
-int  CMMCIFData::GetLoopReal ( realtype & R, const pstr CName,
-                               const pstr TName, int nrow,
+int  CMMCIFData::GetLoopReal ( realtype & R, cpstr CName,
+                               cpstr TName, int nrow,
                                Boolean Remove )  {
 int i = GetCategoryNo ( CName );
   if (i<0)  return CIFRC_NoCategory;
@@ -2640,8 +2620,8 @@ int i = GetCategoryNo ( CName );
   return PCMMCIFLoop(Category[i])->GetReal ( R,TName,nrow,Remove );
 }
 
-int  CMMCIFData::GetLoopInteger ( int & I, const pstr CName,
-                                  const pstr TName, int nrow,
+int  CMMCIFData::GetLoopInteger ( int & I, cpstr CName,
+                                  cpstr TName, int nrow,
                                   Boolean Remove )  {
 int j = GetCategoryNo ( CName );
   if (j<0)  return CIFRC_NoCategory;
@@ -2651,8 +2631,8 @@ int j = GetCategoryNo ( CName );
 }
 
 
-int  CMMCIFData::GetLoopSVector ( psvector & S, const pstr CName,
-                                  const pstr TName, int i1, int i2,
+int  CMMCIFData::GetLoopSVector ( psvector & S, cpstr CName,
+                                  cpstr TName, int i1, int i2,
                                   Boolean Remove )  {
 //   GetLoopSVector(..), GetLoopRVector(..) and GetLoopIVector(..)
 // read CIF 'loop_' data into allocated vectors of strings, reals and
@@ -2673,8 +2653,8 @@ int i;
   return PCMMCIFLoop(Category[i])->GetSVector ( S,TName,i1,i2,Remove );
 }
 
-int  CMMCIFData::GetLoopRVector ( rvector & R, const pstr CName,
-                                  const pstr TName, int i1, int i2,
+int  CMMCIFData::GetLoopRVector ( rvector & R, cpstr CName,
+                                  cpstr TName, int i1, int i2,
                                   Boolean Remove )  {
 int i;
   i = GetCategoryNo ( CName );
@@ -2684,8 +2664,8 @@ int i;
   return PCMMCIFLoop(Category[i])->GetRVector ( R,TName,i1,i2,Remove );
 }
 
-int  CMMCIFData::GetLoopIVector ( ivector & I, const pstr CName,
-                                  const pstr TName, int i1, int i2,
+int  CMMCIFData::GetLoopIVector ( ivector & I, cpstr CName,
+                                  cpstr TName, int i1, int i2,
                                   Boolean Remove )  {
 int j;
   j = GetCategoryNo ( CName );
@@ -2698,13 +2678,13 @@ int j;
 
 // ---------------  Storing data
 
-void  CMMCIFData::PutDataName ( const pstr dname )  {
+void  CMMCIFData::PutDataName ( cpstr dname )  {
 // stores name for 'data_' record
   CreateCopy ( name,dname );
 }
 
-int  CMMCIFData::PutNoData ( int NoDataType, const pstr CName,
-                             const pstr TName )  {
+int  CMMCIFData::PutNoData ( int NoDataType, cpstr CName,
+                             cpstr TName )  {
 PCMMCIFStruct Struct;
 int           i,RC;
 
@@ -2733,8 +2713,8 @@ int           i,RC;
 
 }
 
-int  CMMCIFData::PutString ( const pstr S, const pstr CName,
-                             const pstr TName, Boolean Concatenate )  {
+int  CMMCIFData::PutString ( cpstr S, cpstr CName,
+                             cpstr TName, Boolean Concatenate )  {
 //   PutString(..), PutReal(..) and PutInteger(..) will put the
 // values given into the specified category (CName) under the
 // specified tag (TName). The category, tag and field are created
@@ -2773,7 +2753,7 @@ int           i,RC;
 
 }
 
-int   CMMCIFData::PutDate ( const pstr CName, const pstr TName )  {
+int   CMMCIFData::PutDate ( cpstr CName, cpstr TName )  {
 PCMMCIFStruct Struct;
 int           i,RC;
 
@@ -2802,15 +2782,15 @@ int           i,RC;
 
 }
 
-int  CMMCIFData::PutReal ( realtype R, const pstr CName,
-                           const pstr TName, int prec )  {
+int  CMMCIFData::PutReal ( realtype R, cpstr CName,
+                           cpstr TName, int prec )  {
 char rS[100];
   sprintf ( rS,"%.*g",prec,R );
   return PutString ( rS,CName,TName,False );
 }
 
-int  CMMCIFData::PutInteger ( int I, const pstr CName,
-                                     const pstr TName )  {
+int  CMMCIFData::PutInteger ( int I, cpstr CName,
+                                     cpstr TName )  {
 char iS[100];
   if (I>MinInt4)  sprintf ( iS,"%i",I );
   else  {
@@ -2822,7 +2802,7 @@ char iS[100];
 }
 
 
-int  CMMCIFData::AddLoop ( const pstr CName, PCMMCIFLoop & Loop )  {
+int  CMMCIFData::AddLoop ( cpstr CName, PCMMCIFLoop & Loop )  {
 int  i,RC;
 
   RC = CIFRC_Ok;
@@ -2849,7 +2829,7 @@ int  i,RC;
 
 }
 
-int  CMMCIFData::AddStructure ( const pstr CName,
+int  CMMCIFData::AddStructure ( cpstr CName,
                                 PCMMCIFStruct & Struct )  {
 int  i,RC;
 
@@ -2878,8 +2858,8 @@ int  i,RC;
 }
 
 
-int  CMMCIFData::PutLoopNoData ( int NoDataType, const pstr CName,
-                                 const pstr TName, int nrow )  {
+int  CMMCIFData::PutLoopNoData ( int NoDataType, cpstr CName,
+                                 cpstr TName, int nrow )  {
 PCMMCIFLoop Loop;
 int         i,RC;
 
@@ -2909,8 +2889,8 @@ int         i,RC;
 }
 
 
-int  CMMCIFData::PutLoopString ( const pstr S, const pstr CName,
-                                 const pstr TName, int nrow )  {
+int  CMMCIFData::PutLoopString ( cpstr S, cpstr CName,
+                                 cpstr TName, int nrow )  {
 //   PutLoopString(..), PutLoopReal(..) and PutLoopInteger(..) act
 // like PutString(..), PutReal(..) and PutInteger(..) above for
 // nrow-th element of the 'loop_' CName (indexed begining from 0).
@@ -2946,23 +2926,23 @@ int         i,RC;
 
 }
 
-int  CMMCIFData::PutLoopReal ( realtype R, const pstr CName,
-                               const pstr TName, int nrow, int prec )  {
+int  CMMCIFData::PutLoopReal ( realtype R, cpstr CName,
+                               cpstr TName, int nrow, int prec )  {
 char rS[100];
   sprintf ( rS,"%.*g",prec,R );
   return PutLoopString ( rS,CName,TName,nrow );
 }
 
-int  CMMCIFData::PutLoopInteger ( int I, const pstr CName,
-                                  const pstr TName, int nrow )  {
+int  CMMCIFData::PutLoopInteger ( int I, cpstr CName,
+                                  cpstr TName, int nrow )  {
 char iS[100];
   sprintf ( iS,"%i",I );
   return PutLoopString ( iS,CName,TName,nrow );
 }
 
 
-int  CMMCIFData::PutLoopSVector ( psvector S, const pstr CName,
-                                  const pstr TName, int i1, int i2 )  {
+int  CMMCIFData::PutLoopSVector ( psvector S, cpstr CName,
+                                  cpstr TName, int i1, int i2 )  {
 //   PutLoopSVector(..), PutLoopRVector(..) and PutLoopIVector(..)
 // put vectors of values into specified loop fields. Parameters i1
 // and i2 give the range of indices of values which are to be
@@ -3000,8 +2980,8 @@ int         i,RC;
 
 }
 
-int  CMMCIFData::PutLoopRVector ( rvector R, const pstr CName,
-                                  const pstr TName,
+int  CMMCIFData::PutLoopRVector ( rvector R, cpstr CName,
+                                  cpstr TName,
                                   int i1, int i2, int prec )  {
 PCMMCIFLoop Loop;
 int         i,RC;
@@ -3031,8 +3011,8 @@ int         i,RC;
 
 }
 
-int  CMMCIFData::PutLoopIVector ( ivector I, const pstr CName, 
-                                  const pstr TName,
+int  CMMCIFData::PutLoopIVector ( ivector I, cpstr CName, 
+                                  cpstr TName,
                                   int i1, int i2 )  {
 PCMMCIFLoop Loop;
 int         j,RC;
@@ -3063,8 +3043,8 @@ int         j,RC;
 }
 
 
-int CMMCIFData::RenameCategory ( const pstr CName,
-                                 const pstr newCName )  {
+int CMMCIFData::RenameCategory ( cpstr CName,
+                                 cpstr newCName )  {
 int i,RC;
   i = GetCategoryNo ( CName );
   if (i>=0)  {
@@ -3101,8 +3081,8 @@ int i;
 }
 
 
-int  CMMCIFData::CopyCategory ( PCMMCIFData Data, const pstr CName,
-                                const pstr newCName )  {
+int  CMMCIFData::CopyCategory ( PCMMCIFData Data, cpstr CName,
+                                cpstr newCName )  {
 PCMMCIFCategory Cat;
 int             i,di,dc,RC;
 
@@ -3215,7 +3195,7 @@ CMMCIFFile::CMMCIFFile() : CStream()  {
   InitMMCIFFile();
 }
 
-CMMCIFFile::CMMCIFFile ( const pstr FName, byte gzipMode )  : CStream()  {
+CMMCIFFile::CMMCIFFile ( cpstr FName, byte gzipMode )  : CStream()  {
   InitMMCIFFile ();
   ReadMMCIFFile ( FName,gzipMode );
 }
@@ -3258,7 +3238,7 @@ pstr GetMMCIFInputBuffer ( int & LineNo )  {
   return S;
 }
 
-int  CMMCIFFile::ReadMMCIFFile ( const pstr FName, byte gzipMode )  {
+int  CMMCIFFile::ReadMMCIFFile ( cpstr FName, byte gzipMode )  {
 CFile       f;
 PCMMCIFData CIF;
 int         RC;
@@ -3295,7 +3275,7 @@ int         RC;
   
 }
 
-int CMMCIFFile::WriteMMCIFFile ( const pstr FName, byte gzipMode )  {
+int CMMCIFFile::WriteMMCIFFile ( cpstr FName, byte gzipMode )  {
 CFile f;
   f.assign ( FName,True,False,gzipMode );
   if (f.rewrite())  {
@@ -3335,7 +3315,7 @@ int      i;
   }
 }
 
-int  CMMCIFFile::GetCIFDataNo ( const pstr DName )  {
+int  CMMCIFFile::GetCIFDataNo ( cpstr DName )  {
 //   Binary search for index of DName ttag in data[].
 // Return:
 //    >=0 : position of the DName found
@@ -3374,7 +3354,7 @@ PCMMCIFData  CMMCIFFile::GetCIFData ( int dataNo )  {
                                 else  return NULL;
 }
 
-PCMMCIFData  CMMCIFFile::GetCIFData ( const pstr DName )  {
+PCMMCIFData  CMMCIFFile::GetCIFData ( cpstr DName )  {
 int l;
   l = GetCIFDataNo ( DName );
   if (l>=0)  return data[l];
@@ -3405,7 +3385,7 @@ ivector      index1;
   }
 }
 
-int  CMMCIFFile::AddMMCIFData ( const pstr DName )  {
+int  CMMCIFFile::AddMMCIFData ( cpstr DName )  {
 //  return -1: the CIF data structure has been added on the
 //             top of data array; the index is added and sorted
 //             automatically
@@ -3488,7 +3468,7 @@ int i,k;
 MakeStreamFunctions(CMMCIFFile)
 
 
-int  isCIF ( pstr FName, byte gzipMode )  {
+int  isCIF ( cpstr FName, byte gzipMode )  {
 CFile   f;
 char    S[256];
 Boolean Done;
@@ -3545,11 +3525,11 @@ pstr InputLine;
       sprintf ( M,"unexpected end of file; line %i reads\n%s",
                 LineNo,InputLine );
     else if (RC & CIFW_LoopFieldMissing)
-      sprintf ( M,"expected data field in a loop not found; line %i reads\n%s",
-                LineNo,InputLine );
+      sprintf ( M,"expected data field in a loop not found; "
+                  "line %i reads\n%s", LineNo,InputLine );
     else if (RC & CIFW_LoopFieldMissing)
-      sprintf ( M,"expected data field in a loop not found; line %i reads\n%s",
-                LineNo,InputLine );
+      sprintf ( M,"expected data field in a loop not found; "
+                  "line %i reads\n%s", LineNo,InputLine );
     else if (RC & CIFW_NotAStructure)
       sprintf ( M,"a loop is used as a structure on line %i\n%s",
                 LineNo,InputLine );
@@ -3564,7 +3544,8 @@ pstr InputLine;
                 LineNo,InputLine );
   } else if (RC<0)
     switch (RC)  {
-      case CIFRC_StructureNoTag : strcpy(M,"tag of a structure not found");
+      case CIFRC_StructureNoTag : strcpy(M,"tag of a structure not "
+                                           "found");
                              break;
       case CIFRC_LoopNoTag      : strcpy(M,"tag of a loop not found");
                              break;
@@ -3574,11 +3555,13 @@ pstr InputLine;
                              break;
       case CIFRC_NoTag          : strcpy(M,"tag not found");
                              break;
-      case CIFRC_NotAStructure  : strcpy(M,"category is not a structure");
+      case CIFRC_NotAStructure  : strcpy(M,"category is not a "
+                                           "structure");
                              break;
       case CIFRC_NotALoop       : strcpy(M,"category is not a loop");
                              break;
-      case CIFRC_WrongIndex     : strcpy(M,"index outside the loop's limits");
+      case CIFRC_WrongIndex     : strcpy(M,"index outside the loop's "
+                                           "limits");
                              break;
       case CIFRC_NoField        : strcpy(M,"data is absent");
                              break;
