@@ -1,53 +1,28 @@
-//  $Id: mmdb_mmcif.h,v 1.6 2004/01/28 17:22:33 keb Exp $
+//  $Id: mmdb_mmcif.h,v 1.9 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    30.05.03   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -61,7 +36,7 @@
 //                  CMMCIFData     ( mmCIF data block  )
 //                  CMMCIFFile     ( mmCIF file        )
 //
-//  E. Krissinel 2000-2003
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -93,7 +68,7 @@ class CMMCIFCategory : public CStream  {
   public :
 
     CMMCIFCategory ();
-    CMMCIFCategory ( const pstr N );
+    CMMCIFCategory ( cpstr N );
     CMMCIFCategory ( RPCStream Object );
     ~CMMCIFCategory();
 
@@ -103,8 +78,8 @@ class CMMCIFCategory : public CStream  {
     virtual void Optimize     ();
 
     void  Sort     ();
-    int   GetTagNo ( const pstr ttag );
-    int   AddTag   ( const pstr ttag );
+    int   GetTagNo ( cpstr ttag );
+    int   AddTag   ( cpstr ttag );
 
     int   GetNofTags  () { return nTags; }
     pstr  GetTag      ( int tagNo );  // 0..nTags-1
@@ -126,7 +101,7 @@ class CMMCIFCategory : public CStream  {
     void          InitMMCIFCategory ();
     virtual void  FreeMemory        ();
     void          ExpandTags        ( int nTagsNew );
-    void          PutCategoryName   ( const pstr newName );
+    void          PutCategoryName   ( cpstr newName );
 
 };
 
@@ -146,34 +121,39 @@ class CMMCIFStruct : public CMMCIFCategory  {
     psvector field;
 
     CMMCIFStruct ();
-    CMMCIFStruct ( const pstr N );
+    CMMCIFStruct ( cpstr N );
     CMMCIFStruct ( RPCStream Object );
     ~CMMCIFStruct();
 
-    void AddField     ( const pstr F, const pstr T,
+    void AddField     ( cpstr F, cpstr T,
                         Boolean Concatenate=False );
     int  GetCategoryID() { return MMCIF_Struct; }
     void Optimize     ();
 
     pstr GetField    ( int tagNo );  // 0..nTags-1
-    int  GetString   ( pstr     & S, const pstr TName, Boolean Remove=False );
-    pstr GetString   ( const pstr TName, int & RC ); // NULL if TName
+    int  GetString   ( pstr     & S, cpstr TName,
+                                     Boolean Remove=False );
+    pstr GetString   ( cpstr TName, int & RC ); // NULL if TName
                                                      // is not there
-    int  DeleteField ( const pstr TName );   // <0 the field was not there
-    int  GetReal     ( realtype & R, const pstr TName, Boolean Remove=False );
-    int  GetInteger  ( int      & I, const pstr TName, Boolean Remove=False );
+    int  DeleteField ( cpstr TName );  // <0 the field was not
+                                            // there
+    int  GetReal     ( realtype & R, cpstr TName,
+                                     Boolean Remove=False );
+    int  GetInteger  ( int      & I, cpstr TName,
+                                     Boolean Remove=False );
 
-    //    PutString WILL NOT add the string if it consists only of spaces
-    // and NonBlankOnly is set to True. Any previous field for this tag
-    // will be removed, though.
-    void PutString   ( const pstr S, const pstr TName,
+    //    PutString WILL NOT add the string if it consists only of
+    // spaces and NonBlankOnly is set to True. Any previous field for
+    // this tag will be removed, though.
+    void PutString   ( cpstr S, cpstr TName,
                        Boolean NonBlankOnly=False );
-    void PutDate     ( const pstr T );
-    void PutNoData   ( int NoDataType, const pstr T  );
-    void PutReal     ( realtype R, const pstr TName, int prec=8 );
-    void PutInteger  ( int      I, const pstr TName );
+    void PutDate     ( cpstr T );
+    void PutNoData   ( int NoDataType, cpstr T  );
+    void PutReal     ( realtype R, cpstr TName, int prec=8 );
+    void PutInteger  ( int      I, cpstr TName );
 
-    Boolean WriteMMCIFStruct ( const pstr FName, byte gzipMode=GZM_CHECK );
+    Boolean WriteMMCIFStruct ( cpstr FName,
+                               byte gzipMode=GZM_CHECK );
     void    WriteMMCIF       ( RCFile f   );
 
     void Copy  ( PCMMCIFCategory Struct );
@@ -202,58 +182,63 @@ class CMMCIFLoop : public CMMCIFCategory  {
   public :
 
     CMMCIFLoop ();
-    CMMCIFLoop ( const pstr N );
+    CMMCIFLoop ( cpstr N );
     CMMCIFLoop ( RPCStream Object );
     ~CMMCIFLoop();
 
     //  AddLoopTag(..) adds tag T to the list of tags and removes
     //  any existing fields if Remove is set to True.
-    void  AddLoopTag   ( const pstr T, Boolean Remove=True );
+    void  AddLoopTag   ( cpstr T, Boolean Remove=True );
     //   AddString will add NULL if S consists only of spaces and
     // NonBlankOnly is set to True.
-    void  AddString    ( const pstr S, Boolean NonBlankOnly=False );
+    void  AddString    ( cpstr S, Boolean NonBlankOnly=False );
     void  AddNoData    ( int NoDataType );
     void  AddReal      ( realtype R, int prec=8 );
     void  AddInteger   ( int I          );
 
     int   GetLoopLength() { return nRows; }
     pstr  GetField     ( int rowNo, int tagNo );
-    int   GetString    ( pstr & S, const pstr TName, int nrow,
+    int   GetString    ( pstr & S, cpstr TName, int nrow,
                                        Boolean Remove=False );
 
     //  if GetString returns NULL but RC=0 then the field was
     //  either '.' or '?'
-    pstr  GetString    ( const pstr TName, int nrow, int & RC );
-    int   DeleteField  ( const pstr TName, int nrow );
+    pstr  GetString    ( cpstr TName, int nrow, int & RC );
+    int   DeleteField  ( cpstr TName, int nrow );
     int   DeleteRow    ( int nrow );
 
     // returns 0 if GetXxxx(..) had success
-    int   GetReal      ( realtype & R, const pstr TName, int nrow,
+    int   GetReal      ( realtype & R, cpstr TName, int nrow,
                                        Boolean Remove=False );
-    int   GetInteger   ( int      & I, const pstr TName, int nrow,
+    int   GetInteger   ( int      & I, cpstr TName, int nrow,
                                        Boolean Remove=False );
 
-    int   GetSVector   ( psvector & S, const pstr TName,
-                            int i1=0, int i2=MaxInt4, Boolean Remove=False );
-    int   GetRVector   ( rvector  & R, const pstr TName,
-                            int i1=0, int i2=MaxInt4, Boolean Remove=False );
-    int   GetIVector   ( ivector  & I, const pstr TName,
-                            int i1=0, int i2=MaxInt4, Boolean Remove=False );
+    int   GetSVector   ( psvector & S, cpstr TName,
+                           int i1=0, int i2=MaxInt4,
+                           Boolean Remove=False );
+    int   GetRVector   ( rvector  & R, cpstr TName,
+                           int i1=0, int i2=MaxInt4,
+                           Boolean Remove=False );
+    int   GetIVector   ( ivector  & I, cpstr TName,
+                           int i1=0, int i2=MaxInt4,
+                           Boolean Remove=False );
 
-    void  PutString    ( const pstr S, const pstr T, int nrow );
-    void  PutNoData    ( int NoDataType, const pstr T, int nrow );
-    void  PutReal      ( realtype R, const pstr T, int nrow, int prec=8 );
-    void  PutInteger   ( int      I, const pstr T, int nrow );
+    void  PutString    ( cpstr S, cpstr T, int nrow );
+    void  PutNoData    ( int NoDataType, cpstr T, int nrow );
+    void  PutReal      ( realtype R, cpstr T, int nrow,
+                                     int prec=8 );
+    void  PutInteger   ( int      I, cpstr T, int nrow );
 
-    void  PutSVector   ( psvector S, const pstr T, int i1, int i2 );
-    void  PutRVector   ( rvector  R, const pstr T, int i1, int i2,
+    void  PutSVector   ( psvector S, cpstr T, int i1, int i2 );
+    void  PutRVector   ( rvector  R, cpstr T, int i1, int i2,
                                                    int prec=8 );
-    void  PutIVector   ( ivector  I, const pstr T, int i1, int i2 );
+    void  PutIVector   ( ivector  I, cpstr T, int i1, int i2 );
 
     int   GetCategoryID() { return MMCIF_Loop; }
     void  Optimize     ();
 
-    Boolean WriteMMCIFLoop ( const pstr FName, byte gzipMode=GZM_CHECK );
+    Boolean WriteMMCIFLoop ( cpstr FName,
+                             byte gzipMode=GZM_CHECK );
     void    WriteMMCIF     ( RCFile f   );
 
     void  Copy  ( PCMMCIFCategory Loop );
@@ -357,7 +342,7 @@ class CMMCIFData : public CStream  {
   public :
 
     CMMCIFData ();
-    CMMCIFData ( const pstr N );
+    CMMCIFData ( cpstr N );
     CMMCIFData ( RPCStream Object );
     ~CMMCIFData();
 
@@ -383,19 +368,21 @@ class CMMCIFData : public CStream  {
     // CIF files.
     void    SetWrongFields   ( pstr * cats, pstr * tags );
 
-    int     ReadMMCIFData    ( const pstr FName, byte gzipMode=GZM_CHECK );
+    int     ReadMMCIFData    ( cpstr FName,
+                               byte gzipMode=GZM_CHECK );
     int     ReadMMCIFData    ( RCFile f, pstr S, int & lcount );
 
-    Boolean WriteMMCIFData   ( const pstr FName, byte gzipMode=GZM_CHECK );
+    Boolean WriteMMCIFData   ( cpstr FName,
+                               byte gzipMode=GZM_CHECK );
     void    WriteMMCIF       ( RCFile f   );
 
 
     // -------- Retrieving data
 
     int   GetNumberOfCategories ()  { return nCategories; }
-    PCMMCIFCategory GetCategory ( int   categoryNo ); // 0..nCategories-1
-    PCMMCIFStruct GetStructure  ( const pstr CName );
-    PCMMCIFLoop   GetLoop       ( const pstr CName );
+    PCMMCIFCategory GetCategory ( int categoryNo ); // 0..nCategories-1
+    PCMMCIFStruct GetStructure  ( cpstr CName );
+    PCMMCIFLoop   GetLoop       ( cpstr CName );
 
     //  FindLoop(..) finds a loop containing all tags from the
     //  tag list provided. The tag list should be terminated
@@ -406,9 +393,9 @@ class CMMCIFData : public CStream  {
     //   If dname is not NULL, first it is attempted to deallocate.
     // If Remove flag is set on then no actual space reallocation is
     // occured: just the internal pointer is copied to dname. The
-    // calling process wiil then be responsible for eventual deallocation
-    // of dname. This is also true for all string-extracting functions
-    // below.
+    // calling process wiil then be responsible for eventual
+    // deallocation of dname. This is also true for all
+    // string-extracting functions below.
     void GetDataName ( pstr & dname, Boolean Remove=False );
     pstr GetDataName ()  { return name; }
 
@@ -427,11 +414,11 @@ class CMMCIFData : public CStream  {
     // If TName is set to NULL then only the CName is checked and
     // possible returns are CIFRC_Structure, CIFRC_Loop and 
     // CIFRC_NoCategory.
-    int  CheckData       ( const pstr CName, const pstr TName );
+    int  CheckData       ( cpstr CName, cpstr TName );
 
-    int  DeleteCategory  ( const pstr CName );
-    int  DeleteStructure ( const pstr CName );
-    int  DeleteLoop      ( const pstr CName );
+    int  DeleteCategory  ( cpstr CName );
+    int  DeleteStructure ( cpstr CName );
+    int  DeleteLoop      ( cpstr CName );
 
     //   Optimize() optimizes the CIF data in memory allocation. It is
     // a good idea to call it once after extraction of data (GetXXXXXX
@@ -441,32 +428,32 @@ class CMMCIFData : public CStream  {
     //   GetString(..), GetReal(..) and GetInteger(..) return 0 if the
     // requested field was found and successfully converted. Negative
     // returns mean:
-    //    CIFRC_WrongFormat    the field was found but failed to convert
+    //    CIFRC_WrongFormat   the field was found but failed to convert
     //                        due to improper numeric format
-    //    CIFRC_NoTag          category CName was found, but it does not
+    //    CIFRC_NoTag         category CName was found, but it does not
     //                        have tag TName
-    //    CIFRC_NoCategory     category CName was not found
-    //    CIFRC_NotAStructure  category CName was found, but it is a loop
-    //                        rather than a structure.
+    //    CIFRC_NoCategory    category CName was not found
+    //    CIFRC_NotAStructure category CName was found, but it is
+    //                        a loop rather than a structure.
     //   GetString(..) will try to dispose Dest unless it is assigned
     // NULL value before the call. The string will be then dynamically
     // allocated and copied.
     //   If Remove is set to True, the field will be removed after
     // extraction.
-    int  GetString   ( pstr & Dest, const pstr CName, const pstr TName,
+    int  GetString   ( pstr & Dest, cpstr CName, cpstr TName,
                                     Boolean Remove=False );
-    pstr GetString   ( const pstr CName, const pstr TName, int & RC );
-    int  DeleteField ( const pstr CName, const pstr TName );
-    int  GetReal     ( realtype & R, const pstr CName, const pstr TName,
-                                     Boolean Remove=False );
-    int  GetInteger  ( int & I, const pstr CName, const pstr TName,
+    pstr GetString   ( cpstr CName, cpstr TName, int & RC );
+    int  DeleteField ( cpstr CName, cpstr TName );
+    int  GetReal     ( realtype & R, cpstr CName,
+                       cpstr TName, Boolean Remove=False );
+    int  GetInteger  ( int & I, cpstr CName, cpstr TName,
                                 Boolean Remove=False );
 
     //   GetLoopLength(..) returns CIFRC_NotALoop if the category CName
     // is not a loop, CIFRC_NoCategory if the category CName is not
     // found. Non-negative returns give the length of the loop (may be
     // 0 if the loop is empty).
-    int  GetLoopLength ( const pstr CName );
+    int  GetLoopLength ( cpstr CName );
 
     //   GetLoopString(..), GetLoopReal(..) and GetLoopInteger(..) act
     // like GetString(..), GetReal(..) and GetInteger(..) above for
@@ -475,39 +462,40 @@ class CMMCIFData : public CStream  {
     // CIFRC_WrongIndex if nrow is out of range.
     //   If Remove is set to True, the field will be removed after
     // extraction.
-    int  GetLoopString   ( pstr & Dest, const pstr CName,
-                                        const pstr TName, int nrow,
+    int  GetLoopString   ( pstr & Dest, cpstr CName,
+                                        cpstr TName, int nrow,
                                         Boolean Remove=False );
-    pstr GetLoopString   ( const pstr CName, const pstr TName,
+    pstr GetLoopString   ( cpstr CName, cpstr TName,
                            int nrow, int & RC );
-    int  DeleteLoopField ( const pstr CName, const pstr TName, int nrow );
-    int  GetLoopReal     ( realtype & R, const pstr CName,
-                                         const pstr TName, int nrow,
+    int  DeleteLoopField ( cpstr CName, cpstr TName,
+                           int nrow );
+    int  GetLoopReal     ( realtype & R, cpstr CName,
+                                         cpstr TName, int nrow,
                                          Boolean Remove=False );
-    int  GetLoopInteger  ( int & I, const pstr CName,
-                                    const pstr TName, int nrow,
+    int  GetLoopInteger  ( int & I, cpstr CName,
+                                    cpstr TName, int nrow,
                                     Boolean Remove=False );
 
     //   GetLoopSVector(..), GetLoopRVector(..) and GetLoopIVector(..)
-    // read CIF 'loop_' data into allocated vectors of strings, reals and
-    // integers, correspondingly. The vectors may be deallocated prior
-    // to call and assigned NULL, in which case they will be allocated
-    // with offsets of i1, which is also the lower index of the 'loop_'
-    // data transferred into it. The upper vector index is given by i2 or
-    // by the loop's length whichever is less. If vectors are not assigned
-    // NULL prior the call, it is assumed that they are properly (i1-offset,
-    // i2-i1+1 length) allocated.
+    // read CIF 'loop_' data into allocated vectors of strings, reals
+    // and integers, correspondingly. The vectors may be deallocated
+    // prior to call and assigned NULL, in which case they will be
+    // allocated with offsets of i1, which is also the lower index of
+    // the 'loop_' data transferred into it. The upper vector index is
+    // given by i2 or by the loop's length whichever is less. If
+    // vectors are not assigned NULL prior the call, it is assumed
+    // that they are properly (i1-offset, i2-i1+1 length) allocated.
     //   The return codes are same as those of GetLoopString(..),
     // GetLoopReal(..) and GetLoopInteger(..).
-    int  GetLoopSVector ( psvector & S, const pstr CName,
-                          const pstr TName,
-                          int i1=0, int i2=MaxInt4, Boolean Remove=False );
-    int  GetLoopRVector ( rvector  & R, const pstr CName,
-                          const pstr TName,
-                          int i1=0, int i2=MaxInt4, Boolean Remove=False );
-    int  GetLoopIVector ( ivector  & I, const pstr CName,
-                          const pstr TName,
-                          int i1=0, int i2=MaxInt4, Boolean Remove=False );
+    int  GetLoopSVector ( psvector & S, cpstr CName,
+                          cpstr TName, int i1=0, int i2=MaxInt4,
+                          Boolean Remove=False );
+    int  GetLoopRVector ( rvector  & R, cpstr CName,
+                          cpstr TName, int i1=0, int i2=MaxInt4,
+                          Boolean Remove=False );
+    int  GetLoopIVector ( ivector  & I, cpstr CName,
+                          cpstr TName, int i1=0, int i2=MaxInt4,
+                          Boolean Remove=False );
 
 
     // -------- Storing data
@@ -517,7 +505,8 @@ class CMMCIFData : public CStream  {
     // CIF data set.
     void FreeMemory ( int key=0 );
 
-    void PutDataName ( const pstr dname ); // stores name for 'data_' record
+    void PutDataName ( cpstr dname ); // stores name for 'data_'
+                                           // record
 
     //   PutString(..), PutReal(..) and PutInteger(..) will put the
     // values given into the specified category (CName) under the
@@ -529,13 +518,14 @@ class CMMCIFData : public CStream  {
     // of first appearance in PutString(..), PutReal(..) or
     // PutInteger(..).
     //   Return code - one of CIFRC_Ok or CIFRC_NotAStruct
-    int  PutNoData   ( int NoDataType, const pstr CName, const pstr TName );
-    int  PutString   ( const pstr S, const pstr CName, const pstr TName,
-                                     Boolean Concatenate=False );
-    int  PutDate     ( const pstr CName, const pstr TName );
-    int  PutReal     ( realtype R, const pstr CName, const pstr TName,
+    int  PutNoData   ( int NoDataType, cpstr CName,
+                       cpstr TName );
+    int  PutString   ( cpstr S, cpstr CName,
+                       cpstr TName, Boolean Concatenate=False );
+    int  PutDate     ( cpstr CName, cpstr TName );
+    int  PutReal     ( realtype R, cpstr CName, cpstr TName,
                                    int prec=8 );
-    int  PutInteger  ( int I, const pstr CName, const pstr TName );
+    int  PutInteger  ( int I, cpstr CName, cpstr TName );
 
     //   If loop category CName is not present in the CIF data
     // structure, AddLoop(..) creates an empty one and returns
@@ -560,8 +550,8 @@ class CMMCIFData : public CStream  {
     //     CIFRC_NotALoop category was present as a structure, but
     //                    has been replaced for a loop;
     //                    the category is empty.
-    int  AddLoop      ( const pstr CName, PCMMCIFLoop   & Loop   );
-    int  AddStructure ( const pstr CName, PCMMCIFStruct & Struct );
+    int  AddLoop      ( cpstr CName, PCMMCIFLoop   & Loop   );
+    int  AddStructure ( cpstr CName, PCMMCIFStruct & Struct );
 
     //   PutLoopString(..), PutLoopReal(..) and PutLoopInteger(..) act
     // like PutString(..), PutReal(..) and PutInteger(..) above for
@@ -572,14 +562,14 @@ class CMMCIFData : public CStream  {
     // is to start with nrow=0 and move up.
     //   These functions allow to form loops in arbitrary way.
     //   The functions may return CIFRC_Ok or CIFRC_NotALoop. 
-    int  PutLoopNoData  ( int NoDataType, const pstr CName,
-                                          const pstr TName, int nrow );
-    int  PutLoopString  ( const pstr S,   const pstr CName,
-                                          const pstr TName, int nrow );
-    int  PutLoopReal    ( realtype R, const pstr CName,
-                                      const pstr TName, int nrow,
+    int  PutLoopNoData  ( int NoDataType, cpstr CName,
+                                          cpstr TName, int nrow );
+    int  PutLoopString  ( cpstr S,   cpstr CName,
+                                          cpstr TName, int nrow );
+    int  PutLoopReal    ( realtype R, cpstr CName,
+                                      cpstr TName, int nrow,
                                       int  prec=8 );
-    int  PutLoopInteger ( int I, const pstr CName, const pstr TName,
+    int  PutLoopInteger ( int I, cpstr CName, cpstr TName,
                                  int nrow );
 
     //   PutLoopSVector(..), PutLoopRVector(..) and PutLoopIVector(..)
@@ -592,20 +582,21 @@ class CMMCIFData : public CStream  {
     // elements of the CIF loop for the corresponding tag undefined
     // (will be output like '?').
     //   These functions allow to form loops in arbitrary way.
-    int  PutLoopSVector ( psvector S, const pstr CName, const pstr TName,
-                                      int i1, int i2 );
-    int  PutLoopRVector ( rvector  R, const pstr CName, const pstr TName,
-                                      int i1, int i2, int prec=8 );
-    int  PutLoopIVector ( ivector  I, const pstr CName, const pstr TName,
-                                      int i1, int i2 );
+    int  PutLoopSVector ( psvector S, cpstr CName,
+                          cpstr TName, int i1, int i2 );
+    int  PutLoopRVector ( rvector  R, cpstr CName,
+                          cpstr TName, int i1, int i2,
+                          int prec=8 );
+    int  PutLoopIVector ( ivector  I, cpstr CName,
+                          cpstr TName, int i1, int i2 );
 
-    int  RenameCategory ( const pstr CName, const pstr newCName );
+    int  RenameCategory ( cpstr CName, cpstr newCName );
 
     // --------
 
     void Copy         ( PCMMCIFData Data );
-    int  CopyCategory ( PCMMCIFData Data, const pstr CName,
-                                          const pstr newCName=NULL );
+    int  CopyCategory ( PCMMCIFData Data, cpstr CName,
+                                          cpstr newCName=NULL );
 
     void PrintCategories();  // for debuging only
 
@@ -627,7 +618,7 @@ class CMMCIFData : public CStream  {
 
     void    InitMMCIFData   ();
     void    FreeWrongFields ();
-    Boolean CheckWrongField ( const pstr C, const pstr T );
+    Boolean CheckWrongField ( cpstr C, cpstr T );
     void    Sort            ();
 
     //   GetCategoryNo searches for index of category cname
@@ -635,8 +626,8 @@ class CMMCIFData : public CStream  {
     //    >=0 : position of the category found
     //     <0 : the category was not found, it could be inserted before
     //          (-RC-1)th element, where RC is the return value
-    int  GetCategoryNo  ( const pstr cname );
-    int  AddCategory    ( const pstr cname );
+    int  GetCategoryNo  ( cpstr cname );
+    int  AddCategory    ( cpstr cname );
     int  DeleteCategory ( int  CatNo );
 
     void GetDataItem    ( RCFile f, pstr S, pstr & L, pstr & p,
@@ -663,21 +654,21 @@ class CMMCIFFile : public CStream  {
     PPCMMCIFData data;
 
     CMMCIFFile ();
-    CMMCIFFile ( const pstr FName, byte gzipMode=GZM_CHECK );
+    CMMCIFFile ( cpstr FName, byte gzipMode=GZM_CHECK );
     CMMCIFFile ( RPCStream Object );
     ~CMMCIFFile();
 
     void  SetPrintWarnings ( Boolean SPW ) { PrintWarnings = SPW; }
     void  SetStopOnWarning ( Boolean SOW ) { StopOnWarning = SOW; }
 
-    int   ReadMMCIFFile    ( const pstr FName, byte gzipMode=GZM_CHECK );
-    int   WriteMMCIFFile   ( const pstr FName, byte gzipMode=GZM_CHECK );
+    int   ReadMMCIFFile    ( cpstr FName,byte gzipMode=GZM_CHECK);
+    int   WriteMMCIFFile   ( cpstr FName,byte gzipMode=GZM_CHECK);
 
     int   GetNofData() { return nData; }
     PCMMCIFData GetCIFData ( int        dataNo );  // 0..nData-1
-    PCMMCIFData GetCIFData ( const pstr DName  );
-    int   AddMMCIFData     ( const pstr DName  );
-    int   GetCIFDataNo     ( const pstr DName  );
+    PCMMCIFData GetCIFData ( cpstr DName  );
+    int   AddMMCIFData     ( cpstr DName  );
+    int   GetCIFDataNo     ( cpstr DName  );
 
     void  WriteMMCIF       ( RCFile f    );
 
@@ -704,7 +695,7 @@ extern pstr GetMMCIFInputBuffer ( int & LineNo );
 //    -1   if file FName does not exist
 //     0   if file FName is likely a CIF file ( 'data_' is present )
 //     1   if file FName is not a CIF file ( 'data_' is absent )
-extern int isCIF ( pstr FName, byte gzipMode=GZM_CHECK );
+extern int isCIF ( cpstr FName, byte gzipMode=GZM_CHECK );
 
 pstr GetCIFMessage ( pstr M, int RC );
 
