@@ -1,53 +1,28 @@
-//  $Id: mmdb_sbase0.h,v 1.2 2004/04/22 16:36:02 keb Exp $
+//  $Id: mmdb_sbase0.h,v 1.6 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    21.02.06   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +34,7 @@
 //                  CSBStructure ( SB structure (monomer) class )
 //                  CSBIndex     ( SB index class               )
 //
-//    E. Krissinel 2001-2006
+//   (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -98,9 +73,9 @@ typedef char SBPDBName    [pdb_name_len+1];
 typedef char SBElementName[element_len+1];
 
 //  -- names for graph files
-#define sbIndexFile   pstr("index.sbase")
-#define sbGraphFile   pstr("graph.sbase")
-#define sbStructFile  pstr("struct.sbase")
+#define sbIndexFile   cpstr("index.sbase")
+#define sbGraphFile   cpstr("graph.sbase")
+#define sbStructFile  cpstr("struct.sbase")
 
 
 //  ======================  SB Atom Class  =========================
@@ -112,6 +87,7 @@ class CSBAtom : public CStream  {
   public :
     char sca_name  [sca_name_len+1];    // SCA atom name
     char pdb_name  [pdb_name_len+1];    // PDB atom name (aligned)
+    char old_pdb_name[pdb_name_len+1];  // old PDB atom name (aligned)
     char element   [element_len+1];     // chemical element (aligned)
     char energyType[energy_type_len+1]; // energy type; set to empty
                                         // string "" if not provided
@@ -290,10 +266,10 @@ class CSBStructure : public CStream  {
 
     void  Reset    ();
 
-    void  PutFormula ( const pstr F );
-    void  PutName    ( const pstr N );
-    void  PutSynonym ( const pstr S );
-    void  PutCharge  ( const pstr G );
+    void  PutFormula ( cpstr F );
+    void  PutName    ( cpstr N );
+    void  PutSynonym ( cpstr S );
+    void  PutCharge  ( cpstr G );
 
     void  AddAtom    ( PCSBAtom    atom    );
     void  AddBond    ( PCSBBond    bond    );
@@ -302,13 +278,13 @@ class CSBStructure : public CStream  {
     void  AddTorsion ( PCSBTorsion torsion );
 
     void  RemoveEnergyTypes();
-    int   SetEnergyType ( const pstr sca_name, const pstr energyType,
+    int   SetEnergyType ( cpstr sca_name, cpstr energyType,
                           realtype partial_charge );
 
-    int   GetAtomNo  ( const pstr sca_name ); // returns 0 if atom not
+    int   GetAtomNo  ( cpstr sca_name ); // returns 0 if atom not
                             // found, >0 gives the atom ordinal number
 
-    PCSBAtom GetAtom ( const pstr sca_name );
+    PCSBAtom GetAtom ( cpstr sca_name );
 
     //  GetAtomTable(..)  does not deallocate atomTable!
     void  GetAtomTable ( PPCAtom & atomTable, int & nOfAtoms );
@@ -321,7 +297,7 @@ class CSBStructure : public CStream  {
 
     //  GetAtomNameMatch(..) returns anmatch[i], i=0..nAtoms-1, equal
     // to j such that name(Atom[i])==name(A[j]). Note that atom names
-    // are similarly aligned and space-padded in both MMDb and SBase.
+    // are similarly aligned and space-padded in both MMDB and SBase.
     // If ith atom in the structue is not found in A, anmatch[i] is
     // set -1.
     //   If array A contains atoms in different alternative
@@ -462,24 +438,24 @@ class CSBase0  {
     // The index must be loaded once before retrieving any
     // information from the database.
     //   LoadIndex() may return either SBASE_Ok or SBASE_FileNotFound.
-    int  LoadIndex  ( const pstr path   );
-    int  LoadIndex1 ( const pstr EnvVar );
+    int  LoadIndex  ( cpstr path   );
+    int  LoadIndex1 ( cpstr EnvVar );
 
     //   LoadStructure(..) reads structure from *.sbase files and
     // stores it in RAM for faster access. There is no special
     // functions to access loaded structures, all requests to
     // *.sbase files and RAM-storage are dispatched automatically.
-    int  LoadStructure   ( const pstr compoundID );
+    int  LoadStructure   ( cpstr compoundID );
     //   UnloadStructure(..) deletes strtucture from RAM and releases
     // its memory. The structure is then accessible in the normal
     // way from *.sbase files, which is slower.
-    int  UnloadStructure ( const pstr compoundID );
+    int  UnloadStructure ( cpstr compoundID );
 
     //   GetPath() returns full path to a file with file name FName
     // in the database directory.  Length of S should suffice for
     // accomodating the path. The function returns S.
     //   GetPath() will work only after loading the database index.
-    pstr GetPath ( pstr & S, const pstr FName );
+    pstr GetPath ( pstr & S, cpstr FName );
 
     //   GetStructFile() creates and open the database structure
     // file and returns its pointer. In the case of errors returns
@@ -500,7 +476,7 @@ class CSBase0  {
     // the structure after use (simply use delete).
     //   See description of CSBStructure for the explanation of
     // its fields.
-    PCSBStructure GetStructure ( const pstr compoundID );
+    PCSBStructure GetStructure ( cpstr compoundID );
     PCSBStructure GetStructure ( int structNo, PCFile structFile );
                                  // 0...nStructures-1
 
@@ -508,10 +484,10 @@ class CSBase0  {
     // file, which allows to save on opening/closing file if
     // multiple access to SBase structures is required. The file
     // is neither open nor closed by the function
-    PCSBStructure GetStructure ( const pstr compoundID,
+    PCSBStructure GetStructure ( cpstr compoundID,
                                  PCFile structFile );
 
-    PCResidue makeCResidue ( const pstr compoundID, 
+    PCResidue makeCResidue ( cpstr compoundID, 
                              PCFile     structFile,
                              Boolean    includeHydrogens=False,
                              Boolean    makeTer=False );
@@ -535,16 +511,16 @@ class CSBase0  {
                     int Hflag );
     int  GetGraph ( PCFile graphFile, RPCGraph G, int Hflag );
     int  GetGraph ( int   structNo  , RPCGraph G, int Hflag );
-    int  GetGraph ( const pstr  compoundID, RPCGraph G, int Hflag );
+    int  GetGraph ( cpstr  compoundID, RPCGraph G, int Hflag );
 
     //   GetStructNo() returns position of the structure with
     // (3-letter) name 'name' as found in the database index.
     //   Non-negative return means success, otherwise
     // SBASE_StructNotFound indicates that the requested structure
     // was not found in the database index.
-    int  GetStructNo ( const pstr compoundID );
+    int  GetStructNo ( cpstr compoundID );
 
-    int  GetNofAtoms ( const pstr compoundID );
+    int  GetNofAtoms ( cpstr compoundID );
     int  GetNofAtoms ( int   structNo );
 
     //   GetNofStructures() returns number of structures in the
@@ -590,7 +566,7 @@ class CSBase0  {
     int  CheckResidue ( PCResidue R, int Hflag, Boolean Cflag,
                         int & nInResidue, int & nInStructure,
                         int & nMatched,   ivector match,
-                        const pstr altLoc=pstr(""),
+                        cpstr altLoc=pstr(""),
                         int minMatchSize=0 );
 
 
@@ -718,7 +694,7 @@ class CSBase0  {
     // NonHAtom[Hconnect[i]], if Hconnect[i]>=0.
     //   The function may return SBASE_Ok, SNASE_StructNotFound,
     // SBASE_FileNotFound, SBASE_ReadError, SBASE_ConnectivityError.
-    int  GetAtoms ( const pstr compoundID,
+    int  GetAtoms ( cpstr compoundID,
                     int & nNonHAtoms, PAtomName NonHAtName,
                     int & nHAtoms,    PAtomName HAtName,
                     ivector Hconnect, ivector Elem,
@@ -733,13 +709,13 @@ class CSBase0  {
     // maxNBonds is the length of bondPairs[][].
     //   The function may return SBASE_Ok, SBASE_StructNotFound,
     // SBASE_FileNotFound, SBASE_ReadError.
-    int  GetBonds ( const pstr    compoundID,
+    int  GetBonds ( cpstr   compoundID,
                     ivector nBonds, imatrix bondPair,
                     int &   nAtoms, int     maxNAtoms,
                     int     maxNBonds );
 
 
-    int  GetHetInfo ( const pstr  name,
+    int  GetHetInfo ( cpstr       name,
                       pstr        Formula,
                       pstr        Hname,
                       pstr        Hsynonym,
