@@ -1,53 +1,28 @@
-//  $Id: mmdb_selmngr.h,v 1.11 2004/11/11 10:46:32 keb Exp $
+//  $Id: mmdb_selmngr.h,v 1.15 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    26.03.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CMMDBSelManager ( MMDB atom selection manager )
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2001-2004
+//   (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -251,79 +226,79 @@ class CMMDBSelManager : public CMMDBCoorManager  {
     //   Selecting by atom ID, space condition (a sphere) and some
     // other bits.
     void  SelectAtoms (
-             int      selHnd,   // must be obtained from NewSelection()
-             int      iModel,   // model number; iModel=0 means
-                                // 'any model'
-             const pstr Chains, // may be several chains "A,B,W"; "*"
-                                // means 'any chain' (in selected
-                                // model(s))
-             int      ResNo1,   // starting residue sequence number
-             const pstr Ins1,   // starting residue insertion code; "*"
-                                // means 'any code'
-             int      ResNo2,   // ending residue sequence number.
-             const pstr Ins2,   // ending residue insertion code; "*"
-                                // means 'any code'. Combination of
-                                // ResNo1=ResNo2=ANY_RES and
-                                // Ins1=Ins2="*" means 'any residue'
-                                // (in selected chain(s))
-             const pstr RNames, // may be several residue names
-                                // "ALA,GLU,CIS"; "*" means 'any
-                                // residue name'
-             const pstr ANames, // may be several names "CA,CB"; "*"
-                                // means 'any atom' (in selected
-                                // residue(s))
-             const pstr Elements, // may be several element types
-                                 // 'H,C,O,CU'; "*" means 'any element'
-             const pstr altLocs, // may be several alternative
-                                 // locations 'A,B'; "*" means
-                                 // 'any alternative location'
-             const pstr Segments, // may be several segment IDs
-                                  // like "S1,S2,A234"; "*" means
-                                  // 'any segment'
-             const pstr Charges,  // may be several charges like
-                                  // "+1,-2,  "; "*" means 'any charge'
-             realtype occ1, // lowest occupancy
-             realtype occ2, // highest occupancy; occ1=occ2<0.0
-                            // means "any occupancy"
-             realtype x0,   // reference x-point
-             realtype y0,   // reference y-point
-             realtype z0,   // reference z-point
-             realtype d0,   // selection distance from the
-                            // reference point; d0<=0.0
-                            // means "any distance" and values
-                            // of x0, y0 and z0 are ignored
+             int   selHnd,   // must be obtained from NewSelection()
+             int   iModel,   // model number; iModel=0 means
+                             // 'any model'
+             cpstr Chains,   // may be several chains "A,B,W"; "*"
+                             // means 'any chain' (in selected
+                             // model(s))
+             int   ResNo1,   // starting residue sequence number
+             cpstr Ins1,     // starting residue insertion code; "*"
+                             // means 'any code'
+             int   ResNo2,   // ending residue sequence number.
+             cpstr Ins2,     // ending residue insertion code; "*"
+                             // means 'any code'. Combination of
+                             // ResNo1=ResNo2=ANY_RES and
+                             // Ins1=Ins2="*" means 'any residue'
+                             // (in selected chain(s))
+             cpstr RNames,   // may be several residue names
+                             // "ALA,GLU,CIS"; "*" means 'any
+                             // residue name'
+             cpstr ANames,   // may be several names "CA,CB"; "*"
+                             // means 'any atom' (in selected
+                             // residue(s))
+             cpstr Elements, // may be several element types
+                             // 'H,C,O,CU'; "*" means 'any element'
+             cpstr altLocs,  // may be several alternative
+                             // locations 'A,B'; "*" means
+                             // 'any alternative location'
+             cpstr Segments, // may be several segment IDs
+                             // like "S1,S2,A234"; "*" means
+                             // 'any segment'
+             cpstr Charges,  // may be several charges like
+                             // "+1,-2,  "; "*" means 'any charge'
+             realtype occ1,  // lowest occupancy
+             realtype occ2,  // highest occupancy; occ1=occ2<0.0
+                             // means "any occupancy"
+             realtype x0,    // reference x-point
+             realtype y0,    // reference y-point
+             realtype z0,    // reference z-point
+             realtype d0,    // selection distance from the
+                             // reference point; d0<=0.0
+                             // means "any distance" and values
+                             // of x0, y0 and z0 are ignored
              int  selKey=SKEY_OR // selection key
            );
 
     //  Selecting by just atom ID, no other conditions
     void  SelectAtoms (
-             int      selHnd,   // must be obtained from NewSelection()
-             int      iModel,   // model number; iModel=0 means
-                                // 'any model'
-             const pstr Chains, // may be several chains "A,B,W"; "*"
-                                // means 'any chain' (in selected
-                                // model(s))
-             int      ResNo1,   // starting residue sequence number
-             const pstr Ins1,   // starting residue insertion code; "*"
-                                // means 'any code'
-             int      ResNo2,   // ending residue sequence number.
-             const pstr Ins2,   // ending residue insertion code; "*"
-                                // means 'any code'. Combination of
-                                // ResNo1=ResNo2=ANY_RES and
-                                // Ins1=Ins2="*" means 'any residue
-                                // number' (in selected chain(s))
-             const pstr RNames, // may be several residue names
-                                // "ALA,GLU,CIS"; "*" means 'any
-                                // residue name'
-             const pstr ANames, // may be several names "CA,CB"; "*"
-                                // means 'any atom' (in selected
-                                // residue(s))
-             const pstr Elements, // may be several element types
-                                 // "H,C,O,CU"; "*" means 'any element'
-             const pstr altLocs, // may be several alternative
-                                 // locations 'A,B'; "*" means
-                                 // 'any alternative location'
-             int  selKey=SKEY_OR // selection key
+             int   selHnd,   // must be obtained from NewSelection()
+             int   iModel,   // model number; iModel=0 means
+                             // 'any model'
+             cpstr Chains,   // may be several chains "A,B,W"; "*"
+                             // means 'any chain' (in selected
+                             // model(s))
+             int   ResNo1,   // starting residue sequence number
+             cpstr Ins1,     // starting residue insertion code; "*"
+                             // means 'any code'
+             int   ResNo2,   // ending residue sequence number.
+             cpstr Ins2,     // ending residue insertion code; "*"
+                             // means 'any code'. Combination of
+                             // ResNo1=ResNo2=ANY_RES and
+                             // Ins1=Ins2="*" means 'any residue
+                             // number' (in selected chain(s))
+             cpstr RNames,   // may be several residue names
+                             // "ALA,GLU,CIS"; "*" means 'any
+                             // residue name'
+             cpstr ANames,   // may be several names "CA,CB"; "*"
+                             // means 'any atom' (in selected
+                             // residue(s))
+             cpstr Elements, // may be several element types
+                             // "H,C,O,CU"; "*" means 'any element'
+             cpstr altLocs,  // may be several alternative
+                             // locations 'A,B'; "*" means
+                             // 'any alternative location'
+             int selKey=SKEY_OR // selection key
            );
 
 
@@ -348,7 +323,7 @@ class CMMDBSelManager : public CMMDBCoorManager  {
              int        selHnd, // must be obtained from NewSelection()
              int       selType, // selection type STYPE_XXXXX
              int     UDDhandle, // UDD handle
-             const pstr selStr, // selection string
+             cpstr selStr, // selection string
              int       cmpRule, // comparison rule
              int        selKey  // selection key
            );
@@ -454,82 +429,82 @@ class CMMDBSelManager : public CMMDBCoorManager  {
     //   Selecting by atom ID, space condition (a sphere) and some
     // other bits.
     void  Select (
-             int      selHnd,   // must be obtained from NewSelection()
-             int      selType,  // selection type STYPE_XXXXX
-             int      iModel,   // model number; iModel=0 means
-                                // 'any model'
-             const pstr Chains, // may be several chains "A,B,W"; "*"
-                                // means 'any chain' (in selected
-                                // model(s))
-             int      ResNo1,   // starting residue sequence number
-             const pstr Ins1,   // starting residue insertion code; "*"
-                                // means 'any code'
-             int      ResNo2,   // ending residue sequence number.
-             const pstr Ins2,   // ending residue insertion code; "*"
-                                // means 'any code'. Combination of
-                                // ResNo1=ResNo2=ANY_RES and
-                                // Ins1=Ins2="*" means 'any residue'
-                                // (in selected chain(s))
-             const pstr RNames, // may be several residue names
-                                // "ALA,GLU,CIS"; "*" means
-                                // 'any residue name'
-             const pstr ANames, // may be several names "CA,CB"; "*"
-                                // means 'any atom' (in selected
-                                // residue(s))
-             const pstr Elements, // may be several element types
-                                 // 'H,C,O,CU'; "*" means 'any element'
-             const pstr altLocs, // may be several alternative
-                                 // locations 'A,B'; "*" means
-                                 // 'any alternative location'
-             const pstr Segments, // may be several segment IDs like
-                                 // "S1,S2,A234"; "*" means
-                                 // 'any segment'
-             const pstr Charges, // may be several charges like
-                                 // "+1,-2,  "; "*" means 'any charge'
-             realtype occ1, // lowest occupancy
-             realtype occ2, // highest occupancy; occ1=occ2<0.0
-                            // means "any occupancy"
-             realtype x0,   // reference x-point
-             realtype y0,   // reference y-point
-             realtype z0,   // reference z-point
-             realtype d0,   // selection distance from the
-                            // reference point; d0<=0.0
-                            // means "any distance" and values
-                            // of x0, y0 and z0 are ignored
-             int  selKey=SKEY_OR // selection key
+             int   selHnd,   // must be obtained from NewSelection()
+             int   selType,  // selection type STYPE_XXXXX
+             int   iModel,   // model number; iModel=0 means
+                             // 'any model'
+             cpstr Chains,   // may be several chains "A,B,W"; "*"
+                             // means 'any chain' (in selected
+                             // model(s))
+             int   ResNo1,   // starting residue sequence number
+             cpstr Ins1,     // starting residue insertion code; "*"
+                             // means 'any code'
+             int   ResNo2,   // ending residue sequence number.
+             cpstr Ins2,     // ending residue insertion code; "*"
+                             // means 'any code'. Combination of
+                             // ResNo1=ResNo2=ANY_RES and
+                             // Ins1=Ins2="*" means 'any residue'
+                             // (in selected chain(s))
+             cpstr RNames,   // may be several residue names
+                             // "ALA,GLU,CIS"; "*" means
+                             // 'any residue name'
+             cpstr ANames,   // may be several names "CA,CB"; "*"
+                             // means 'any atom' (in selected
+                             // residue(s))
+             cpstr Elements, // may be several element types
+                             // 'H,C,O,CU'; "*" means 'any element'
+             cpstr altLocs,  // may be several alternative
+                             // locations 'A,B'; "*" means
+                             // 'any alternative location'
+             cpstr Segments, // may be several segment IDs like
+                             // "S1,S2,A234"; "*" means
+                             // 'any segment'
+             cpstr Charges,  // may be several charges like
+                             // "+1,-2,  "; "*" means 'any charge'
+             realtype occ1,  // lowest occupancy
+             realtype occ2,  // highest occupancy; occ1=occ2<0.0
+                             // means "any occupancy"
+             realtype x0,    // reference x-point
+             realtype y0,    // reference y-point
+             realtype z0,    // reference z-point
+             realtype d0,    // selection distance from the
+                             // reference point; d0<=0.0
+                             // means "any distance" and values
+                             // of x0, y0 and z0 are ignored
+             int selKey=SKEY_OR // selection key
            );
 
 
     //  Selecting by just atom ID, no other conditions
     void  Select (
-             int      selHnd,   // must be obtained from NewSelection()
-             int      selType,  // selection type STYPE_XXXXX
-             int      iModel,   // model number; iModel=0 means
-                                // 'any model'
-             const pstr Chains, // may be several chains "A,B,W"; "*"
-                                // means 'any chain' (in selected
-                                // model(s))
-             int      ResNo1,   // starting residue sequence number
-             const pstr Ins1,   // starting residue insertion code; "*"
-                                // means 'any code'
-             int      ResNo2,   // ending residue sequence number.
-             const pstr Ins2,   // ending residue insertion code; "*"
-                                // means 'any code'. Combination of
-                                // ResNo1=ResNo2=ANY_RES and
-                                // Ins1=Ins2="*" means 'any residue
-                                // number' (in selected chain(s))
-             const pstr RNames, // may be several residue names
-                                // "ALA,GLU,CIS"; "*" means
-                                // 'any residue name'
-             const pstr ANames, // may be several names "CA,CB"; "*"
-                                // means 'any atom' (in selected
-                                // residue(s))
-             const pstr Elements, // may be several element types
-                                 // "H,C,O,CU"; "*" means 'any element'
-             const pstr altLocs, // may be several alternative
-                                 // locations 'A,B'; "*" means
-                                 // 'any alternative location'
-             int  selKey=SKEY_OR // selection key
+             int   selHnd,   // must be obtained from NewSelection()
+             int   selType,  // selection type STYPE_XXXXX
+             int   iModel,   // model number; iModel=0 means
+                             // 'any model'
+             cpstr Chains,   // may be several chains "A,B,W"; "*"
+                             // means 'any chain' (in selected
+                             // model(s))
+             int   ResNo1,   // starting residue sequence number
+             cpstr Ins1,     // starting residue insertion code; "*"
+                             // means 'any code'
+             int   ResNo2,   // ending residue sequence number.
+             cpstr Ins2,     // ending residue insertion code; "*"
+                             // means 'any code'. Combination of
+                             // ResNo1=ResNo2=ANY_RES and
+                             // Ins1=Ins2="*" means 'any residue
+                             // number' (in selected chain(s))
+             cpstr RNames,   // may be several residue names
+                             // "ALA,GLU,CIS"; "*" means
+                             // 'any residue name'
+             cpstr ANames,   // may be several names "CA,CB"; "*"
+                             // means 'any atom' (in selected
+                             // residue(s))
+             cpstr Elements, // may be several element types
+                             // "H,C,O,CU"; "*" means 'any element'
+             cpstr altLocs,  // may be several alternative
+                             // locations 'A,B'; "*" means
+                             // 'any alternative location'
+             int selKey=SKEY_OR // selection key
            );
 
 
@@ -577,10 +552,10 @@ class CMMDBSelManager : public CMMDBCoorManager  {
     //  otherwise.
 
     int   Select (
-             int  selHnd,    // must be obtained from NewSelection()
-             int  selType,   // selection type STYPE_XXXXX
-             const pstr CID, // coordinate ID
-             int  selKey     // selection key
+             int   selHnd,    // must be obtained from NewSelection()
+             int   selType,   // selection type STYPE_XXXXX
+             cpstr CID,       // coordinate ID
+             int   selKey     // selection key
            );
 
     //  Propagating the selection up and down coordinate hierarchy
@@ -636,7 +611,8 @@ class CMMDBSelManager : public CMMDBCoorManager  {
                               int    selKey,  int &  nsel );
     void  SelectObject      ( PCMask object,  int    maskNo,
                               int    selKey,  int &  nsel );
-    void  MakeSelIndex      ( int    selHnd,  int    selType, int nsel );
+    void  MakeSelIndex      ( int    selHnd,  int    selType,
+                              int    nsel );
 
     void  ResetManager();
 
