@@ -1,53 +1,28 @@
-//  $Id: mmdb_symop.cpp,v 1.12 2005/09/22 16:32:39 keb Exp $
+//  $Id: mmdb_symop.cpp,v 1.14 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    06.12.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +34,7 @@
 //  **** Classes :   CSymOp  ( symmetry operator )
 //       ~~~~~~~~~   
 //
-//  E. Krissinel 2000-2003
+//   (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -111,7 +86,7 @@ void CSymOp::FreeMemory()  {
   XYZOp = NULL;
 }
 
-int  CSymOp::SetSymOp ( const pstr XYZOperation )  {
+int  CSymOp::SetSymOp ( cpstr XYZOperation )  {
 int  i,j;
 
   CreateCopy ( XYZOp,XYZOperation );
@@ -513,10 +488,10 @@ int i;
   Nops = 0;
 }
 
-#define  symop_file  pstr("symop.lib")
+#define  symop_file  cpstr("symop.lib")
 
-int  CSymOps::SetGroupSymopLib ( const pstr SpaceGroup,
-                                 const pstr symop_lib )  {
+int  CSymOps::SetGroupSymopLib ( cpstr SpaceGroup,
+                                 cpstr symop_lib )  {
 char  S[500];
 char  G[100];
 pstr  p;
@@ -527,9 +502,9 @@ int   i,RC;
 
   CreateCopy ( SpGroup,SpaceGroup );
 
-  if (!symop_lib)          p = symop_file;
-  else if (!symop_lib[0])  p = symop_file;
-                     else  p = symop_lib;
+  if (!symop_lib)          p = pstr(symop_file);
+  else if (!symop_lib[0])  p = pstr(symop_file);
+                     else  p = pstr(symop_lib);
   f.assign ( p,True );
   if (!f.reset(True))  {
     p = getenv ( "SYMOP" );
@@ -580,10 +555,10 @@ int   i,RC;
 }
 
 
-#define  syminfo_file  pstr("syminfo.lib")
+#define  syminfo_file  cpstr("syminfo.lib")
 
-int  CSymOps::SetGroup ( const pstr SpaceGroup, 
-                         const pstr syminfo_lib )  {
+int  CSymOps::SetGroup ( cpstr SpaceGroup, 
+                         cpstr syminfo_lib )  {
 CFile f;
 pstr  p;
 char  S[500];
@@ -601,9 +576,9 @@ long  symop_start;
 
   CreateCopy ( SpGroup,SpaceGroup );
 
-  if (!syminfo_lib)          p = syminfo_file;
-  else if (!syminfo_lib[0])  p = syminfo_file;
-                       else  p = syminfo_lib;
+  if (!syminfo_lib)          p = pstr(syminfo_file);
+  else if (!syminfo_lib[0])  p = pstr(syminfo_file);
+                       else  p = pstr(syminfo_lib);
   f.assign ( p,True );
   if (!f.reset(True))  {
     p = getenv ( "SYMINFO" );
@@ -710,7 +685,7 @@ void CSymOps::Reset()  {
   FreeMemory();
 }
 
-int  CSymOps::AddSymOp ( const pstr XYZOperation )  {
+int  CSymOps::AddSymOp ( cpstr XYZOperation )  {
 // adds a symmetry operation
 PPCSymOp SymOp1;
 int      i;
@@ -725,7 +700,7 @@ int      i;
   return SymOp[i]->SetSymOp ( XYZOperation );
 }
 
-void CSymOps::PutGroupName ( const pstr SpGroupName )  {
+void CSymOps::PutGroupName ( cpstr SpGroupName )  {
   CreateCopy ( SpGroup,SpGroupName );
 }
 
