@@ -1,53 +1,28 @@
-//  $Id: mmdb_tables.cpp,v 1.10 2005/05/09 10:07:56 keb Exp $
+//  $Id: mmdb_tables.cpp,v 1.15 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    06.02.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -65,7 +40,7 @@
 //                   RName1 ( 1-characters amino acid names         )
 //
 //
-//  (C) E. Krissinel  2001-2004
+//  (C) E. Krissinel  2000-2008
 //
 //  =================================================================
 //
@@ -85,7 +60,7 @@
 
 //  =================================================================
 
-pstr const ElementName[nElementNames] = {
+cpstr const ElementName[nElementNames] = {
   " H", "HE",
   "LI", "BE", " B", " C", " N", " O", " F", "NE",
   "NA", "MG", "AL", "SI", " P", " S", "CL", "AR",
@@ -177,7 +152,7 @@ realtype const VdWaalsRadius[nElementNames] = {
 //            ^^^^
   2.75, 2.65,
 //      ^^^^
-        2.55, 2.45, 2.35, 2.20, 2.05, 1.90,
+        2.55, 2.45, 2.35, 2.20, 1.73, 1.90,
 //      ^^^^  ^^^^  ^^^^  ^^^^  ^^^^  ^^^^
         1.75, 1.63, 1.40, 1.39,
 //      ^^^^
@@ -233,7 +208,7 @@ realtype const IonicRadius[nElementNames] = {
      2.87, 2.85
 };
 
-pstr const ElementMetal[nElementMetals] = {
+cpstr const ElementMetal[nElementMetals] = {
   "LI", "BE", "NA", "MG", "AL", " K", "CA", "SC", "TI", " V",
   "MN", "FE", "CO", "NI", "CU", "ZN", "GA", "RB", "SR", " Y",
   "ZR", "NB", "MO", "TC", "RU", "RH", "PD", "AG", "CD", "IN",
@@ -247,7 +222,7 @@ pstr const ElementMetal[nElementMetals] = {
 };
 
 
-pstr const HydAtomName[nHydAtomNames] = {
+cpstr const HydAtomName[nHydAtomNames] = {
   "0H", "1H", "2H", "3H", "4H", "5H", "6H", "7H", "8H", "9H",
   "HH", "*H", "'H", """H"
 };
@@ -255,7 +230,7 @@ pstr const HydAtomName[nHydAtomNames] = {
 
 
 
-Boolean isMetal ( const pstr element )  {
+Boolean isMetal ( cpstr element )  {
 char    name[3];
 Boolean isThere;
 int     i;
@@ -271,7 +246,7 @@ int     i;
   return isThere;
 }
 
-int  getElementNo ( const pstr element )  {
+int  getElementNo ( cpstr element )  {
 int  type=0;
 char El[3];
   if ((!element[1]) || (element[1]==' '))  {
@@ -290,7 +265,7 @@ char El[3];
   return type+1;  // so that hydrogen is 1
 }
 
-realtype  getMolecWeight ( const pstr element )  {
+realtype  getMolecWeight ( cpstr element )  {
 int  type=0;
 char El[3];
   if ((!element[1]) || (element[1]==' '))  {
@@ -309,7 +284,7 @@ char El[3];
   return MolecWeight[type];
 }
 
-realtype  getCovalentRadius ( const pstr element )  {
+realtype  getCovalentRadius ( cpstr element )  {
 int  type=0;
 char El[3];
   if ((!element[1]) || (element[1]==' '))  {
@@ -328,7 +303,7 @@ char El[3];
   return CovalentRadius[type];
 }
 
-realtype  getVdWaalsRadius ( const pstr element )  {
+realtype  getVdWaalsRadius ( cpstr element )  {
 int  type=0;
 char El[3];
   if ((!element[1]) || (element[1]==' '))  {
@@ -347,14 +322,14 @@ char El[3];
   return VdWaalsRadius[type];
 }
 
-pstr const ResidueName[nResNames] = {
+cpstr const ResidueName[nResNames] = {
   "ALA", "ARG", "ASN", "ASP", "CYS", "CYH", "GLN",
   "GLU", "GLY", "HIS", "ILE", "LEU", "LYS", "MET",
   "PHE", "PRO", "SER", "THR", "TRP", "TYR", "VAL",
   "HEM", "WAT", "SUL", "END", "DUM"
 };
 
-int getResidueNo ( pstr resName )  {
+int getResidueNo ( cpstr resName )  {
 int i,m;
   m = -1;
   for (i=0;(i<nResNames) && (m<0);i++)
@@ -371,7 +346,7 @@ char const ResidueName1[nResNames] = {
 };
 
 
-pstr const StdSolventName[nSolventNames] = {
+cpstr const StdSolventName[nSolventNames] = {
   "ADE", "CYT", "GUA", "INO", "THY", "URA",
   "WAT", "HOH", "TIP", "H2O", "DOD", "MOH"
 };
@@ -403,7 +378,7 @@ SAAProperty const AAProperty[nAminoacidNames] = {
   { "VAL",  4.2,  0.0,  1.66 }
 };
 
-realtype GetAAHydropathy ( const pstr resName )  {
+realtype GetAAHydropathy ( cpstr resName )  {
 int  i1,j;
   i1 = -1;
   j  = 0;
@@ -417,7 +392,7 @@ int  i1,j;
   return AAProperty[i1].hydropathy;
 }
 
-realtype GetAACharge ( const pstr resName )  {
+realtype GetAACharge ( cpstr resName )  {
 int  i1,j;
   i1 = -1;
   j  = 0;
@@ -431,7 +406,7 @@ int  i1,j;
   return AAProperty[i1].charge;
 }
 
-realtype GetAASolvationEnergy ( const pstr resName )  {
+realtype GetAASolvationEnergy ( cpstr resName )  {
 int  i1,j;
   i1 = -1;
   j  = 0;
@@ -476,7 +451,7 @@ int const AASimilarity[nAminoacidNames][nAminoacidNames] = {
 };
 
 
-int  GetAASimilarity ( const pstr resName1, const pstr resName2 )  {
+int  GetAASimilarity ( cpstr resName1, cpstr resName2 )  {
 int  i1,i2,j;
 
   i1 = -1;
@@ -532,7 +507,7 @@ realtype const AAHydropathyScale[nAminoacidNames] = {
     2.8,  -1.6,  -0.8,  -0.7,  -0.9,  -1.3,   4.2
 };
 
-realtype GetAAHydropathy ( const pstr resName )  {
+realtype GetAAHydropathy ( cpstr resName )  {
 int  i1,j;
   i1 = -1;
   j  = 0;
@@ -554,7 +529,7 @@ realtype const AACharge[nAminoacidNames] = {
     0.0,   0.0,   0.0,   0.0,   0.0,   0.0,   0.0
 };
 
-int  GetAASimilarity ( const pstr resName1, const pstr resName2 )  {
+int  GetAASimilarity ( cpstr resName1, cpstr resName2 )  {
 int  i1,i2,j;
 
   i1 = -1;
@@ -581,7 +556,7 @@ int  i1,i2,j;
   
 }
 
-Boolean isAminoacid ( const pstr resName )  {
+Boolean isAminoacid ( cpstr resName )  {
 Boolean isThere;
 int     i;
   isThere = False;
@@ -593,14 +568,15 @@ int     i;
 */
 
 
-pstr const NucleotideName[nNucleotideNames] = {
-   "A",  "C",  "G",  "I",  "T",  "U",
-  "+A", "+C", "+G", "+I", "+T", "+U",
-  "RA", "RC", "RG", "RU"
+cpstr const NucleotideName[nNucleotideNames] = {
+   "A",  "C",  "G",  "I",   "T",   "U",
+  "+A", "+C", "+G", "+I",  "+T",  "+U",
+  "DA", "DC", "DG", "DI",  "DT",  "DU",
+  "RA", "RC", "RG", "RU", "5NC", "TYD"
 };
 
 
-Boolean isSolvent ( const pstr resName )  {
+Boolean isSolvent ( cpstr resName )  {
 Boolean isThere;
 int     i;
   isThere = False;
@@ -609,7 +585,7 @@ int     i;
   return isThere;
 }
 
-Boolean isAminoacid ( const pstr resName )  {
+Boolean isAminoacid ( cpstr resName )  {
 Boolean isThere;
 int     i;
   isThere = False;
@@ -618,7 +594,7 @@ int     i;
   return isThere;
 }
 
-Boolean isNucleotide ( const pstr resName )  {
+Boolean isNucleotide ( cpstr resName )  {
 Boolean isThere;
 int     i;
   isThere = False;
@@ -627,12 +603,12 @@ int     i;
   return isThere;
 }
 
-Boolean isSugar ( const pstr resName )  {
+Boolean isSugar ( cpstr resName )  {
   return False;
 }
 
 
-pstr const Res1Code[] = {
+cpstr const Res1Code[] = {
 
   // standard aminoacids
 
@@ -707,7 +683,7 @@ pstr const Res1Code[] = {
 };
 
 
-void Get1LetterCode ( const pstr res3name, pstr res1code )  {
+void Get1LetterCode ( cpstr res3name, pstr res1code )  {
 char r[4];
 int  i;
 
