@@ -1,53 +1,28 @@
-//  $Id: mmdb_title.cpp,v 1.8 2005/12/19 16:51:28 keb Exp $
+//  $Id: mmdb_title.cpp,v 1.10 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    17.05.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -70,7 +45,7 @@
 //                  CRemark
 //                  CMMDBTitle  ( MMDB title section )
 //
-//  E. Krissinel 2000-2003
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -122,7 +97,7 @@ CObsLine::CObsLine() : CContainerClass()  {
   InitObsLine();
 }
 
-CObsLine::CObsLine ( const pstr S ) : CContainerClass()  {
+CObsLine::CObsLine ( cpstr S ) : CContainerClass()  {
   InitObsLine();
   ConvertPDBASCII ( S );
 }
@@ -179,7 +154,7 @@ char        DateCIF[20];
   }
 }
 
-int CObsLine::ConvertPDBASCII ( const pstr S )  {
+int CObsLine::ConvertPDBASCII ( cpstr S )  {
 int i;
   Date9to11 ( &(S[11]),repDate );
   strncpy   ( idCode,&(S[21]),4 );
@@ -222,6 +197,7 @@ IDCode      idCode1;
         else  idCode1[0] = char(0);
         if (i==0)  {
           DateCIFto11 ( DateCIF,repDate );
+          DateCIF[11] = char(0);
           strcpy ( idCode  ,idCode1 );
           strcpy ( DateCIF0,DateCIF );
         } else if ((strcmp(DateCIF0,DateCIF)) ||
@@ -279,7 +255,7 @@ CTitleLine::CTitleLine() : CContString()  {
   InitTitleLine();
 }
 
-CTitleLine::CTitleLine ( const pstr S ) : CContString()  {
+CTitleLine::CTitleLine ( cpstr S ) : CContString()  {
   InitTitleLine();
   ConvertPDBASCII ( S );
 }
@@ -296,7 +272,7 @@ void  CTitleLine::InitTitleLine()  {
   CreateCopy ( CIFTag,     CIFTAG_TITLE  );
 }
 
-int  CTitleLine::ConvertPDBASCII ( const pstr S )  {
+int  CTitleLine::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10]) );
   else CreateCopy ( Line,pstr(" ") );
@@ -331,7 +307,7 @@ CCaveat::CCaveat() : CContString()  {
   InitCaveat();
 }
 
-CCaveat::CCaveat ( const pstr S ) : CContString()  {
+CCaveat::CCaveat ( cpstr S ) : CContString()  {
   InitCaveat();
   ConvertPDBASCII ( S );
 }
@@ -348,7 +324,7 @@ void  CCaveat::InitCaveat()  {
   CreateCopy ( CIFTag     ,CIFTAG_TEXT                );
 }
 
-int  CCaveat::ConvertPDBASCII ( const pstr S )  {
+int  CCaveat::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>12)  {
     strncpy ( idCode,&(S[11]),4 );
     idCode[4] = char(0);
@@ -419,7 +395,7 @@ CCompound::CCompound() : CContString()  {
   InitCompound();
 }
 
-CCompound::CCompound ( const pstr S ) : CContString()  {
+CCompound::CCompound ( cpstr S ) : CContString()  {
   InitCompound();
   ConvertPDBASCII ( S );
 }
@@ -435,7 +411,7 @@ void  CCompound::InitCompound()  {
   CreateCopy ( CIFTag     ,CIFTAG_NDB_DESCRIPTOR );
 }
 
-int  CCompound::ConvertPDBASCII ( const pstr S )  {
+int  CCompound::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10])  );
   else CreateCopy ( Line,pstr(" ") );
@@ -470,7 +446,7 @@ CSource::CSource() : CContString()  {
   InitSource();
 }
 
-CSource::CSource ( const pstr S ) : CContString()  {
+CSource::CSource ( cpstr S ) : CContString()  {
   InitSource();
   ConvertPDBASCII ( S );
 }
@@ -486,7 +462,7 @@ void  CSource::InitSource()  {
   CreateCopy ( CIFTag     ,CIFTAG_SOURCE );
 }
 
-int  CSource::ConvertPDBASCII ( const pstr S )  {
+int  CSource::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10])  );
   else CreateCopy ( Line,pstr(" ") );
@@ -520,7 +496,7 @@ CKeyWords::CKeyWords() : CStream()  {
   Init();
 }
 
-CKeyWords::CKeyWords ( const pstr S ) : CStream()  {
+CKeyWords::CKeyWords ( cpstr S ) : CStream()  {
   Init();
   ConvertPDBASCII ( S );
 }
@@ -552,7 +528,7 @@ int i;
   Cont      = False;
 }
 
-int  CKeyWords::ConvertPDBASCII ( const pstr S )  {
+int  CKeyWords::ConvertPDBASCII ( cpstr S )  {
 //  we anticipate that length of S is 80 characters
 //  -- pad with spaces if necessary
 char   L[85];
@@ -780,7 +756,7 @@ CExpData::CExpData() : CContString()  {
   InitExpData();
 }
 
-CExpData::CExpData ( const pstr S ) : CContString()  {
+CExpData::CExpData ( cpstr S ) : CContString()  {
   InitExpData();
   ConvertPDBASCII ( S );
 }
@@ -796,7 +772,7 @@ void  CExpData::InitExpData()  {
   CreateCopy ( CIFTag     ,CIFTAG_METHOD );
 }
 
-int  CExpData::ConvertPDBASCII ( const pstr S )  {
+int  CExpData::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10])  );
   else CreateCopy ( Line,pstr(" ") );
@@ -831,7 +807,7 @@ CAuthor::CAuthor() : CContString()  {
   InitAuthor();
 }
 
-CAuthor::CAuthor ( const pstr S ) : CContString()  {
+CAuthor::CAuthor ( cpstr S ) : CContString()  {
   InitAuthor();
   ConvertPDBASCII ( S );
 }
@@ -847,7 +823,7 @@ void  CAuthor::InitAuthor()  {
   CreateCopy ( CIFTag     ,CIFTAG_NAME         );
 }
 
-int  CAuthor::ConvertPDBASCII ( const pstr S )  {
+int  CAuthor::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10])  );
   else CreateCopy ( Line,pstr(" ") );
@@ -882,7 +858,7 @@ CRevData::CRevData() : CContainerClass()  {
   InitRevData();
 }
 
-CRevData::CRevData ( const pstr S ) : CContainerClass()  {
+CRevData::CRevData ( cpstr S ) : CContainerClass()  {
   InitRevData();
   ConvertPDBASCII ( S );
 }
@@ -1006,7 +982,7 @@ pstr        F;
 
 }
 
-int CRevData::ConvertPDBASCII ( const pstr S )  {
+int CRevData::ConvertPDBASCII ( cpstr S )  {
 int  i;
 pstr endptr;
 char N[20];
@@ -1073,7 +1049,7 @@ CSupersede::CSupersede() : CContainerClass()  {
   InitSupersede();
 }
 
-CSupersede::CSupersede ( const pstr S ) : CContainerClass()  {
+CSupersede::CSupersede ( cpstr S ) : CContainerClass()  {
   InitSupersede();
   ConvertPDBASCII ( S );
 }
@@ -1135,7 +1111,7 @@ char        DateCIF[20];
   }
 }
 
-int CSupersede::ConvertPDBASCII ( const pstr S )  {
+int CSupersede::ConvertPDBASCII ( cpstr S )  {
 int i;
   if (S[9]==' ')  {
     Date9to11 ( &(S[11]),sprsdeDate );
@@ -1181,6 +1157,7 @@ IDCode      idCode1;
         else  idCode1[0] = char(0);
         if (i==0)  {
           DateCIFto11 ( DateCIF,sprsdeDate );
+          DateCIF[11] = char(0);
           strcpy ( idCode  ,idCode1 );
           strcpy ( DateCIF0,DateCIF );
         } else if ((strcmp(DateCIF0,DateCIF)) ||
@@ -1238,7 +1215,7 @@ CJournal::CJournal() : CContString()  {
   InitJournal();
 }
 
-CJournal::CJournal ( const pstr S ) : CContString()  {
+CJournal::CJournal ( cpstr S ) : CContString()  {
   InitJournal();
   ConvertPDBASCII ( S );
 }
@@ -1254,7 +1231,7 @@ void  CJournal::InitJournal()  {
   CreateCopy ( CIFTag     ,CIFTAG_TEXT     );
 }
 
-int  CJournal::ConvertPDBASCII ( const pstr S )  {
+int  CJournal::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>10)
        CreateCopy ( Line,&(S[10]) );
   else CreateCopy ( Line,pstr(" ") );
@@ -1288,7 +1265,7 @@ CRemark::CRemark() : CContainerClass()  {
   InitRemark();
 }
 
-CRemark::CRemark ( const pstr S ) : CContainerClass()  {
+CRemark::CRemark ( cpstr S ) : CContainerClass()  {
   InitRemark();
   ConvertPDBASCII ( S );
 }
@@ -1306,7 +1283,7 @@ void  CRemark::InitRemark()  {
   Remark    = NULL;
 }
 
-int  CRemark::ConvertPDBASCII ( const pstr S )  {
+int  CRemark::ConvertPDBASCII ( cpstr S )  {
 int i;
   GetInteger ( remarkNum,&(S[7]),3 );
   if (remarkNum==MinInt4)  CreateCopy ( Remark,S );
@@ -1440,9 +1417,9 @@ void  CMMDBTitle::FreeMemory()  {
 
 }
 
-void  CMMDBTitle::SetHeader ( const pstr Classification,
-                              const pstr DepDate,
-                              const pstr IDCode )  {
+void  CMMDBTitle::SetHeader ( cpstr Classification,
+                              cpstr DepDate,
+                              cpstr IDCode )  {
 // fills the PDB file header
   CreateCopy ( classification ,Classification  );
   strncpy    ( depDate,DepDate,sizeof(depDate) );
