@@ -1,53 +1,28 @@
-//  $Id: mmdb_utils.h,v 1.13 2005/11/17 18:07:17 keb Exp $
+//  $Id: mmdb_utils.h,v 1.16 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    12.01.06   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -85,7 +60,7 @@
 //                   ParseResID     ( parses residue ID line          )
 //                   ParseAtomPath  ( parses full atom path           )
 //
-//  E. Krissinel  2001-2006
+//   (C) E. Krissinel  2000-2008
 //
 //  =================================================================
 //
@@ -111,22 +86,22 @@
 // ==================  Date functions  ===================
 
 // converts  DD-MMM-YY  to DD-MMM-YYYY; appends terminating zero
-extern void  Date9to11   ( const pstr Date9, pstr Date11 );
+extern void  Date9to11   ( cpstr Date9, pstr Date11 );
 
 // converts DD-MMM-YYYY to DD-MMM-YY;  does not append terminating zero
-extern void  Date11to9   ( const pstr Date11, pstr Date9 );
+extern void  Date11to9   ( cpstr Date11, pstr Date9 );
 
 // converts DD-MMM-YY   to YYYY-MM-DD;  appends terminating zero
-extern void  Date9toCIF  ( const pstr Date9, pstr DateCIF );
+extern void  Date9toCIF  ( cpstr Date9, pstr DateCIF );
 
 // converts DD-MMM-YYYY to YYYY-MM-DD;  appends terminating zero
-extern void  Date11toCIF ( const pstr Date11, pstr DateCIF );
+extern void  Date11toCIF ( cpstr Date11, pstr DateCIF );
 
 // converts YYYY-MM-DD  to DD-MMM-YY;   appends terminating zero
-extern void  DateCIFto9  ( const pstr DateCIF, pstr Date9 );
+extern void  DateCIFto9  ( cpstr DateCIF, pstr Date9 );
 
 // converts YYYY-MM-DD  to DD-MMM-YYYY; appends terminating zero
-extern void  DateCIFto11 ( const pstr DateCIF, pstr Date11 );
+extern void  DateCIFto11 ( cpstr DateCIF, pstr Date11 );
 
 
 // =================  Format functions  ==================
@@ -135,13 +110,13 @@ extern void  DateCIFto11 ( const pstr DateCIF, pstr Date11 );
 // first M characters. This number is returned in N.
 //   The return is False if no integer number may be
 // recognized. In this case, N is assigned MinInt4 value.
-extern Boolean GetInteger ( int & N, const pstr S, int M );
+extern Boolean GetInteger ( int & N, cpstr S, int M );
 
 //   Returns True if S contains a real number in its
 // first M characters. This number is returned in R.
 //   The return is False if no real number may be
 // recognized. In this case, R is assigned -MaxReal value.
-extern Boolean GetReal ( realtype & R, const pstr S, int M );
+extern Boolean GetReal ( realtype & R, cpstr S, int M );
 
 //   Returns True if S contains an integer number in its
 // first M characters. This number is returned in N. In addition
@@ -151,7 +126,7 @@ extern Boolean GetReal ( realtype & R, const pstr S, int M );
 //   The return is False if no integer number may be
 // recognized. In this case, N is assigned MinInt4 value,
 // "ins" just returns (M+1)th symbol of S (+terminating 0).
-extern Boolean  GetIntIns ( int & N, pstr ins, const pstr S, int M ); 
+extern Boolean  GetIntIns ( int & N, pstr ins, cpstr S, int M ); 
 
 //  Integer N is converted into ASCII string of length M
 // and pasted onto first M characters of string S. No
@@ -175,7 +150,7 @@ extern void  PutRealF ( pstr S, realtype R, int M, int L );
 //  If N is set to MinInt4, then first M+1 characters of
 // string S are set to space, and no insert code are
 // appended.
-extern void  PutIntIns ( pstr S, int N, int M, const pstr ins );
+extern void  PutIntIns ( pstr S, int N, int M, cpstr ins );
 
 
 //   CIFInteger(..), CIFReal(..) and CIFGetString(..) automate
@@ -183,28 +158,28 @@ extern void  PutIntIns ( pstr S, int N, int M, const pstr ins );
 // is erroneous or absent, they store an error message in
 // CIFErrorLocation string (below) and return non-zero.
 extern int CIFGetInteger  ( int & I, PCMMCIFStruct Struct,
-                                     const pstr Tag,
+                                     cpstr Tag,
                                      Boolean Remove=True );
 extern int CIFGetReal     ( realtype & R, PCMMCIFStruct Struct,
-                                     const pstr Tag,
+                                     cpstr Tag,
                                      Boolean Remove=True );
 extern int CIFGetString   ( pstr S, PCMMCIFStruct Struct,
-                                    const pstr Tag, int SLen,
-                                    const pstr DefS,
+                                    cpstr Tag, int SLen,
+                                    cpstr DefS,
                                     Boolean Remove=True );
 
-extern int CIFGetInteger  ( int & I, PCMMCIFLoop Loop, const pstr Tag,
+extern int CIFGetInteger  ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                             int & Signal );
-extern int CIFGetInteger1 ( int & I, PCMMCIFLoop Loop, const pstr Tag,
+extern int CIFGetInteger1 ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                             int nrow );
 
 extern int CIFGetReal     ( realtype & R, PCMMCIFLoop Loop,
-                            const pstr Tag, int & Signal );
+                            cpstr Tag, int & Signal );
 extern int CIFGetReal1    ( realtype & R, PCMMCIFLoop Loop,
-                            const pstr Tag, int nrow );
+                            cpstr Tag, int nrow );
 
-extern int CIFGetString   ( pstr S, PCMMCIFLoop Loop, const pstr Tag,
-                            int row, int SLen, const pstr DefS );
+extern int CIFGetString   ( pstr S, PCMMCIFLoop Loop, cpstr Tag,
+                            int row, int SLen, cpstr DefS );
 
 //  Calculates AI=A^{-1}
 extern void  Mat4Inverse ( mat44 & A, mat44 & AI );
@@ -246,7 +221,7 @@ extern char CIFErrorLocation[200];
 
 //  Returns ASCII string explaining the nature of
 // Error_xxxx error code. 
-extern pstr  GetErrorDescription ( int ErrorCode );
+extern cpstr  GetErrorDescription ( int ErrorCode );
 
 
 
@@ -267,7 +242,7 @@ class CContainerClass : public CStream  {
 
     //    ConvertPDBASCII(..) will return one of the Error_XXXXX
     // constants, see <mmdb_defs.h>
-    virtual int     ConvertPDBASCII ( const pstr S  ) { return 0;     }
+    virtual int     ConvertPDBASCII ( cpstr S  ) { return 0;     }
     virtual void    PDBASCIIDump    ( pstr S, int N ) {}
     virtual Boolean PDBASCIIDump1   ( RCFile f )      { return False; }
     virtual void    MakeCIF         ( PCMMCIFData CIF, int N ) {}
@@ -326,11 +301,11 @@ class CContString : public CContainerClass  {
     pstr Line;  // a string
 
     CContString ();
-    CContString ( const pstr S );
+    CContString ( cpstr S );
     CContString ( RPCStream Object );
     ~CContString();
 
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  PDBASCIIDump    ( pstr S, int N );
     Boolean PDBASCIIDump1 ( RCFile f );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
@@ -430,7 +405,7 @@ class CAtomPath : public CStream  {
     int      isSet;
 
     CAtomPath  ();
-    CAtomPath  ( const pstr ID );
+    CAtomPath  ( cpstr ID );
     CAtomPath  ( RPCStream Object );
     ~CAtomPath ();
 
@@ -476,7 +451,7 @@ class CAtomPath : public CStream  {
     //   0   <-> Ok
     //   -1  <-> wrong numerical format for model
     //   -2  <-> wrong numerical format for sequence number
-    int SetPath ( const pstr ID );
+    int SetPath ( cpstr ID );
 
     void write ( RCFile f );
     void read  ( RCFile f );
@@ -514,7 +489,7 @@ class CQuickSort : public CStream  {
 
 //  --------------------------------------------------------------
 
-void  takeWord ( pstr & p, pstr wrd, const pstr ter, int l );
+void  takeWord ( pstr & p, pstr wrd, cpstr ter, int l );
 
 //   ParseAtomID(..) reads the atom ID of the following form:
 //    {name} {[element]} {:altcode}
@@ -540,7 +515,7 @@ void  takeWord ( pstr & p, pstr wrd, const pstr ter, int l );
 //        *[*]:A      (same as above)
 //        *[*]:*      (any atom)
 //        *           (any atom with no alternate location indicator)
-extern void ParseAtomID ( const  pstr ID, AtomName aname,
+extern void ParseAtomID ( cpstr ID, AtomName aname,
                           Element elname, AltLoc   aloc );
 
 //   ParseResID(..) reads the residue ID of the following form:
@@ -572,7 +547,7 @@ extern void ParseAtomID ( const  pstr ID, AtomName aname,
 //        *(*).A      (same as above)
 //        *(*).*      (any residue)
 //        *           (any residue with no insertion code)
-extern int ParseResID ( const pstr ID, int & sn,
+extern int ParseResID ( cpstr ID, int & sn,
                         InsCode inscode, ResName resname );
 
 
@@ -619,7 +594,7 @@ extern int ParseResID ( const pstr ID, int & sn,
 //      -1                - wrong numerical format for model (fatal)
 //      -2                - wrong numerical format for seqNum (fatal)
 
-extern int ParseAtomPath ( const pstr ID,
+extern int ParseAtomPath ( cpstr      ID,
                            int &      mdl,
                            ChainID    chn,
                            int &      sn,
@@ -632,7 +607,7 @@ extern int ParseAtomPath ( const pstr ID,
 
 
 
-extern int ParseSelectionPath ( const pstr CID,
+extern int ParseSelectionPath ( cpstr   CID,
                                 int &   iModel,
                                 pstr    Chains,
                                 int &   sNum1,
@@ -648,15 +623,15 @@ extern int ParseSelectionPath ( const pstr CID,
 
 extern void MakeSelectionPath ( pstr       CID,
                                 int        iModel,
-                                const pstr Chains,
+                                cpstr      Chains,
                                 int        sNum1,
                                 const InsCode ic1,
                                 int        sNum2,
                                 const InsCode ic2,
-                                const pstr RNames,
-                                const pstr ANames,
-                                const pstr Elements,
-                                const pstr altLocs );
+                                cpstr      RNames,
+                                cpstr      ANames,
+                                cpstr      Elements,
+                                cpstr      altLocs );
 
 #endif
 
