@@ -1,53 +1,28 @@
-//  $Id: mmdb_xml.cpp,v 1.9 2004/11/11 10:46:32 keb Exp $
+//  $Id: mmdb_xml.cpp,v 1.11 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    26.01.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -58,7 +33,7 @@
 //  **** Classes :  CXMLObject
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2002-2004
+//   (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -83,39 +58,39 @@ CXMLObject::CXMLObject() : CStream()  {
   InitXMLObject();
 }
 
-CXMLObject::CXMLObject ( const pstr Tag ) : CStream()  {
+CXMLObject::CXMLObject ( cpstr Tag ) : CStream()  {
   InitXMLObject();
   SetTag  ( Tag );
 }
 
-CXMLObject::CXMLObject ( const pstr Tag, const pstr Data )
+CXMLObject::CXMLObject ( cpstr Tag, cpstr Data )
           : CStream()  {
   InitXMLObject();
   SetTag  ( Tag  );
   SetData ( Data );
 }
 
-CXMLObject::CXMLObject ( const pstr Tag, realtype V, int length )
+CXMLObject::CXMLObject ( cpstr Tag, realtype V, int length )
           : CStream()  {
   InitXMLObject();
   SetTag  ( Tag      );
   SetData ( V,length );
 }
 
-CXMLObject::CXMLObject ( const pstr Tag, int iV, int length )
+CXMLObject::CXMLObject ( cpstr Tag, int iV, int length )
           : CStream()  {
   InitXMLObject();
   SetTag  ( Tag       );
   SetData ( iV,length );
 }
 
-CXMLObject::CXMLObject ( const pstr Tag, Boolean bV ) : CStream()  {
+CXMLObject::CXMLObject ( cpstr Tag, Boolean bV ) : CStream()  {
   InitXMLObject();
   SetTag  ( Tag );
   SetData ( bV  );
 }
 
-CXMLObject::CXMLObject ( const pstr Tag, PCXMLObject XMLObject )
+CXMLObject::CXMLObject ( cpstr Tag, PCXMLObject XMLObject )
           : CStream()  {
   InitXMLObject();
   SetTag    ( Tag       );
@@ -155,12 +130,12 @@ int i;
   object   = NULL;
 }
 
-void  CXMLObject::SetTag ( const pstr Tag )  {
+void  CXMLObject::SetTag ( cpstr Tag )  {
 pstr p,t;
 int  n;
 
   // count ampersands
-  p = Tag;
+  p = pstr(Tag);
   n = 0;
   while (*p)  {
     if (*p=='&')  n++;
@@ -169,7 +144,7 @@ int  n;
   // calculate the tag length
   n = n*4 + strlen(Tag) + 1;
   // substract leading underscores
-  p = Tag;
+  p = pstr(Tag);
   while (*p=='_')  {
     p++;
     n--;
@@ -201,11 +176,11 @@ int  n;
   *t = char(0);
 }
 
-void  CXMLObject::SetData ( const pstr Data )  {
+void  CXMLObject::SetData ( cpstr Data )  {
 pstr p,d;
 int  n;
   // count ampersands
-  p = Data;
+  p = pstr(Data);
   n = 0;
   while (*p)  {
     if (*p=='&')  n += 4;
@@ -217,7 +192,7 @@ int  n;
   if (objData)  delete[] objData;
   objData = new char[n];
   // copy data, preceeding ampersands with the escape
-  p = Data;
+  p = pstr(Data);
   d = objData;
   while (*p)  {
     if (*p=='&')  {
@@ -236,7 +211,7 @@ int  n;
   *d = char(0);
 }
 
-void  CXMLObject::AddData ( const pstr Data )  {
+void  CXMLObject::AddData ( cpstr Data )  {
 pstr d1,d2;
   d1      = objData;
   objData = NULL;
@@ -272,7 +247,7 @@ int CXMLObject::AddMMCIFCategory ( PCMMCIFCategory mmCIFCat )  {
   return -1;
 }
 
-pstr getCCIFTag ( pstr & ccifTag, const pstr Tag )  {
+pstr getCCIFTag ( pstr & ccifTag, cpstr Tag )  {
   if (Tag[0]=='_')  return CreateCopCat ( ccifTag,pstr("ccif") ,Tag );
               else  return CreateCopCat ( ccifTag,pstr("ccif_"),Tag );
 }
@@ -389,14 +364,14 @@ int             nCats,i,k,n;
   return k;
 }
 
-pstr  CXMLObject::GetData ( const pstr Tag, int objNo )  {
+pstr  CXMLObject::GetData ( cpstr Tag, int objNo )  {
 PCXMLObject XMLObject;
   XMLObject = GetObject ( Tag,objNo );
   if (XMLObject)  return XMLObject->objData;
   return NULL;
 }
 
-int  CXMLObject::GetData ( pstr & Data, const pstr Tag, int objNo )  {
+int  CXMLObject::GetData ( pstr & Data, cpstr Tag, int objNo )  {
 PCXMLObject XMLObject;
   XMLObject = GetObject ( Tag,objNo );
   if (XMLObject)  {
@@ -408,7 +383,7 @@ PCXMLObject XMLObject;
   }
 }
 
-int  CXMLObject::GetData ( realtype & V, const pstr Tag, int objNo )  {
+int  CXMLObject::GetData ( realtype & V, cpstr Tag, int objNo )  {
 int  rc;
 pstr d,p;
   rc = GetData ( d,Tag,objNo );
@@ -421,7 +396,7 @@ pstr d,p;
   return rc;
 }
 
-int  CXMLObject::GetData ( int & iV, const pstr Tag, int objNo )  {
+int  CXMLObject::GetData ( int & iV, cpstr Tag, int objNo )  {
 int  rc;
 pstr d,p;
   rc = GetData ( d,Tag,objNo );
@@ -434,7 +409,7 @@ pstr d,p;
   return rc;
 }
 
-int  CXMLObject::GetData ( Boolean & bV, const pstr Tag, int objNo )  {
+int  CXMLObject::GetData ( Boolean & bV, cpstr Tag, int objNo )  {
 int  rc;
 pstr d;
   rc = GetData ( d,Tag,objNo );
@@ -450,14 +425,14 @@ pstr d;
   return rc;
 }
 
-PCXMLObject CXMLObject::GetObject ( const pstr Tag, int objNo )  {
+PCXMLObject CXMLObject::GetObject ( cpstr Tag, int objNo )  {
 // allow for "tag1>tag2>tag3>..."
 PCXMLObject XMLObject;
 int         i,j,k,l;
 pstr        p,p1;
   XMLObject = this;
   if (Tag)  {
-    p = Tag;
+    p = pstr(Tag);
     do  {
       p1 = p;
       l  = 0;
@@ -512,7 +487,7 @@ int          i;
 
 }
 
-int  CXMLObject::WriteObject ( const pstr FName, int pos, int indent )  {
+int  CXMLObject::WriteObject ( cpstr FName, int pos, int indent )  {
 CFile f;
   f.assign ( FName,True );
   if (f.rewrite())  {
@@ -607,7 +582,7 @@ Boolean sngline;
 }
 
 
-int  CXMLObject::ReadObject ( const pstr FName )  {
+int  CXMLObject::ReadObject ( cpstr FName )  {
 CFile f;
 char  S[500];
 int   i,rc;
@@ -642,7 +617,8 @@ Boolean     Done;
     k = strlen(S);
     while ((pos<k) && (k1<0))
       if (S[pos]=='<')  {
-        if (S[pos+1]=='?') // in this version, ignore <?xxx ?> constructions
+        if (S[pos+1]=='?') // in this version, ignore <?xxx ?>
+                           //   constructions
              pos++;
         else if (S[pos+1]!='<')
              k1 = pos;
@@ -813,7 +789,7 @@ int         k;
   return XMLObject;
 }
 
-PCXMLObject mmCIF2XML ( const pstr XMLName, PCMMCIFFile mmCIFFile,
+PCXMLObject mmCIF2XML ( cpstr XMLName, PCMMCIFFile mmCIFFile,
                         int * rc )  {
 PCXMLObject XMLObject1,XMLObject2;
 PCMMCIFData mmCIFData;
